/***************************************************************
*             Copyright (C) 2003 by SerComm Corp.
*                    All Rights Reserved.
*
*      Use of this software is restricted to the terms and
*      conditions of SerComm's software license agreement.
*
*                        www.sercomm.com
****************************************************************/
#include <stdio.h>
#include <linux/types.h>
#include <asm/types.h>
#include <unistd.h>
#include <asm/arch/hardware.h>


#include "rtc.h"

#define DEBUG	0
#define EXAMPLE 0  //using the simple example to demostrate the function.
//--------------Some Constant Definition--------------//

#define REG_DIR1	0x00030602
#define REG_BITSET1	0x0003060A
#define REG_BITCLR1	0x0003060E

#define SCL	11		//GIO BIT 27
#define SDA	12		//GIO BIT 28
#define OUTPUT	0
#define INPUT	1
#define LOW	0
#define HIGH	1

#define SLAVE_BYTE_EEPROM_READ 	0xaf
#define SLAVE_BYTE_EEPROM_WRITE 0xae
#define SLAVE_BYTE_CCR_READ 	0xdf
#define SLAVE_BYTE_CCR_WRITE 	0xde



//------------ Macro Declaration ---------------//

#define  inw(p)    (*(volatile unsigned short *)(p))
#define  outw(v,p) (*(volatile unsigned short *)(p) = (v))
#define  delay(k)  { int i=k; while(i)  i--; }

#define SET_DIR1_INPUT  { u16 temp=0x0000; temp=inw(REG_DIR1)|0x1000; outw(temp, REG_DIR1);}
#define SET_DIR1_OUTPUT	{ u16 temp=0x0000; temp=inw(REG_DIR1)&0xefff; outw(temp, REG_DIR1);}
#define SET_SDA_HIGH	outw(0x1000, REG_BITSET1)
#define SET_SDA_LOW	outw(0x1000, REG_BITCLR1)
#define SET_SCL_HIGH	outw(0x0800, REG_BITSET1)
#define SET_SCL_LOW	outw(0x0800, REG_BITCLR1)


// ----------local prototypes --------------- //
static void i2c_start(void);
static void i2c_stop(void);
void write_CCR( u8 address1, u8 address0, u8 data);
void write_byte(u8 address1, u8 address0, u8 data);
static void i2c_transfer(u8 byte);
static void i2c_writebit(u8 val);
u8 read_CCR( u8 address1, u8 address0);
u8 read_byte(u8 address1, u8 address0);
static u8 i2c_receive(void);
u8 i2c_readbit(void);


/**************************************************************
 * A HIGH to LOW transition on the SDA line while SCL is HIGH 
 *  indicates a START condition.
 **************************************************************/
static void i2c_start()
{
	// set SDA to output
	SET_DIR1_OUTPUT;

	// make sure the clock and SDA are HIGH
	SET_SDA_HIGH;
	SET_SCL_HIGH;
	delay(5);

	// pull SDA from high to low  
	SET_SDA_LOW;
	delay(5);         
	SET_SCL_LOW;
}

/**************************************************************
 * A LOW to HIGH transition on the SDA line while SCL is HIGH 
 *  indicates a STOP condition.
 **************************************************************/
static void i2c_stop()
{
	
	SET_SDA_LOW;

        SET_SCL_HIGH;

	delay(5);

	SET_SDA_HIGH;

}

/////////////////////////////////////////////////////////////
//////////////////// Read and Write FUNCTIONS////////////////
/////////////////////////////////////////////////////////////

/**************************************************************
 * Write time to CCR only, if we want to write the time into
 * EEPROM, the slave byte used should be changed, and we have to
 * implement ack polling to obtain the acknowledge. However, 
 * considering that we have a battery in use, I choose to read
 * CCR only to simplify the problem.
 * Return : The required byte
 **************************************************************/
void write_CCR( u8 address1, u8 address0, u8 data)
{


	write_byte(0, 0x3f, 0x02);   // set WEL 

	write_byte(0, 0x3f, 0x06);   // set RWEL

	write_byte(address1, address0, data);
	
	write_byte(0, 0x3f, 0x00);  //disable WEL & RWEL bit

}


/**************************************************************
 * A whole procedure to write a byte into the CLK.
 **************************************************************/
void write_byte(u8 address1, u8 address0, u8 data)
{

	i2c_start();
	
	i2c_transfer(SLAVE_BYTE_CCR_WRITE);
	
	i2c_transfer(address1);
	i2c_transfer(address0);	
	i2c_transfer(data);	

	i2c_stop();
}

/**************************************************************
 * Data is transfered with the most significant bit (MSB) 
 * first.
 **************************************************************/
static void i2c_transfer(u8 byte)
{
        u32 i;
        u8 mask = 0x80;

        for ( i = 0; i < 8; i++ )
        {

                if ( byte & mask )
                        i2c_writebit(1);
                else
                        i2c_writebit(0);
		if( i==7 )
			break;
		else
                mask = mask >> 1;
        }

	// try to obtain an acknowledgement

	{  
	// set the SDA to listen
	SET_DIR1_INPUT;

        // issue clock pulse
	SET_SCL_HIGH;        
	
	delay(5);

	SET_SCL_LOW;

	// set the SDA to write
	SET_DIR1_OUTPUT;
	}
}


/**************************************************************
 * The HIGH or LOW state of the data line can only change when
 * the clock signal on the SCL line is LOW. And if we want to 
 * write data , we have to set DIR1 register to OUTPUT.
 **************************************************************/
static void i2c_writebit(u8 val)
{
        if ( val )
        	{
		SET_DIR1_OUTPUT;	
               	SET_SCL_LOW;
               	SET_SDA_HIGH;        
                }
        else
        	{
		SET_DIR1_OUTPUT;	
               	SET_SCL_LOW;
               	SET_SDA_LOW;
                }
  
        SET_SCL_HIGH;

	delay(5);

	SET_SCL_LOW;  //for further write

}


/**************************************************************
 * Read time from CCR only, if we want to read the time from
 * EEPROM, the slave byte used should be changed. However, 
 * considering that we have a battery in use, I choose to read
 * CCR only to simplify the problem.
 * Return : The required byte
 **************************************************************/
u8 read_CCR( u8 address1, u8 address0)
{
	u8 val;
	
	val=read_byte(address1, address0);

	return val;

}

/**************************************************************
 * Using Random Address Read Sequence to read a byte
 * Return : The required byte
 **************************************************************/

u8 read_byte(u8 address1, u8 address0)
{
	u8 val;

	// Dummy Write
	i2c_start();	
	i2c_transfer(SLAVE_BYTE_CCR_WRITE);
	i2c_transfer(address1);
	i2c_transfer(address0);	

	// Issue a read command
	i2c_start();
	i2c_transfer(SLAVE_BYTE_CCR_READ);	

	val=i2c_receive();

#if DEBUG
	printf(" we read byte : %x \n",val);
#endif
	i2c_stop();

	return val;
}

/**************************************************************
 * Data is received with the most significant bit (MSB) 
 * first.
 * I don't use for loop because I try to reduce the delay time
 * between instructions.
 * Return received value
 **************************************************************/
static u8 i2c_receive(void)
{
        u8 val=0x00;
        u8 temp;

	// set RTC SDA direction to input
        SET_DIR1_INPUT;

#if DEBUG
	printf(" readbit" );
#endif

	temp=i2c_readbit();

#if DEBUG
        printf("%x-", temp);
#endif                
	temp = temp << 7;
	val = (val | temp);

	temp=i2c_readbit();
#if DEBUG
        printf("%x-", temp);
#endif                
	temp = temp << 6;
	val = (val | temp);

	temp=i2c_readbit();
#if DEBUG
                printf("%x-", temp);
#endif                
	temp = temp << 5;
	val = (val | temp);

	temp=i2c_readbit();
#if DEBUG
                printf("%x-", temp);
#endif                
	temp = temp << 4;
	val = (val | temp);
	temp=i2c_readbit();
#if DEBUG
                printf("%x-", temp);
#endif                
	temp = temp << 3;
	val = (val | temp);

	temp=i2c_readbit();
#if DEBUG
                printf("%x-", temp);
#endif                
	temp = temp << 2;
	val = (val | temp);

	temp=i2c_readbit();
#if DEBUG
                printf("%x-", temp);
#endif                
	temp = temp << 1;
	val = (val | temp);

	temp=i2c_readbit();
#if DEBUG
                printf("%x-", temp);
#endif                
	val = (val | temp);

#if DEBUG
printf("\n");
#endif

	// set RTC SDA direction to output
        SET_DIR1_OUTPUT;

        return val;
}


/**************************************************************
 * Reading a single bit from SDA line.
 * Return : 0x0 / 0x1 
 **************************************************************/

u8 i2c_readbit(void)
{
    u8 dat;
    u16 val; //for storing the whole GIO register data
    
    // first issue a clock 
    SET_SCL_HIGH;

    // then read the SDA bit

    // Read GIO register and store it to val 
    val=inw(REG_BITSET1);
    val&=(1<<SDA);

    if ( val ) //in HIGH status
        dat=HIGH;
    else //in LOW status 
	dat=LOW;
   
    // finally , pull down the clock 
    SET_SCL_LOW;

    return dat;
}



/////////////////////////////////////////////////////////////
//////////////////// EXPORT FUNCTIONS////////////////////////
/////////////////////////////////////////////////////////////

/**************************************************************
 * Usage: Declare a 16 element array to store the time data 
 *	  which we are going to set. In the format of:
 *	  (Y2K/Week/YR/MONTH/DAY:HR:MN:SC)
 * 	  NOTICE: All the time is store in the BCD format. 	  
 *  	  See the simple example below.
 * Return : None 
 **************************************************************/
void set_RTC_time(u8 *time_array)
{
	u8 ten_digit, unit_digit, val;
	int i=0;
	
	for ( i=0; i < 8 ; i++)  // Y2K DW YR MO DT HR MN SC
	{ 
	
	ten_digit = time_array[2*i];
	
	// set MIL =1  ( in HR register , 24 HR) 
	if (i==5)
		ten_digit = ten_digit|(0x8);
#if DEBUG
	printf("10s:%x ", ten_digit);
#endif
	unit_digit = time_array[2*i+1];
#if DEBUG
	printf("1s:%x ", unit_digit);	
#endif
	val = ( ten_digit << 4 ) | (unit_digit);

#if DEBUG			
	printf("write:%2x\n", val);
#endif
	//write a byte to the corresponding register 	
	write_CCR(0, 0x37-i , val);

	}
	
}

/**************************************************************
 * Usage: Declare a 16 element array for the function to  store 
 *	  the time data. In the format of:
 *	  (Y2K/Week/YR/MONTH/DAY:HR:MN:SC)
 * 	  NOTICE: All the time is store in the BCD format.
 *  	  See the simple example below.
 * Return : None 
 **************************************************************/
void read_RTC_time(u8 *time_array)
{
	u8 ten_digit, unit_digit, val;
	int i=0;

	
	for ( i=0; i < 8; i++)  // Y2K DW YR MO DT HR MN SC
	{ 
	//read a byte to the corresponding register
	val=read_CCR(0x00, 0x37-i);
#if DEBUG
	printf("read : %2x \n", val);
#endif 
	if (i==5)
	ten_digit = val&(0x70);  //for HR , remove MIL to reply with correct HR
	else
	ten_digit = val&(0xf0);
	
	unit_digit = val&(0x0f);

	time_array[i*2]= ( ten_digit >> 4 );
	time_array[i*2+1]= unit_digit;
	}
	
}
