/* ============================================================================
 * Copyright (C) 1998-2001 Angus Mackay. All rights reserved; 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * ============================================================================
 */

/*
 * ez-ipupdate
 *
 * a very simple dynDNS client for the ez-ip dynamic dns service 
 * (http://www.ez-ip.net).
 *
 * why this program when something like:
 *   curl -u user:pass http://www.ez-ip.net/members/update/?key=val&...
 * would do the trick? because there are nicer clients for other OSes and
 * I don't like to see UNIX get the short end of the stick.
 *
 * tested under Linux and Solaris.
 * 
 */
 
/*
 *  1.Filter un-necessary code out
 *  2.Directly read configuration from system.conf
 *  3.Add check external ip from http://checkip.dyndns.org
 *  Sand Tu<sand_tu@sercomm.com>
 *  2002/09/23 16:35 
 */

#include "../../cgi/inc/SYS_log.h"
#include "common.h"

//#define DEBUG 0
#undef DEBUG

#ifdef X86
   #define AddLog(x,y)
#endif

#define SYSLOG_SECTION	"ez-ipupdate"

#define VERSION "3.0.11b7"
#define OS "i686-pc-linux-gnu"
#define DEF_SERVICE "dyndns"

#define DYNDNS_DEFAULT_SERVER "members.dyndns.org"
#define DYNDNS_DEFAULT_PORT "80"
#define DYNDNS_REQUEST "/nic/update"
#define DYNDNS_MAX_INTERVAL (25*24*3600)

/*----------------------------------------------------------------*/
/*    according TZO Update Protocol for Linksys, it provides 4    */
/*    servers (clusterlookup1.tzo.com, clusterlookup2.tzo.com,    */
/*    clusterlookup3.tzo.com, clusterlookup4.tzo.com) , we choose */
/*    clusterlookup1.tzo.com for default server			  */
/*----------------------------------------------------------------*/
#define TZO_DEFAULT_SERVER "clusterlookup1.tzo.com"
#define TZO_STEP1_PORT "21340"
#define TZO_STEP2_PORT "21347"
//#define TZO_REQUEST "/webclient/signedon.html"
#define DYNDNS_NAME	"www.dyndns.org"
#define TZO_NAME	"linksys.tzo.com"



#define DEFAULT_TIMEOUT 120

#define BUFFER_SIZE (1*1024-1) //reduce buffer size to about 1k


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <asm/sockios.h>


#define  snprintf(x, y, z...) sprintf(x, ## z)
#define ARRAY_LEN(x) (sizeof(x)/sizeof(x[0]))


// the max time we will wait if the server tells us to
#define MAX_WAITRESPONSE_WAIT (24*3600)

/**************************************************/

struct service_t
{
  char *title;
  char *names[3];
  int (*update_entry)(void);
  char *default_server;
  char *default_port;
  char *default_request;
};

enum {
  UPDATERES_OK = 0,
  UPDATERES_ERROR,
  UPDATERES_SHUTDOWN,
};

/**************************************************/
char *ip_address = NULL;
char *server = NULL;
char *port = NULL;
char user[256];
char auth[512];
char user_name[128];
char password[128];
char address[20];
int date;  // to record the last update time and date
char *request = NULL;
int wildcard = 0;
char host[40];
struct timeval timeout;
//char buf[BUFFER_SIZE+1];
static volatile int client_sockfd;

/* service objects for  sololy dyndns services */

int DYNDNS_update_entry(void);
int TZO_update_entry(void);

//struct service_t service = 
struct service_t service1 = 
  { "dyndns",
    { "dyndns", 0, 0, },
    DYNDNS_update_entry,
    DYNDNS_DEFAULT_SERVER,
    DYNDNS_DEFAULT_PORT,
    DYNDNS_REQUEST
  };
struct service_t service2 = 
  { "tzo",
    { "tzo", 0, 0, },
    TZO_update_entry,
    TZO_DEFAULT_SERVER,
    TZO_STEP1_PORT,
    NULL 
  };

static struct service_t *service = NULL;

/**************************************************/
int do_connect(int *sock, char *host, char *port);
void base64Encode(char *intext, char *output);
int main(void);
/**************************************************/

/*
 * do_connect
 *
 * connect a socket and return the file descriptor
 *
 */
int do_connect(int *sock, char *host, char *port)
{
  struct sockaddr_in address;
  int len;
  int result;
  struct hostent *hostinfo;
  struct servent *servinfo;
  
  // set up the socket
  if((*sock=socket(AF_INET, SOCK_STREAM, 0)) == -1)
  {
    return(-1);
  }
  address.sin_family = AF_INET;

  // get the host address
  hostinfo = gethostbyname(host);
  if(!hostinfo)
  {
    close(*sock);
    return(-1);
  }
  address.sin_addr = *(struct in_addr *)*hostinfo -> h_addr_list;

  // get the host port
  servinfo = getservbyname(port, "tcp");
  if(servinfo)
  {
    address.sin_port = servinfo -> s_port;
  }
  else
  {
    address.sin_port = htons(atoi(port));
  }

  // connect the socket
  len = sizeof(address);
  if((result=connect(*sock, (struct sockaddr *)&address, len)) == -1) 
  {
    close(*sock);
    return(-1);
  }

  return 0;
}

static char table64[]=
  "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
void base64Encode(char *intext, char *output)
{
  unsigned char ibuf[3];
  unsigned char obuf[4];
  int i;
  int inputparts;

  while(*intext) {
    for (i = inputparts = 0; i < 3; i++) { 
      if(*intext) {
        inputparts++;
        ibuf[i] = *intext;
        intext++;
      }
      else
        ibuf[i] = 0;
    }

    obuf [0] = (ibuf [0] & 0xFC) >> 2;
    obuf [1] = ((ibuf [0] & 0x03) << 4) | ((ibuf [1] & 0xF0) >> 4);
    obuf [2] = ((ibuf [1] & 0x0F) << 2) | ((ibuf [2] & 0xC0) >> 6);
    obuf [3] = ibuf [2] & 0x3F;

    switch(inputparts) {
      case 1: /* only one byte read */
        sprintf(output, "%c%c==", 
            table64[obuf[0]],
            table64[obuf[1]]);
        break;
      case 2: /* two bytes read */
        sprintf(output, "%c%c%c=", 
            table64[obuf[0]],
            table64[obuf[1]],
            table64[obuf[2]]);
        break;
      default:
        sprintf(output, "%c%c%c%c", 
            table64[obuf[0]],
            table64[obuf[1]],
            table64[obuf[2]],
            table64[obuf[3]] );
        break;
    }
    output += 4;
  }
  *output=0;
}


void output(void *buf)
{
  fd_set writefds;
  int max_fd;
  struct timeval tv;
  int ret;

#if DEBUG
  printf("I say: %s\n", (char *)buf);
#endif
  // set up our fdset and timeout
  FD_ZERO(&writefds);
  FD_SET(client_sockfd, &writefds);
  max_fd = client_sockfd;
  memcpy(&tv, &timeout, sizeof(struct timeval));

  ret = select(max_fd + 1, NULL, &writefds, NULL, &tv);

  if(ret == -1)
  {
#if DEBUG  
    printf("select:\n");    
#endif   
  }
  else if(ret == 0)
  {
#if DEBUG  
    printf("timeout\n");
#endif    
  }
  else
  {
    /* if we woke up on client_sockfd do the data passing */
    if(FD_ISSET(client_sockfd, &writefds))
    {
      if(send(client_sockfd, buf, strlen(buf), 0) == -1)
      {
#if DEBUG      
        printf("error send()ing request");        
#endif        
      }
    }
    else
    {
#if DEBUG
      printf("error: case not handled.");
#endif      
    }
  }
}

int read_input(char *buf, int len)
{
  fd_set readfds;
  int max_fd;
  struct timeval tv;
  int ret;
  int bread = -1;
  
  // set up our fdset and timeout
  FD_ZERO(&readfds);
  FD_SET(client_sockfd, &readfds);
  max_fd = client_sockfd;
  memcpy(&tv, &timeout, sizeof(struct timeval));

  ret = select(max_fd + 1, &readfds, NULL, NULL, &tv);

#if DEBUG  
  printf("ret: %d\n", ret);
#endif  

  if(ret == -1)
  {
#if DEBUG
    printf("select:\n");
#endif    
  }
  else if(ret == 0)
  {
#if DEBUG
    printf("timeout\n");    
#endif    
  }
  else
  {
    /* if we woke up on client_sockfd do the data passing */
    if(FD_ISSET(client_sockfd, &readfds))
    {
      bread = recv(client_sockfd, buf, len-1, 0);
      buf[bread] = '\0';
#if DEBUG
      printf("got: %s\n", buf);
#endif
      if(bread == -1)
      {
#if DEBUG
        printf("error recv()ing reply");
#endif        
      }
    }
    else
    {
#if DEBUG    
      printf("error: case not handled.");
#endif      
    }
  }

  return(bread);
}



char *DYNDNS_check_ip(void)
{
  char buf[BUFFER_SIZE+1];
  char *bp = buf;
  int bytes;
  int btot;
  char *server1="checkip.dyndns.org";
  char *port1="80";

	// buffer initialize 
  buf[BUFFER_SIZE] = '\0';

	// here connect to checkip.dyndns.org:80
  do_connect((int*)&client_sockfd, server1, port1);

  snprintf(buf, BUFFER_SIZE, "GET / HTTP/1.0\015\012");
  output(buf);

  snprintf(buf, BUFFER_SIZE, "Accept : */*\015\012");
  output(buf);

  snprintf(buf, BUFFER_SIZE, "User-Agent: %s %s [%s] (%s)\015\012", 
      "ez-update", OS, "daemon" , "by Angus Mackay");
  output(buf);

  snprintf(buf, BUFFER_SIZE, "Host: %s\015\012", server1);
  output(buf);
  snprintf(buf, BUFFER_SIZE, "\015\012");
  output(buf);

  bp = buf;
  bytes = 0;
  btot = 0;
  while((bytes=read_input(bp, BUFFER_SIZE-btot)) > 0)
  {
    bp += bytes;
    btot += bytes;
  }
  close(client_sockfd);
  buf[btot] = '\0';

#if DEBUG
  printf("server output: %s\n", buf);
#endif

	// grep the outer ip address from buf
  ip_address=strstr(buf,"Current IP Address:");
  strtok(ip_address,":\n");
  ip_address=strtok(NULL,":\n");

	// remove the space in front of the string
  while(*ip_address==' ')
	ip_address++;

  return(ip_address);
}

int DYNDNS_update_entry(void)
{
  char buf[BUFFER_SIZE+1];
  char *bp = buf;
  int bytes;
  int btot;
  int ret;
  int retval = UPDATERES_OK;
  buf[BUFFER_SIZE] = '\0';

  if(do_connect((int*)&client_sockfd, server, port) != 0)
  {
    AddLog(SYSLOG_SECTION, SYSLOG_ERROR, 1);
    return(UPDATERES_ERROR);
  }
  snprintf(buf, BUFFER_SIZE, "GET %s?", request);
  output(buf);
  snprintf(buf, BUFFER_SIZE, "%s=%s&", "hostname", host);
  output(buf);
  snprintf(buf, BUFFER_SIZE, "%s=%s&", "myip", ip_address);
  output(buf);
  snprintf(buf, BUFFER_SIZE, "%s=%s&", "wildcard", wildcard ? "ON" : "OFF");
  output(buf);
  snprintf(buf, BUFFER_SIZE, " HTTP/1.0\015\012");
  output(buf);
  snprintf(buf, BUFFER_SIZE, "Authorization: Basic %s\015\012", auth);
  output(buf);
  snprintf(buf, BUFFER_SIZE, "User-Agent: %s- %s [%s] (%s)\015\012", 
      "ez-update", OS, "daemon" , "by Angus Mackay");
  output(buf);
  snprintf(buf, BUFFER_SIZE, "Host: %s\015\012", server);
  output(buf);
  snprintf(buf, BUFFER_SIZE, "\015\012");
  output(buf);
  bp = buf;
  bytes = 0;
  btot = 0;
  while((bytes=read_input(bp, BUFFER_SIZE-btot)) > 0)
  {
    bp += bytes;
    btot += bytes;
#ifdef DEBUG    
    printf("btot: %d\n", btot);
#endif    
  }
  close(client_sockfd);
  buf[btot] = '\0';

#ifdef DEBUG
  printf("server output: %s\n", buf);
#endif
  if(sscanf(buf, " HTTP/1.%*c %3d", &ret) != 1)
  {
    ret = -1;
  }

  switch(ret)
  {
    case -1:
      retval = UPDATERES_ERROR;
      break;

    case 200:
      if(strstr(buf, "\ngood ") != NULL || strstr(buf, "\nnochg") != NULL)
      {
      }
      else
      {
        if(strstr(buf, "\nnohost") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\nnotfqdn") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\n!yours") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\nabuse") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\nbadauth") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\nbadsys") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\nbadagent") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\nnumhost") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\ndnserr") != NULL)
        {
          retval = UPDATERES_ERROR;
        }
        else if(strstr(buf, "\n911") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\n999") != NULL)
        {
          retval = UPDATERES_SHUTDOWN;
        }
        else if(strstr(buf, "\n!donator") != NULL)
        {
          retval = UPDATERES_OK;
        }
        // this one should be last as it is a stupid string to signify waits
        // with as it is so short
        else if(strstr(buf, "\nw") != NULL)
        {
          int howlong = 0;
          char *p = strstr(buf, "\nw");
          char reason[256];
          char mult;

          // get time and reason
          if(strlen(p) >= 2)
          {
            sscanf(p, "%d%c %255[^\r\n]", &howlong, &mult, reason);
            if(mult == 'h')
            {
              howlong *= 3600;
            }
            else if(mult == 'm')
            {
              howlong *= 60;
            }
            if(howlong > MAX_WAITRESPONSE_WAIT)
            {
              howlong = MAX_WAITRESPONSE_WAIT;
            };
          }
          else
          {
            sprintf(reason, "problem parsing reason for wait response");
          }

          sleep(howlong);
          retval = UPDATERES_ERROR;
        }
        else
        {
          retval = UPDATERES_ERROR;
        }
      }
      break;

    case 401:
      retval = UPDATERES_SHUTDOWN;
      break;

    default:
      retval = UPDATERES_ERROR;
      break;
  }
  
  if ( retval == UPDATERES_OK)
    	AddLog(SYSLOG_SECTION, SYSLOG_NORMAL, 2);
  else if (retval == UPDATERES_ERROR)
	AddLog(SYSLOG_SECTION, SYSLOG_ERROR, 3);
  else if (retval == UPDATERES_SHUTDOWN)
    	AddLog(SYSLOG_SECTION, SYSLOG_ERROR, 4);

  return(retval);
}

static char * select_ip(char *ptr)	//?? now we choose first ip of 4 ip sets
{	
	char tmp[128];
	char *ipp;

	strcpy(tmp, ptr);
	strtok(tmp, ",");
	ipp=strtok(NULL, ",");

	return ipp;
}
static int parse_server_ip(char *str, char *ip)
{
	char buf[128];
	char *pt1,*pt2;

	strcpy(buf, str);		//str -> "XXX\r\nYYY\r\n"

	strtok(buf, "\r\n");		
	pt1=strtok(NULL, "\r\n");	//ip -> "YYY"
	if( *pt1 == '0' )	
		return -1;		//"0\r\n" -> invalid key

	pt2=select_ip(pt1);
	strcpy(ip,pt2);

	return 0;
}
	

static int parse_resp(char *str)
{
	char buf[128];
	char *p;
	
	strcpy(buf, str);		//str -> "XXX\r\nYYY\r\n"

	strtok(buf, "\r\n");		
	p=strtok(NULL, "\r\n");		//ip -> "YYY"
	if(strcmp(p, "40 OK"))
		return -1;
	return 0;
}


int TZO_update_entry(void)
{
  char buf[BUFFER_SIZE+1];
  char *bp = buf;
  int bytes;
  int btot;
  int retval;
  char step2_ip[20];

  buf[BUFFER_SIZE] = '\0';

  if(do_connect((int*)&client_sockfd, server, port) != 0)
  {
    AddLog(SYSLOG_SECTION, SYSLOG_ERROR, 1);
    return(UPDATERES_ERROR);
  }

  snprintf(buf, BUFFER_SIZE, "I %s\r\n", password);
  output(buf);

  bp = buf;
  bytes = 0;
  btot = 0;
  while((bytes=read_input(bp, BUFFER_SIZE-btot)) > 0)
  {
    bp += bytes;
    btot += bytes;
  }
  close(client_sockfd);
  buf[btot] = '\0';

#ifdef DEBUG
  printf("server output: %s\n", buf);
#endif

  //parse 1 ip:
  if( parse_server_ip(buf,step2_ip) ){
    retval=UPDATERES_ERROR;
    goto err;
  }
#ifdef DEBUG
  printf("ip output: %s\n", step2_ip);
#endif

  //create 2nd session ,port=21347
  if(do_connect((int*)&client_sockfd, step2_ip, TZO_STEP2_PORT) != 0)
  {
    AddLog(SYSLOG_SECTION, SYSLOG_ERROR, 1);
    return(UPDATERES_ERROR);
  }

  //register
  snprintf(buf, BUFFER_SIZE, "R %s,%s,%s,%s\r\n", host, user_name, password,ip_address);
  output(buf);

  //read server response
  bp = buf;
  bytes = 0;
  btot = 0;
  while((bytes=read_input(bp, BUFFER_SIZE-btot)) > 0)
  {
    bp += bytes;
    btot += bytes;
  }
  close(client_sockfd);
  buf[btot] = '\0';

#ifdef DEBUG
  printf("server output: %s\n", buf);
#endif

  //parse server response , ex:"40 OK"
  if( parse_resp(buf) )
    retval=UPDATERES_ERROR;

  retval=UPDATERES_OK;

err:
  if ( retval == UPDATERES_OK)
    	AddLog(SYSLOG_SECTION, SYSLOG_NORMAL, 2);
  else if (retval == UPDATERES_ERROR)
	AddLog(SYSLOG_SECTION, SYSLOG_ERROR, 3);
  else if (retval == UPDATERES_SHUTDOWN)
    	AddLog(SYSLOG_SECTION, SYSLOG_ERROR, 4);

  return(retval);
}

// server name -> service pointer
void parse_service(char *str)
{
	if(!strcmp(str, DYNDNS_NAME))
		service=&service1;
	else if(!strcmp(str, TZO_NAME))
		service=&service2;
	else
		service=NULL;
	
	return;
}

int main(void)
{
  int retval = 1;
  FILE *fd;
  time_t now;
  int need_update = 0;

    char srv[GENERIC_STRING_LEN];
    char ip[20];

    server=&srv[0];
    ip_address=&ip[0];

// some variables initializations
  *user = '\0';
  *user_name = '\0';
  *password = '\0';  

  timeout.tv_sec = DEFAULT_TIMEOUT;
  timeout.tv_usec = 0;  
// originally we will parse services to find out which service we are going to
// use, now we directly set default to dyndns, skip the parsing procedure.
//  service = &(services[0]);

 // input need to be process here , all the necessary information will 
 // either obtain from system.conf or by DYNDNS' default.

	// retrieve from system.conf
  fd =  fopen(DDNS_SYSTEM_CONF,"rt");
  if(fd==NULL)
	return DDNS_ERROR;  
	// host, address, user_name, and password are read from system.conf
  
  if(PRO_GetStr(DDNS_SECTION, DDNS_SERVER , server, GENERIC_STRING_LEN, fd) != DDNS_OK){
	fclose(fd);
	return DDNS_ERROR;
  }

  if(PRO_GetStr(DDNS_SECTION, DDNS_ACCOUNT , user_name, DDNS_ACCOUNT_LEN, fd) != DDNS_OK){
	fclose(fd);
	return DDNS_ERROR;
  }
  if(PRO_GetStr(DDNS_SECTION, DDNS_PASSWORD , password, GENERIC_STRING_LEN, fd) != DDNS_OK){
	fclose(fd);
	return DDNS_ERROR;
  }
  if(PRO_GetStr(DDNS_SECTION, DDNS_HOSTNAME , host, DDNS_MAX_HOSTNAME_LEN, fd) != DDNS_OK){
	fclose(fd);
	return DDNS_ERROR;
  }
  if(PRO_GetStr(DDNS_SECTION, DDNS_IPADDR , address, GENERIC_STRING_LEN, fd) != DDNS_OK){
	fclose(fd);
	return DDNS_ERROR;
  }
  if(PRO_GetInt(DDNS_SECTION, DDNS_DATE , &date, fd) != DDNS_OK){
	fclose(fd);
	return DDNS_ERROR;
  }
  fclose(fd);
  parse_service(server);

  	// DYNDNS's default , currently we only support DYNDNS, may be changed
  	// to read from system.conf if we support multi-DNS
  server = strdup(service->default_server);
  port = strdup(service->default_port);


#ifdef DEBUG
  printf("server: %s, port: %s\n",server,port);
#endif






	// obtain external ip address
  DYNDNS_check_ip();
#if DEBUG
  printf("returned IP %s\n",ip_address);
#endif
  time(&now);

 	// if time now is 25 days later of the last update time. 
  if( now > date+(DYNDNS_MAX_INTERVAL) )
  	need_update = 1;

 	// if ip changed 

    //Here we check the system.conf to read the outer_ipadd
    //and compare with the ip we obtain from DYNDNS_check_ip().
    //If the same, then we don't have to update the DNS.
    //If different, then update it.
    if(strcmp(ip_address, &address[0]) != 0)
  	need_update = 1;

	// process auth word and request command  
  sprintf(user, "%s:%s", user_name, password);
  base64Encode(user, auth);  // to generate the authentication word
	// request == GET /nic/update
  request = strdup(service->default_request);

  if(need_update==0)
  {
	AddLog(SYSLOG_SECTION, SYSLOG_NORMAL, 5);
	retval= 0;
  }
  else if(need_update ==1)
    {
       if(service->update_entry() == UPDATERES_OK)
          retval = 0;

      // Here we change write cache file into updating system.conf
      // the column "outer_ipadd" will be modified to record the 
      // current ip address of gateway device.

      strcpy(&address[0],ip_address);
      
      if(PRO_SetStr(DDNS_SECTION, DDNS_IPADDR, address ,DDNS_SYSTEM_CONF) != DDNS_OK)
		return DDNS_ERROR;      
      
      if(PRO_SetInt(DDNS_SECTION, DDNS_DATE, now ,DDNS_SYSTEM_CONF) != DDNS_OK)
		return DDNS_ERROR;      	
#if DEBUG
      printf(" out need_update=1 \n");
#endif
    }

#ifndef X86
  if(ip_address) { free(ip_address); }
  if(address) { free(address); }
  if(host) { free(host); }
  if(port) { free(port); }
  if(request) { free(request); }
  if(server) { free(server); }

  if(service) { free(service); }
#endif
  return(retval);
}

