/***************************************************************
*             Copyright (C) 2003 by SerComm Corp.
*                    All Rights Reserved.
*
*      Use of this software is restricted to the terms and
*      conditions of SerComm's software license agreement.
*
*                        www.sercomm.com
****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>
#include <string.h>
#include <unistd.h>

#ifdef CATCH_SIGINT
#include <signal.h>
#endif
#include "smtpc.h"
#include "smtpc_client.h"
#include "smtpc_mime.h"

#define LISTENQ	1024				//listen queue
#define	MAX_BUF_SIZE	sizeof(CMD_CONF)+1		

#define CMD_SET_CONFIG		1
#define CMD_GET_CONFIG		2
#define CMD_GET_STATUS		3
#define CMD_SEND_MAIL		4
#define CMD_SEND_ABORT		5
#define CMD_ENABLE_SMTPC	6

/* if allow multi-connection, the following 
 * variables must in every session		*/
BYTE	buf[MAX_BUF_SIZE];		//reseive buf

CMD_HEAD	cmd_head;		//receive command head
CMD_CONF	cmd;			//receive command conf
CMD_STATUS	get_stc;
CMD_SENDMAIL	s_mail;		
CMD_ENSMTPC	en_smtpc;

static BYTE cmd_type;		//buf[3]
static BYTE cmd_version;	//buf[0]



extern	FILE	*sfp;			//mail buffer

int	count,buf_count;		
int	conf_flag;			//1:config set ; 0:not yet
int	m_flag;				//1:terminate sending mail ; 0:not 

#ifdef UCLIBC_RR
#define M_NOSIGNAL	0		/* no MSG_NOSIGNAL in uClibc */
#else
#define M_NOSIGNAL	MSG_NOSIGNAL
#endif

#ifdef _MY_DEBUG_
#define DBG(x)	x
char logmsg[128];
void ClrLog(void)
{
	FILE *fh;
	fh=fopen("/tmp/smtp.log","w");
	if(fh)
		fclose(fh);
}
void LogMsg(char *s)
{
	FILE *fh;
	fh=fopen("/tmp/smtp.log","a+");
	if(fh){
		fputs(s,fh);
		fclose(fh);
 	}
}
#else
#define DBG(x)
#endif

void inet_itow(int i,BYTE w[2])
{
     w[0] = (BYTE)((i&0xff00)>>8);
     w[1] = (BYTE)(i&0x0ff);
}

int inet_wtoi(BYTE w[2])
{
     int i=0;

     i+=w[1];
     i+=w[0]<<8;  
     return(i);
}
int inet_dwtoi(BYTE w[4])
{
	int i=0;
	
	i += w[3];
	i += w[2]<<8;
	i += w[1]<<16;
	i += w[0]<<24;
	return(i);
}

static int StartSMTPC()
{
//initialize memory
	memset(&buf[0],0,sizeof(buf));	
	memset(&cmd_head,0,sizeof(cmd_head));	
	memset(&cmd,0,sizeof(cmd));	
	memset(&get_stc,0,sizeof(get_stc));	
	memset(&s_mail,0,sizeof(s_mail));	
	memset(&en_smtpc,0,sizeof(en_smtpc));	
//initialize global variables
	en_smtpc.enable=0;
	conf_flag=0;
	m_flag=0;

	return 0;	
}

int set_resp(int error,CMD_HEAD *pcmd1,CMD_CONF *pcmd2,CMD_STATUS *pcmd3)
{
	CMD_CONF	*conf=NULL;	

	if(cmd_type == 2){
		if(!en_smtpc.enable){
			memset(conf, 0, sizeof(CMD_CONF));
			inet_itow(error,conf->head.result);
			inet_itow(sizeof(CMD_CONF)-sizeof(CMD_HEAD),conf->head.len);
			memcpy(buf, conf, sizeof(CMD_CONF));
			free(conf);
		}
		else{
			inet_itow(error,pcmd2->head.result);
			inet_itow(sizeof(CMD_CONF)-sizeof(CMD_HEAD),pcmd2->head.len);
			memcpy(buf, pcmd2, sizeof(CMD_CONF));
		}
	}
	else if(cmd_type == 3){
		inet_itow(error,pcmd3->head.result);
		inet_itow(sizeof(CMD_STATUS)-sizeof(CMD_HEAD),pcmd3->head.len);
		memcpy(buf, pcmd3, sizeof(CMD_STATUS));
	}
	else{
		inet_itow(error,pcmd1->result);
		inet_itow(0,pcmd1->len);	
		memcpy(buf, pcmd1, sizeof(CMD_HEAD));
	}
	return 0;	
}

int set_resp1(int error)	//only for SendMailPkt()
{
	inet_itow(error,s_mail.head.result);
	inet_itow(0,s_mail.head.len);	
	memcpy(buf, &s_mail.head, sizeof(CMD_HEAD));

	return 0;	
}


static int send_resp(void *arg)
{
	int ccode;
	int respfd;

	respfd=(int)arg;
	
	if((!en_smtpc.enable) && (cmd_type != CMD_GET_STATUS)){
		ccode = send(respfd, buf, sizeof(buf), M_NOSIGNAL);
		if(ccode == -1)
			DBG( LogMsg("3 can't send data to remote!\n"); )
		return ccode;
	}	
	
	if(cmd_type == CMD_GET_CONFIG){
		ccode = send(respfd, buf, sizeof(CMD_CONF), M_NOSIGNAL);
		if(ccode != sizeof(CMD_CONF) || ccode == -1){
			DBG( LogMsg("4 can't send data to remote normally\n"); )
			return ccode;
		}
	}		
	else if(cmd_type == CMD_GET_STATUS){
		ccode = send(respfd, buf, sizeof(CMD_STATUS), M_NOSIGNAL);
		if(ccode != sizeof(CMD_STATUS) || ccode == -1){
			DBG( LogMsg("5 can't send data to remote normally\n"); )
			return ccode;
		}
	}		
	else{
		ccode = send(respfd, buf, sizeof(CMD_HEAD), M_NOSIGNAL);
		if(ccode != sizeof(CMD_HEAD) || ccode == -1){
			DBG( LogMsg("6 can't send data to remote normally\n"); )
			return ccode;
		}
	}		
	return 0;
}

int SetConfigPkt(void)		//output: CMD_HEAD + CMD_CONF
{
	int i_len;
	DBG( LogMsg("set configuration now...\n"); )

	memcpy(&cmd, buf, sizeof(CMD_CONF));
	
	if(!en_smtpc.enable){
	 	DBG( LogMsg("Enable SMTPC first before set configuration!\n"); )
		return(set_resp(NO_EN_SMTPC,&(cmd.head),NULL,NULL));
	}
	i_len = inet_wtoi(cmd.head.len);
	if(i_len != (sizeof(CMD_CONF)-sizeof(CMD_HEAD)))
		return(set_resp(INVALID_DATA,&(cmd.head),NULL,NULL));

	conf_flag = 1;
	return(set_resp(OK,&(cmd.head),NULL,NULL));
}	

int GetConfigPkt(void)
{
	int i_len;

	DBG( LogMsg("get configuration now...\n"); )
	
	memcpy(&cmd_head, buf, sizeof(CMD_HEAD));
	
	if(!en_smtpc.enable){
		DBG( LogMsg("Enable SMTPC first before get configuration!\n"); )
		return(set_resp(NO_EN_SMTPC,NULL,&(cmd),NULL));
	}
	i_len = inet_wtoi(cmd_head.len);
	if(i_len)
		return(set_resp(INVALID_DATA,NULL,&(cmd),NULL));	
	
	return(set_resp(OK,NULL,&(cmd),NULL));
}

int GetStatusPkt(void)
{
	int i_len;

	DBG( LogMsg("get SMTPC status now...\n"); )

	memcpy(&cmd_head, buf, sizeof(CMD_HEAD));	

	i_len = inet_wtoi(cmd_head.len);
	if(i_len)
		return(set_resp(INVALID_DATA,NULL,NULL,&(get_stc)));	
	
	if(en_smtpc.enable)
		get_stc.stat = STC_ENABLED;

	return(set_resp(OK,NULL,NULL,&(get_stc)));
}

int SendMailPkt(void)
{
	int i_len,ccode;
 unsigned int	shm_len;
 int		shm_id;
 char		*shm_ptr;

//	DBG( LogMsg("send mail now...\n"); )

	memcpy(&s_mail, buf, sizeof(CMD_SENDMAIL));
	if(!en_smtpc.enable){
		DBG( LogMsg("Enable SMTPC first before send mail!\n"); )
		return(set_resp1(NO_EN_SMTPC));
	}
	if(!conf_flag){
		DBG( LogMsg("Set configuration first before send mail!\n"); )
		return(set_resp1(SAVE_ERR));
	}
	
	i_len = inet_wtoi(s_mail.head.len);
	if(i_len != (sizeof(CMD_SENDMAIL)-sizeof(CMD_HEAD)))
		return(set_resp1(INVALID_DATA));

	shm_len = inet_dwtoi(s_mail.size);
	shm_id = inet_dwtoi(s_mail.buf_id);

	shm_ptr = (char *)shm_id;

//	DBG( LogMsg("Enter Connect2SMTP() \n"); )
    	if(!count){
		ccode = Connect2SMTP(&cmd);	//return 0:ok 1:error

		DBG( sprintf(logmsg, "Connect2SMTP() ret=%d\n",ccode); )
		DBG( LogMsg(logmsg); )

		if(ccode)
			return(set_resp1(SERVER_NO_RSP));
	}	
//sending mail data--------------------------------------------------
   if(s_mail.fno==1){		//only one file
	if(!buf_count){			
            if(s_mail.last){		//first&last buf
		Tx_OneFile(s_mail.format,shm_ptr,shm_len,1,1,sfp,s_mail.ifn);
		EndSMTP();
		memset(&cmd,0,sizeof(cmd));
	    	return(set_resp1(OK));
	    }	
	    else{			//first buf
		Tx_OneFile(s_mail.format,shm_ptr,shm_len,1,0,sfp,s_mail.ifn);
	   }
	}
	else{ 
	    if(s_mail.last){		//last buf
		Tx_OneFile(s_mail.format,shm_ptr,shm_len,0,1,sfp,s_mail.ifn);
		EndSMTP();
		memset(&cmd,0,sizeof(cmd));
	    	return(set_resp1(OK));
	    }
	    else			//other buf		 
		Tx_OneFile(s_mail.format,shm_ptr,shm_len,0,0,sfp,s_mail.ifn);
	}
   }
   else{			//over 1 file
	if(s_mail.index==0){		    //first file
	    if(!buf_count){		 
		if(s_mail.last)		//firstst&last buf
		    Tx_MultiFile(s_mail.format,shm_ptr,1,0,shm_len,1,1,sfp,s_mail.ifn);
		else			//first buf
		    Tx_MultiFile(s_mail.format,shm_ptr,1,0,shm_len,1,0,sfp,s_mail.ifn);
	    }
	    else{ 
		if(s_mail.last)		//last buf	
		    Tx_MultiFile(s_mail.format,shm_ptr,1,0,shm_len,0,1,sfp,s_mail.ifn);
		else			//othsr buf
		    Tx_MultiFile(s_mail.format,shm_ptr,1,0,shm_len,0,0,sfp,s_mail.ifn);
	    }
	}
				    //last file
	else if(s_mail.index == (s_mail.fno - 1)){	
	    if(!buf_count){
		if(s_mail.last){	//first&last buf
		    Tx_MultiFile(s_mail.format,shm_ptr,0,1,shm_len,1,1,sfp,s_mail.ifn);
		    EndSMTP();
		    memset(&cmd,0,sizeof(cmd));
	    	    return(set_resp1(OK));
		}
		else			//first buf
		    Tx_MultiFile(s_mail.format,shm_ptr,0,1,shm_len,1,0,sfp,s_mail.ifn);
	    }	
	    else{ 
		if(s_mail.last){	//last buf	
		    Tx_MultiFile(s_mail.format,shm_ptr,0,1,shm_len,0,1,sfp,s_mail.ifn);
		    EndSMTP();
		    memset(&cmd,0,sizeof(cmd));
	    	    return(set_resp1(OK));
		}
		else			//othsr buf
		    Tx_MultiFile(s_mail.format,shm_ptr,0,1,shm_len,0,0,sfp,s_mail.ifn);
	    }
	}
        else{			    //othsr file
	    if(!buf_count){
		if(s_mail.last)		//first&last buf
		    Tx_MultiFile(s_mail.format,shm_ptr,0,0,shm_len,1,1,sfp,s_mail.ifn);
		else			//first buf
		    Tx_MultiFile(s_mail.format,shm_ptr,0,0,shm_len,1,0,sfp,s_mail.ifn);
	    }	
	    else{ 
		if(s_mail.last)		// last buf	
		    Tx_MultiFile(s_mail.format,shm_ptr,0,0,shm_len,0,1,sfp,s_mail.ifn);
		else			//othsr buf
		    Tx_MultiFile(s_mail.format,shm_ptr,0,0,shm_len,0,0,sfp,s_mail.ifn);
	    }	
	}
   }
	set_resp1(OK);
	return 1;		//send next buffer loop
}


int AbortMailPkt(void)
{
	DBG( LogMsg("abort sending mail now...\n"); )
	if(!en_smtpc.enable){
		DBG( LogMsg("Enable SMTPC first before abort send mail!\n"); )
		return(set_resp(NO_EN_SMTPC,&(cmd_head),NULL,NULL));
	}
	memcpy(&cmd_head, buf, sizeof(CMD_HEAD));
	m_flag = 1;
	return(set_resp(SERVER_NO_RSP,&cmd_head,NULL,NULL));
}

int EnableMailPkt(void)
{
	DBG( LogMsg("Enable SMTPC now...\n"); )
	
	memcpy(&en_smtpc, buf, sizeof(CMD_ENSMTPC));
	if(en_smtpc.enable)
		get_stc.stat = STC_ENABLED;
	else
		get_stc.stat = STC_DISABLED;
	return(set_resp(OK,&(en_smtpc.head),NULL,NULL));
}

static void * do_thread(void *arg)
{
   int connfd;
   int ccode;
   int t_stat=1;

   connfd = (int)arg;

   while(t_stat){

	ccode = recv(connfd, buf, sizeof(buf), 0);

	cmd_version=buf[0];
	cmd_type=buf[3];
	
	if( cmd_version != 0x01){		//version check
		memcpy(&cmd_head, buf, sizeof(CMD_HEAD));
		set_resp(INVALID_VER,&cmd_head,NULL,NULL);
		send(connfd, buf, sizeof(CMD_HEAD), 0);
		break;
	}

	switch(cmd_type) {		//parser	
		case CMD_SET_CONFIG:
			if(ccode != sizeof(CMD_CONF)){
				DBG( LogMsg("receive config data error!\n"); )
				goto end;
			}
			t_stat = SetConfigPkt();
			break;
		case CMD_GET_CONFIG:
			if(ccode != sizeof(CMD_HEAD)){
				DBG( LogMsg("receive header data error!\n"); )
				goto end;
			}
			t_stat = GetConfigPkt();
			break;
		case CMD_GET_STATUS:
			if(ccode != sizeof(CMD_HEAD)){
				DBG( LogMsg("receive header data error!\n"); )
				goto end;
			}
			t_stat = GetStatusPkt();
			break; 
		case CMD_SEND_MAIL:
   			buf_count=0;			
			count=0; 
			if(ccode != sizeof(CMD_SENDMAIL)){
				DBG( LogMsg("receive mail data error1!\n"); )
				goto end;
			}
		   for(;;){
			t_stat = SendMailPkt();
			fflush(sfp);
			buf_count++;	
			count++;
			if(s_mail.last)
				buf_count=0;
			ccode = send(connfd, buf, sizeof(CMD_HEAD), M_NOSIGNAL);
			if(ccode != sizeof(CMD_HEAD) || ccode == -1){
				DBG( LogMsg("1 can't send data to remote normally!\n"); )
				goto end;
			}
			ccode = recv(connfd, buf, sizeof(buf), 0);
			if(!t_stat)	
				break;
			if(ccode != sizeof(CMD_SENDMAIL)){
				DBG( LogMsg("receive mail data error!\n"); )
				break;
			}
			if(m_flag){
				DBG( LogMsg("terminate sending mail!\n"); )
				break;
			}
		   }
		   conf_flag = 0;
		   if(m_flag){
			m_flag = 0;
			memset(&cmd,0,sizeof(cmd));
		   }
end:
		   close(connfd);
		   return(NULL);
		case CMD_SEND_ABORT:
			if(ccode != sizeof(CMD_HEAD)){
				DBG( LogMsg("receive header data error!\n"); )
				goto end;
			}
		   	t_stat = AbortMailPkt();
		   	break;
		case CMD_ENABLE_SMTPC:
			if(ccode != sizeof(CMD_ENSMTPC)){
				DBG( LogMsg("receive enable data error!\n"); )
				goto end;
			}
			t_stat = EnableMailPkt();
			break;
		default:
			DBG( LogMsg("parser error!\n"); )
			memcpy(&cmd_head, buf, sizeof(CMD_HEAD));
			set_resp(INVALID_CMD,&cmd_head,NULL,NULL);
			send(connfd, buf, sizeof(CMD_HEAD), 0);
			break;
	}

	ccode=send_resp((void *)connfd);
	if(ccode)
		break;

   }
   close(connfd);
   return (NULL);			//terminate thread
}

#ifdef CATCH_SIGINT
void sig_int(int signo)
{
	DBG( LogMsg("Process terminated by \"Ctrl+C\" !\n"); )
	exit(0);
}
#endif

int main(void)
{
   struct sockaddr_un address;
   int listenfd,sockfd;
   size_t addrLength;
  
   StartSMTPC();

   DBG( ClrLog(); )
   DBG( LogMsg("Server start...\n"); )

   if ((listenfd = socket(PF_UNIX,SOCK_STREAM,0)) < 0){
	DBG( LogMsg("can't create socket\n"); )
	goto err;
   }
   address.sun_family=AF_UNIX;
   unlink(SMTP_SOCKET_PATH);
   strcpy(address.sun_path, SMTP_SOCKET_PATH);
   addrLength = sizeof(address.sun_family) + strlen(address.sun_path);

   if (bind(listenfd, (struct sockaddr *)&address, addrLength)){
	DBG( LogMsg("bind error\n"); )
	goto err;
   }

   if (listen(listenfd, LISTENQ)){
	DBG( LogMsg("listen error\n"); )
	goto err;
   }

#ifdef CATCH_SIGINT
   signal(SIGINT,sig_int);
#endif

   while(1){
	sockfd = accept(listenfd, (struct sockaddr *)&address, &addrLength);
	if( sockfd < 0){
		DBG( sprintf(logmsg, "accept error: errno=%d\n", errno); )
		DBG( LogMsg(logmsg); )
		continue;
	}

//   	alarm(300);			//timeout 300 seconds

	do_thread((void *)sockfd);

   }
err:
   DBG( LogMsg("ready to shutdown normally now...\n"); )
   close(listenfd);
   exit(0);
}

