/***************************************************************
 *             Copyright (C) 2003 by SerComm Corp.
 *                    All Rights Reserved.
 *
 *      Use of this software is restricted to the terms and
 *      conditions of SerComm's software license agreement.
 *
 *                        www.sercomm.com
 ****************************************************************/
#ifndef _PORTING_2038_H
#define _PROTING_2038_H
#include <linux/mm.h>		/* VERIFY_READ/WRITE */
#include <asm/segment.h>	/* memcpy_fromfs/memcpy_tofs */

				/* debug define */
// #define funcprintk(format, argument...)	printk("func: " format, ## argument);
// #define   myprintk(format, argument...)	printk(format, ## argument);
#define funcprintk(format, argument...)
#define   myprintk(format, argument...)
#define dbugprintk(format, argument...)	printk(format, ## argument);

				/* network define (netdevice.h) */
#define net_device		device
#define net_device_stats	enet_statistics
#define skb_tx_check(dev, skb) \
    do { if (skb == NULL) { dev_tint(dev); return 0; } \
    if (skb->len <= 0) return 0; } while (0)
#define add_rx_bytes(stats, n)	do { int x; x = (n); } while (0)
#define add_tx_bytes(stats, n)	do { int x; x = (n); } while (0)

#define DEV_KFREE_SKB(skb)      dev_kfree_skb(skb, FREE_WRITE)

#define netif_stop_queue(dev)	set_bit(0, (void *)&(dev)->tbusy)
#define netif_start_queue(dev)	clear_bit(0, (void *)&(dev)->tbusy)
#define netif_wake_queue(dev) \
    do { netif_start_queue(dev); mark_bh(NET_BH); } while (0)
#define netif_device_attach(dev) \
    do { (dev)->start = 1; netif_start_queue(dev); } while (0)
#define netif_device_detach(dev) \
    do { (dev)->start = 0; netif_stop_queue(dev); } while (0)
#define netif_device_present(dev) ((dev)->start)
#define netif_running(dev)	((dev)->start)
#define netif_mark_up(dev)	do { (dev)->start = 1; } while (0)
#define netif_mark_down(dev)	do { (dev)->start = 0; } while (0)
#define netif_carrier_on(dev)	do { dev->flags |= IFF_RUNNING; } while (0)
#define netif_carrier_off(dev)	do { dev->flags &= ~IFF_RUNNING; } while (0)
#define netif_queue_stopped(dev) ((dev)->tbusy)
#define tx_timeout_check(dev, tx_timeout) \
    do { if (test_and_set_bit(0, (void *)&(dev)->tbusy) != 0) { \
	if (jiffies - (dev)->trans_start < TX_TIMEOUT) return 1; \
	tx_timeout(dev); \
    } } while (0)
#define DEV_KFREE_SKB(skb)      dev_kfree_skb(skb, FREE_WRITE)
#define dev_kfree_skb_irq(skb)	DEV_KFREE_SKB(skb)

/* A nasty hack: we only need to set up dev->init on 2.0.* kernels,
   and this is a convenient place to bury it */
#define init_dev_name(dev, node) \
  do { static int __dummy(struct net_device *dev) { return 0; } \
    dev->init = &__dummy; (dev)->name = (node).dev_name; \
  } while (0)
#define copy_dev_name(node, dev) do { } while (0)

				/* byte order */
#ifndef le16_to_cpu
#define le16_to_cpu(x)		(x)
#define cpu_to_le16(x)		(x)
#define cpu_to_le16s(x)		do { } while (0)
#define le16_to_cpus(x)		do { } while (0)
#endif /* le16_to_cpu */
#define uint16_t		unsigned short /* asm/types.h */
#define uint32_t		unsigned int   /* asm/types.h */

				/* module parameter */
#define MODULE_PARM(a,b)	extern int __bogus_decl
#define MODULE_AUTHOR(a)	extern int __bogus_decl
#define MODULE_DESCRIPTION(a)	extern int __bogus_decl
#define MODULE_SUPPORTED_DEVICE(a) extern int __bogus_decl
#define SET_MODULE_OWNER(m)	do { } while (0)
#define MODULE_LICENSE(m)	extern int __bogus_decl

				/* kernel */
#define __init
#define __initdata
#define __exit
#define __exitdata
#define module_init(x)		int init_module(void) { return x(); }
#define module_exit(x)		void cleanup_module(void) { x(); }
#define capable(x)		(suser())
static inline u_long copy_from_user(void *to, const void *from, u_long n)
{
    int i;
    if ((i = verify_area(VERIFY_READ, from, n)) != 0)
	return i;
    memcpy_fromfs(to, from, n);
    return 0;
}
static inline u_long copy_to_user(void *to, const void *from, u_long n)
{
    int i;
    if ((i = verify_area(VERIFY_WRITE, to, n)) != 0)
	return i;
    memcpy_tofs(to, from, n);
    return 0;
}

				/* spinlock */
#define spin_lock(l)		do { } while (0)
#define spin_unlock(l)		do { } while (0)
#define spin_lock_irqsave(l,f)	do { save_flags(f); cli(); } while (0)
#define spin_unlock_irqrestore(l,f) do { restore_flags(f); } while (0)
#define spin_lock_init(s)	do { } while (0)
#define spin_trylock(l)		(1)
typedef int spinlock_t;
#define spin_is_locked(l)	(0)
#define local_bh_disable()	disable_bh(NET_BH) /* we always do NET_BH */
#define local_bh_enable()	enable_bh(NET_BH)  /* we always do NET_BH */
#define spin_lock_bh(lock)	do { local_bh_disable();  spin_lock(lock); } while (0)
#define spin_unlock_bh(lock)	do { spin_unlock(lock);  local_bh_enable();        } while (0)
#define __cli()			cli() /* asm/system.h */
#define __sti()			sti() /* asm/system.h */

				/* macro */
#define test_and_set_bit      set_bit
#define BUG() \
    do { printk("kernel BUG at %s:%d!\n", __FILE__, __LINE__); \
	 *(int *)0=0; } while (0)
#define min(x,y) ({ \
	const typeof(x) _x = (x);	\
	const typeof(y) _y = (y);	\
	(void) (&_x == &_y);		\
	_x < _y ? _x : _y; })
#define max(x,y) ({ \
	const typeof(x) _x = (x);	\
	const typeof(y) _y = (y);	\
	(void) (&_x == &_y);		\
	_x > _y ? _x : _y; })
#define min_t(type,x,y) \
	({ type __x = (x), __y = (y); __x < __y ? __x: __y; })
#define max_t(type,x,y) \
	({ type __x = (x), __y = (y); __x > __y ? __x: __y; })
#define mdelay(x) udelay(1000*(x))
#define mod_timer(a, b)	\
    do { del_timer(a); (a)->expires = (b); add_timer(a); } while (0)

#endif  /* _PORTING_2038_H */
