/*
 * linux/arch/armnommu/kernel/head-arm-dsc21.S
 *
 * Derived from head-arm-atmel.S and Boot.asm
 * copyright:
 *         (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 * author: Steve Johnson <stevej@ridgerun.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <asm/assembler.h>
#include <asm/hardware.h>
#include <linux/autoconf.h>        
                                
/*
        This is the define to put us into service mode.
*/
#define ARM_MODE_SVC 0x13

/*
        We need a place for our stack.  For now, let's put it at the end of RAM.
*/
#define IRAM_END 0x8000
                        
        .text
        .align

        .globl	entry
        .globl	stext
        .globl start   

start:
stext:
entry:             

        /*  turn off IRQ/FIQ and put into service mode, then define top of stack */
        mov	r0, #(ARM_MODE_SVC | I_BIT | F_BIT )
        msr	cpsr, r0

        /* don't bother to set stack here.  It will get set after sdram_ini
        ldr r13, =IRAM_END
        */
        
        /*  Initialize the various clock(pll) registers to TI values */
        
        bl clock_ini

@        @ wait for a while (for SDRAM)
@        mov	r1, #0xFF00
@wloop:	subs r1, r1, #1
@        bne	wloop
@
@        /* Initialize the SDRAM controller so we can talk with the 16MB SDRAM */
@ asdf
@#ifndef CONFIG_BOOT_FROM_SDRAM
@        bl sdram_ini
@#endif

        /*  Write memory test to IRAM */
        ldr r8, =0x8000
        ldr r5, =0x0
        ldr r4, MEMTEST_DATA
1:      cmp r5, r8 
        strcc r4, [r5], #4
        bcc 1b       
        
#undef MEMORY_CLEAR        
#ifdef MEMORY_CLEAR
        /*  Write memory test to SDRAM */
        ldr r8, =0x8800000	; @@ 0x8c00000->0x8400000
        ldr r5, =0x8400000	; @@ 0x8000000->0x8100000
        ldr r4, MEMTEST_DATA
1:      cmp r5, r8 
        strcc r4, [r5], #4
        bcc 1b       
        
#endif
        
        adr	r5, LC0
        ldmia	r5, {r5, r6, sp}			@ Setup stack

        /* Clear BSS.  */ 
        ldr r4, =0
1:      cmp r5, r6 
        strcc r4, [r5], #4
        bcc 1b       
        
        /* Copy default .data section from Flash to SDRAM. */
        adr     r5, LC1
        ldmia   r5, {r0, r1, r2}

1:      ldmia r0, {r3 - r10}
        stmia r1, {r3 - r10}
        add r1, r1, #32
        add r0, r0, #32
        cmp r1, r2
        blt 1b

        adr	r5, LC0
        ldmia	r5, {r5, r6, sp}  @ restore previous settings.

	/* Uncompress the Kernel.  r0 = decompress dest. */
	ldr	r0, =LOADADDR
	bl	decompress_kernel

	/* Branch to start of Kernel */
        mov fp, #0
	ldr	r0, =LOADADDR
/*        b	LOADADDR  @ Start of Kernel */
	mov	pc, r0
        
LC0:	.long	__bss_start
	.long	__bss_end
        .long	kernel_stack_start

LC1:	.long	__RONLY_end  @ start of where .data defaults are stored in Flash.
        .long	__data_start @ start of where .data needs to reside in SDRAM.
        .long	__data_end

       
/*
        Local functions.
*/
         
        .macro regw, reg_add, reg_data
        ldr r0, \reg_add
        ldr r1, \reg_data
        strh r1, [r0]       
        .endm
 
        

/*******************************************************************
        Initialize SDRAM and PLLs for each DSC.
*******************************************************************/

#if defined (CONFIG_ARCH_DSC21)
                                
/*******************************************************************
;* CLOCK controller initialize
;*******************************************************************/
clock_ini:
	regw	CLOCKC_PLLARM, PLLARM_DATA
	regw	CLOCKC_ARMCLK1, ARMCLK_DATA1
	regw	CLOCKC_ARMCLK2, ARMCLK_DATA2
	regw	CLOCKC_ARMCLK3, ARMCLK_DATA3
	regw	CLOCKC_PLLDSP, PLLDSP_DATA
	regw	CLOCKC_DSPCLK, DSPCLK_DATA
	regw	CLOCKC_PLLSDR, PLLSDR_DATA
	regw	CLOCKC_SDRCLK, SDRCLK_DATA
	regw	CLOCKC_ENCCLK, ENCCLK_DATA
	regw	CLOCKC_USBCLK, USBCLK_DATA
	regw	CLOCKC_CCDCLK, CCDCLK_DATA

	mov		pc, lr
        
        
/*******************************************************************
         SDRAM initialize  (and SDRAM precharge)

        Obviously, there's a lot of redundant work
        being done here.  Reloading r0 and r1 with values they
        already have looks silly.  But I don't know if the
        precharge of the RAM will work if I hammer it 8
        times in a row without the intervening reads.  So
        I'll keep TI's timing.
*******************************************************************/
sdram_ini:
	regw	SDRAMC_MODE, SDMODE_DATA
	regw	SDRAMC_REFCTL, SDREF_DATA
	regw	SDRAMC_MODE, SDCNT_DATA1
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA3
	mov		pc, lr


CLOCKC_PLLARM:    .word	0x30A80
CLOCKC_ARMCLK1: 	.word	0x30A82
CLOCKC_ARMCLK2: 	.word	0x30A84
CLOCKC_ARMCLK3: 	.word	0x30A86
CLOCKC_PLLDSP:  	.word	0x30A88
CLOCKC_DSPCLK:  	.word	0x30A8A
CLOCKC_PLLSDR:  	.word	0x30A8C
CLOCKC_SDRCLK:  	.word	0x30A8E
CLOCKC_ENCCLK:  	.word	0x30A90
CLOCKC_USBCLK:  	.word	0x30A92
CLOCKC_CCDCLK:  	.word	0x30A94

PLLARM_DATA:   		.word	0x2F80			@ ARM CLK is 27MHz * (11/4)/2 = 37.125MHz
ARMCLK_DATA1:   	.word	0x7300
ARMCLK_DATA2:   	.word	0x7E1F
ARMCLK_DATA3:   	.word	0x3C78
PLLDSP_DATA:   		.word	0x2900			@ DSP CLK is 27MHz * 10/3 = 90MHz
DSPCLK_DATA:   		.word	0x0607
PLLSDR_DATA:   		.word	0x5F80			@ SDRAM CLK is 27MHz * (23/4)/2 = 77.625MHz
SDRCLK_DATA:   		.word	0x017E
ENCCLK_DATA:   		.word	0x475F
USBCLK_DATA:   		.word	0x0003
CCDCLK_DATA:   		.word	0x0087

        
SDRAMC_MODE:      .word	0x309A6
SDRAMC_REFCTL:    .word	0x309A8
SDMODE_DATA:      .word	0x0400	; @@ 0x0500->0x0400
SDREF_DATA:       .word	0x0140
SDCNT_DATA1:      .word	0x0402	; @@ 0x0502->0x0402
SDCNT_DATA2:      .word	0x0404	; @@ 0x0504->0x0404
SDCNT_DATA3:      .word	0x0401	; @@ 0x0501->0x0401

#elif defined (CONFIG_ARCH_DSC24)

/*******************************************************************
;* CLOCK controller initialize.
        CLK needs to be set before PLLSDR because it has a /2 that is
        necessary to make SDR's clock 23 / 4 / 2.  Otherwise, setting
        PLLSDR will make the SDR clock too high.

        After setting PLLDSP and PLLSDR, wait for them to settle before
        using them.
;*******************************************************************/
clock_ini:
	      regw	CLOCKC_PLLDSP, PLLDSP_DATA
	      regw	CLOCKC_CLCK, CLCK_DATA
	      regw	CLOCKC_PLLSDR, PLLSDR_DATA
 	      regw	CLOCKC_MOD1, MOD1_DATA
	      regw	CLOCKC_MOD2, MOD2_DATA
	      regw	CLOCKC_MOD3, MOD3_DATA

  @ wait until PLLs settle
dsploop:
        ldr r0, CLOCKC_PLLDSP
        ldrh r1, [r0]
        and r1, r1, #0x8000
        beq	dsploop
sdramloop:
        ldr r0, CLOCKC_PLLSDR
        ldrh r1, [r0]
        and r1, r1, #0x8000
        beq	sdramloop
      	mov		pc, lr
        
        
/*******************************************************************
         SDRAM initialize  (and SDRAM precharge)

        Obviously, there's a lot of redundant work
        being done here.  Reloading r0 and r1 with values they
        already have looks silly.  But I don't know if the
        precharge of the RAM will work if I hammer it 8
        times in a row without the intervening reads.  So
        I'll keep TI's timing.
*******************************************************************/
sdram_ini:
      	regw	SDRAMC_MODE, SDMODE_DATA
      	regw	SDRAMC_REFCTL, SDREF_DATA
      	regw	SDRAMC_MODE, SDCNT_DATA1
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA3
      	mov		pc, lr


CLOCKC_PLLDSP:  	.word	0x30880
CLOCKC_PLLSDR:  	.word	0x30882
CLOCKC_CLCK:    	.word	0x30884
CLOCKC_MOD1:    	.word	0x30886
CLOCKC_MOD2:    	.word	0x30888
CLOCKC_MOD3:    	.word	0x3088A

PLLDSP_DATA:   		.word	0x00F0			@ DSP CLK is 27MHz * 15/4 = 101.25
PLLSDR_DATA:   		.word	0x00c0			@ SDRAM CLK is 27MHz * 12/4 = 77.625 
CLCK_DATA:   		  .word	0x0090      @ ARM CLK = SDRAM CLK / 2 = 38.8125
MOD1_DATA:   		  .word	0x07FF
MOD2_DATA:   	  	.word	0x00FF
MOD3_DATA:     		.word	0x0FFE
        
SDRAMC_MODE:      .word	0x309A6
SDRAMC_REFCTL:    .word	0x309A8
SDMODE_DATA:      .word	0x1D00
SDREF_DATA:       .word	0x0110
SDCNT_DATA1:      .word	0x1D02
SDCNT_DATA2:      .word	0x1D04
SDCNT_DATA3:      .word	0x1D01

#endif
        
MEMTEST_DATA:   .long 0xfed0fede      @ ARM breakpoint code

        /*  These values should stay in sync with linux/kernel.h */
        
L_STACK_MAGIC:  .long 0xdeadbeef
L_STACK_UNTOUCHED_MAGIC:        .long 0xfeef1ef0

SDBUF_AD0:       .word 0x309A0
        
