/* ld script to make ARM zImage Linux kernel on the DSC-21
 * taken from the i386 version by Russell King
 * Written by Brian Mahaffy <brianm@ridgerun.com>
 *
 * This File is very specific for the DSC-21 Platform.
 */
OUTPUT_ARCH(arm)
ENTRY(stext)
SECTIONS
{
  . = 0x1000;
	.bss : {
		. = ALIGN(32);    /* for SDRAM burst mode use */
		__bss_start = .;	/* BSS				*/
		*(.bss)
		*(COMMON)
		__bss_end = . ;
   		kernel_stack_end = .;
	}
  . = 0x8000;                 /* IRAM, top 4K (0x7000-0x8000) */
  	.iram : {
  		*(.text.iram);
  		*(.data.iram);
  		*(.bss.iram);
   		kernel_stack_start = .;
  }

  . = 0x02020020; 	/* Use this for a zImage for Flash */
/*  . = 0x08040020;*/	/* Use this for a zImage for SDRAM */
	.text : {			/* Real text segment		*/
		_text = .;		/* Text and read-only data	*/
			*(.text)
			*(.fixup)
			*(.gnu.warning)
			*(.text.lock)	/* out-of-line lock text */
			*(.rodata)
			*(.kstrtab)
		. = ALIGN(16);
		__start___ex_table = .;	/* Exception table		*/
			*(__ex_table)
		__stop___ex_table = .;

		__start___ksymtab = .;	/* Kernel symbol table		*/
			*(__ksymtab)
		__stop___ksymtab = .;

		*(.got)			/* Global offset table		*/

		_etext = .;		/* End of text section		*/
    		etext = .;
	}

	.piggy.data : {
	}

	. = ALIGN(32); /* Need the next item aligned for the relocate code */

	__RONLY_end = ABSOLUTE(.); /* Please See Appendix A */

  	. = 0x08020000;
	.data : {
		__data_start = ABSOLUTE(.);	/* DATA */
		/*
		 * The usual data section
		 */
		*(.data)
		__data_end = ABSOLUTE(.);
		_edata = .;
		heap_start = .;
	}
	.heap  0x8030000 : {
		heap_end = .;
        }

	.stab 0 : { *(.stab) }
        .stabstr 0 : { *(.stabstr) }
        .stab.excl 0 : { *(.stab.excl) }
        .stab.exclstr 0 : { *(.stab.exclstr) }
        .stab.index 0 : { *(.stab.index) }
        .stab.indexstr 0 : { *(.stab.indexstr) }
        .comment 0 : { *(.comment) }
}

/*
  ----------------
  -- Appendix A --
  ----------------

  The __RONLY_end number represents where the last
  *ReadOnly* (.text,.rodata,etc) section byte fell. The 
  significance of this marker is that this is where
  the top level Makefile will adjust the .data section to for 
  the purpose of flashing the system. We want a flashable
  unit that consists of all the ReadOnly sections +
  .data. At boot time the boot/compressed/head.S file will move
  the .data from the flash space (represented by
  __RONLY_end) to the SDRAM location (represented
  by __data_start) where it really needs to reside for
  running.

  Summary -- at program invoke, the .data section needs 
             specific default values and hence must reside 
             in FLASH, yet the .data section is read-writable 
             by the program and hence needs to reside in 
             SDRAM when running. So, in *early* boot, move
             .data from flash to SDRAM and then run as normal.
*/
