/*
 * linux/arch/armnommu/kernel/head-arm-dsc21.S
 *
 * Derived from head-arm-atmel.S and Boot.asm
 * copyright:
 *         (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 * author: Steve Johnson <stevej@ridgerun.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <asm/assembler.h>
#include <asm/hardware.h>
#include <linux/autoconf.h>        
                                
/*
        This is the define to put us into service mode.
*/
#define ARM_MODE_SVC 0x13

/*
        We need a place for our stack.  For now, let's put it at the end of RAM.
*/
#define IRAM_END 0x8000
                        
        .text
        .align

        .globl	__entry,_entry
        .globl	__stext,_stext
        .globl _start,start   

start:
_start:
_stext:
_entry:             
__stext:
__entry:

        /*  turn off IRQ/FIQ and put into service mode, then define top of stack */
        mov	r0, #(ARM_MODE_SVC | I_BIT | F_BIT )
        msr	cpsr, r0

        /* don't bother to set stack here.  It will get set after sdram_ini
        ldr r13, =IRAM_END
        */
        
        /*  Initialize the various clock(pll) registers to TI values */
#ifdef _NEED_INIT_AGAIN		;;@@        
        bl clock_ini

        @ wait for a while (for SDRAM)
        mov	r1, #0xFF00
wloop:	subs r1, r1, #1
        bne	wloop

        /*  Initialize the SDRAM controller so we can talk with the 16MB SDRAM */

#ifndef CONFIG_BOOT_FROM_SDRAM
        bl sdram_ini
#endif
#endif	;; _NEED_INIT_AGAIN	;;@@

#ifndef CONFIG_DSCXX_USE_IRAM
        /*  Write memory test to IRAM */
        ldr r8, =0x8000
        ldr r5, =0x0
        ldr r4, MEMTEST_DATA
1:      cmp r5, r8 
        strcc r4, [r5], #4
        bcc 1b       
#endif
        
#undef MEMORY_CLEAR        
#ifdef MEMORY_CLEAR
        /*  Write memory test to SDRAM */
        ldr r8, =0x8c00000
        ldr r5, =0x8000000
        ldr r4, MEMTEST_DATA
1:      cmp r5, r8 
        strcc r4, [r5], #4
        bcc 1b       
        
#endif

#ifdef CONFIG_DSCXX_USE_IRAM
        /* Copy default .iram section to IRAM. */
        adr     r5, LC2
        ldmia   r5, {r0, r1, r2, r3}

#if defined(CONFIG_BOOT_FROM_FLASH)
		add		r0, r0, r3
#endif

1:      ldmia r0!, {r3 - r10}
        stmia r1!, {r3 - r10}
        cmp r1, r2
        blt 1b
        
@        adr	r5, LC0
@        ldmia	r5, {r5, r6, r8, sp}  @ restore previous settings.
#endif
        
        adr	r5, LC0
        ldmia	r5, {r5, r6, r8, sp}			@ Setup stack

        /* 
        Clear .bss, using the SDRAM burst controller.
        Using autoincrement mode, you never have to change
        the addresses once they are set.
        The addresses are physical addresses based on the start
        of SDRAM.  So subtract the SDRAM base from the input
        address.
        */
                
        ldr r1, SDBUF_AD0      @ SDRAM Controller base
        mov r0, #0x8           @ Code #8 is to clear burst buffers
        strh r0, [r1, #4]      @ CTL - clear buffers
        ldr r0, =SDRAM_BASE
        sub r8, r8, r5         @ get number of 32-byte blocks 
        mov r8, r8, lsr #5     @  that will be transferred
        sub r5, r5, r0         @ subtract SDRAM base 
        mov r4, r5, lsr #5     @ address / 32 = address
        mov r2, r4, lsr #16    @ take MSB
        and r2, r2, #0x3f      @ sdbuf MSB max 6 bits
        orr r2, r2, #0x100     @ set autoinc bit
        strh r2, [r1]          @ write MSB + autoinc bit
        strh r4, [r1, #2]      @ write LSB
        mov r3, #0x2           @ Code #2 is to Write burst
1:      strh r3, [r1, #4]      @ CTL - write burst
        subs r8, r8, #1        @ repeat as necessary
        bne 1b
                        
#ifdef CONFIG_BOOT_FROM_FLASH
        /* Copy default .data section from Flash to SDRAM. */
        adr     r5, LC1
        ldmia   r5, {r0, r1, r2}

1:      ldmia r0!, {r3 - r10}
        stmia r1!, {r3 - r10}
        cmp r1, r2
        blt 1b
        
        adr	r5, LC0
        ldmia	r5, {r5, r6, r8, sp}  @ restore previous settings.
#endif

        /*  Put initial values into stack.  This would normally be done
        by sched_init() in kernel/sched.c, but that would overwrite the
        stack we're already using.  That would be bad.  This uses a
        fast register push into IRAM, with one loop add/check for 32
        bytes.
        */
        mov r2, r6  @store off setting for later user
        sub r4, sp, #0x800
        ldr r5, L_STACK_UNTOUCHED_MAGIC
        mov r6, r5
        mov r7, r5
        mov r8, r5
        mov r9, r5
        mov r10, r5
        mov r11, r5
        mov r12, r5
1:      stmia r4!, {r5 - r12}
        @add r4, r4, #32
        cmp r4, sp
        blt 1b
        sub r4, sp, #0x800
        ldr r5, L_STACK_MAGIC
        str r5, [r4]        
                
        /*  Pretend we know what our processor code is (for arm_id)   */
        ldr	r6,=0x41007300	
        str	r6, [r2]

        mov fp, #0
        b	start_kernel
        
LC0:	.long	__bss_start
        .long	arm_id
        .long	_end
        .long	init_kernel_stack + 0x800

LC1:	.long	__RONLY_end  @ start of where .data defaults are stored in Flash.
        .long	__data_start @ start of where .data needs to reside in SDRAM.
        .long	__data_end

#ifdef CONFIG_DSCXX_USE_IRAM

#if defined(CONFIG_BOOT_FROM_FLASH)
LC2:	.long	__RONLY_end  @ Where .data is stored in ELF file, we will add sizeof later.
#else
LC2:	.long	_edata  @ Start of where .iram defaults are stored in ELF file.
#endif
        .long	__iram_start @ start of where .iram needs to reside in IRAM.
        .long	__iram_end
        .long	__DATASEG_size
#endif
       
       
/*
        Local functions.
*/
         
        .macro regw, reg_add, reg_data
        ldr r0, \reg_add
        ldr r1, \reg_data
        strh r1, [r0]       
        .endm
 
        

/*******************************************************************
        Initialize SDRAM and PLLs for each DSC.
*******************************************************************/

#if defined (CONFIG_ARCH_DSC21)
                                
/*******************************************************************
;* CLOCK controller initialize
;*******************************************************************/
clock_ini:
	regw	CLOCKC_PLLARM, PLLARM_DATA
	regw	CLOCKC_ARMCLK1, ARMCLK_DATA1
	regw	CLOCKC_ARMCLK2, ARMCLK_DATA2
	regw	CLOCKC_ARMCLK3, ARMCLK_DATA3
	regw	CLOCKC_PLLDSP, PLLDSP_DATA
	regw	CLOCKC_DSPCLK, DSPCLK_DATA
	regw	CLOCKC_PLLSDR, PLLSDR_DATA
	regw	CLOCKC_SDRCLK, SDRCLK_DATA
	regw	CLOCKC_ENCCLK, ENCCLK_DATA
	regw	CLOCKC_USBCLK, USBCLK_DATA
	regw	CLOCKC_CCDCLK, CCDCLK_DATA

	mov		pc, lr
        
        
/*******************************************************************
         SDRAM initialize  (and SDRAM precharge)

        Obviously, there's a lot of redundant work
        being done here.  Reloading r0 and r1 with values they
        already have looks silly.  But I don't know if the
        precharge of the RAM will work if I hammer it 8
        times in a row without the intervening reads.  So
        I'll keep TI's timing.
*******************************************************************/
sdram_ini:
	regw	SDRAMC_MODE, SDMODE_DATA
	regw	SDRAMC_REFCTL, SDREF_DATA
	regw	SDRAMC_MODE, SDCNT_DATA1
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA2
	regw	SDRAMC_MODE, SDCNT_DATA3
	mov		pc, lr


CLOCKC_PLLARM:    .word	0x30A80
CLOCKC_ARMCLK1: 	.word	0x30A82
CLOCKC_ARMCLK2: 	.word	0x30A84
CLOCKC_ARMCLK3: 	.word	0x30A86
CLOCKC_PLLDSP:  	.word	0x30A88
CLOCKC_DSPCLK:  	.word	0x30A8A
CLOCKC_PLLSDR:  	.word	0x30A8C
CLOCKC_SDRCLK:  	.word	0x30A8E
CLOCKC_ENCCLK:  	.word	0x30A90
CLOCKC_USBCLK:  	.word	0x30A92
CLOCKC_CCDCLK:  	.word	0x30A94

PLLARM_DATA:   		.word	0x2F80			@ ARM CLK is 27MHz * (11/4)/2 = 37.125MHz
ARMCLK_DATA1:   	.word	0x7300
ARMCLK_DATA2:   	.word	0x7E1F
ARMCLK_DATA3:   	.word	0x3C78
PLLDSP_DATA:   		.word	0x2900			@ DSP CLK is 27MHz * 10/3 = 90MHz
DSPCLK_DATA:   		.word	0x0607
PLLSDR_DATA:   		.word	0x5F80			@ SDRAM CLK is 27MHz * (23/4)/2 = 77.625MHz
SDRCLK_DATA:   		.word	0x017E
ENCCLK_DATA:   		.word	0x475F
USBCLK_DATA:   		.word	0x0003
CCDCLK_DATA:   		.word	0x0087

        
SDRAMC_MODE:      .word	0x309A6
SDRAMC_REFCTL:    .word	0x309A8
SDMODE_DATA:      .word	0x0400	; @@ 0x0500->0x0400
SDREF_DATA:       .word	0x0140
SDCNT_DATA1:      .word	0x0402	; @@ 0x0502->0x0402
SDCNT_DATA2:      .word	0x0404	; @@ 0x0504->0x0404
SDCNT_DATA3:      .word	0x0401	; @@ 0x0501->0x0401

#elif defined (CONFIG_ARCH_DSC24)

/*******************************************************************
;* CLOCK controller initialize.
        CLK needs to be set before PLLSDR because it has a /2 that is
        necessary to make SDR's clock 23 / 4 / 2.  Otherwise, setting
        PLLSDR will make the SDR clock too high.

        After setting PLLDSP and PLLSDR, wait for them to settle before
        using them.
;*******************************************************************/
clock_ini:
	      regw	CLOCKC_PLLDSP, PLLDSP_DATA
	      regw	CLOCKC_CLCK, CLCK_DATA
	      regw	CLOCKC_PLLSDR, PLLSDR_DATA
 	      regw	CLOCKC_MOD1, MOD1_DATA
	      regw	CLOCKC_MOD2, MOD2_DATA
	      regw	CLOCKC_MOD3, MOD3_DATA

  @ wait until PLLs settle
dsploop:
        ldr r0, CLOCKC_PLLDSP
        ldrh r1, [r0]
        and r1, r1, #0x8000
        beq	dsploop
sdramloop:
        ldr r0, CLOCKC_PLLSDR
        ldrh r1, [r0]
        and r1, r1, #0x8000
        beq	sdramloop
      	mov		pc, lr
        
        
/*******************************************************************
         SDRAM initialize  (and SDRAM precharge)

        Obviously, there's a lot of redundant work
        being done here.  Reloading r0 and r1 with values they
        already have looks silly.  But I don't know if the
        precharge of the RAM will work if I hammer it 8
        times in a row without the intervening reads.  So
        I'll keep TI's timing.
*******************************************************************/
sdram_ini:
      	regw	SDRAMC_MODE, SDMODE_DATA
      	regw	SDRAMC_REFCTL, SDREF_DATA
      	regw	SDRAMC_MODE, SDCNT_DATA1
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA2
      	regw	SDRAMC_MODE, SDCNT_DATA3
      	mov		pc, lr


CLOCKC_PLLDSP:  	.word	0x30880
CLOCKC_PLLSDR:  	.word	0x30882
CLOCKC_CLCK:    	.word	0x30884
CLOCKC_MOD1:    	.word	0x30886
CLOCKC_MOD2:    	.word	0x30888
CLOCKC_MOD3:    	.word	0x3088A

PLLDSP_DATA:   		.word	0x00A3			@ DSP CLK is 27MHz * 10/3 = 90
PLLSDR_DATA:   		.word	0x00B0			@ SDRAM CLK is 27MHz * 11/4 = 74.25 
CLCK_DATA:   		  .word	0x0090      @ ARM CLK = SDRAM CLK / 2 = 37.125
MOD1_DATA:   		  .word	0x07FF
MOD2_DATA:   	  	.word	0x00FF
MOD3_DATA:     		.word	0x0FFE
        
SDRAMC_MODE:      .word	0x309A6
SDRAMC_REFCTL:    .word	0x309A8
SDMODE_DATA:      .word	0x1D00
SDREF_DATA:       .word	0x0110
SDCNT_DATA1:      .word	0x1D02
SDCNT_DATA2:      .word	0x1D04
SDCNT_DATA3:      .word	0x1D01

#endif
        
MEMTEST_DATA:   .long 0xfed0fede      @ ARM breakpoint code

        /*  These values should stay in sync with linux/kernel.h */
        
L_STACK_MAGIC:  .long 0xdeadbeef
L_STACK_UNTOUCHED_MAGIC:        .long 0xfeef1ef0

SDBUF_AD0:       .word 0x309A0
        
