/*
 *
 * linux/arch/armnommu/kernel/head-arm-orion.S
 *
 * copyright:
 *        (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 * author: Steve Kranz <skranz@ridgerun.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
        .global DSPLinux_entry

#include <asm/assembler.h>
#include <linux/autoconf.h>        
        
// --------------------------
// Note: 
//    We assume the bootloader has already
//    initialized most of the h/w for us and
//    that only leaves us having to do some 
//    kernel specific things below.
// --------------------------

DSPLinux_entry:
        
// --------------------------
// Setup initial mode
// --------------------------
  #define ARM_MODE_SVC 0x13
  mov  r0, #(ARM_MODE_SVC | I_BIT | F_BIT )
  msr cpsr, r0
        
// --------------------------
// Setup kernel stack
// --------------------------
  adr   r5, LC0
  ldmia r5, {r5, r6, r8, sp} 

// --------------------------
// Clear kernel BSS section
// --------------------------
    mov r4, #0
1:  cmp r5, r8                                  
    strcc   r4, [r5],#4
    bcc 1b
                        
// --------------------------
// Copy kernel's .data sections to new home.
// --------------------------
#ifdef CONFIG_BOOT_FROM_FLASH
   adr   r5, LC1
   ldmia r5, {r0, r1, r2}
1: ldmia r0!,{r3 - r10}
   stmia r1!,{r3 - r10}
   cmp   r1, r2
   blt   1b
   adr   r5, LC0
   ldmia r5, {r5, r6, r8, sp}  @ restore previous settings.
#endif

// --------------------------
// Initialize Kernel Stack Contents
// --------------------------
   mov r2, r6  // store off setting for later user
   sub r4, sp, #0x1000
   ldr r5, L_STACK_UNTOUCHED_MAGIC
   mov r6, r5
   mov r7, r5
   mov r8, r5
   mov r9, r5
   mov r10, r5
   mov r11, r5
   mov r12, r5
1: stmia r4!, {r5 - r12}
   @add r4, r4, #32
   cmp r4, sp
   blt 1b
   sub r4, sp, #0x1000
   ldr r5, L_STACK_MAGIC
   str r5, [r4]        

// --------------------------
// Jump to kernel entry
// --------------------------
   // Pretend we know what our processor code is (for arm_id)
   ldr r6,=0x41007300  
   str r6, [r2]
   mov fp, #0
   b start_kernel

// --------------------------
// Variables
// --------------------------

LC0: .long  __bss_start
     .long  arm_id
     .long  _end
     .long  init_kernel_stack + 0x1000
           
LC1: .long  __RONLY_end  @ start of where .data defaults are stored in Flash.
     .long  __data_start @ start of where .data needs to reside in SDRAM.
     .long  __data_end

L_STACK_UNTOUCHED_MAGIC: .long 0xfeef1ef0
L_STACK_MAGIC:           .long 0xdeadbeef

