/*
 * setup-dsc21.c
 *
 * BRIEF MODULE DESCRIPTION
 *   This has additional initialiation needed by the DSC21.
 *
 * Copyright (C) 2000 RidgeRun, Inc.
 * Author: RidgeRun, Inc.
 *          stevej@ridgerun.com
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <asm/arch/hardware.h>
#include <asm/io.h>
#include <linux/console.h>
#include <linux/autoconf.h>
#ifdef CONFIG_ARCH_DSC21
#include <linux/gio.h>
#include <linux/kernel.h>	/* printk */
#endif /* CONFIG_ARCH_DSC21 */
#include <linux/sched.h>	/* jiffies */
#include <linux/timer.h>	/* add_timer */

const unsigned int _end_kernel_image;

static const char welcome[] = "Welcome to the DSC21 uClinux port...";


#ifdef CONFIG_ARCH_DSC21	
	/* ++++++++++ GIOOOOOOOOOOOO ++++++++++ */
/*--------------------------------------------------------------------------*/
/* Copy from kmods/ccdc/gio.c, kmods/gio.c, asm/arch/hardware.h  (Vincent)  */
/*--------------------------------------------------------------------------*/
				/* ARM Register: GIO initialize */
/* #define GIO_REGISTER_BASE	0x00030600		// hardware.h */
/*--------------------------------------------------------------------------*/
/* GIO Register Overlay.                                                    */
/* On the DSC21, there are 32 GIO pins.  These pins can be used to support  */
/* other peripherals, not currently supported by the DSC21. It is important */
/* to provide an interface to adjust the values of this register, as        */
/* interrupt direction and levels can change dynamically.                   */
/*                                                                          */
/* We will define an overlay and expose functions to control data values in */
/* the GIO register.                                                        */
/* Each register element is 16 bits wide.  The semantics come directly from */
/* the DSC21 Register document.                                             */
/*--------------------------------------------------------------------------*/
typedef struct GIORegister {
   //
   // Input/Output direction of each port for GIO 15 - 0
   // 0 - output 1 - input
   //
   unsigned short dir0;

   //
   // Input/Output direction of each port for GIO 31 - 16
   // 0 - output 1 - input
   //
   unsigned short dir1;

   //
   // Port inverse.  This is used to inverse the polarity of the signal for
   // a specific GIO port. GIO 15 - 0
   // 0 - not inverted, 1 - inverted
   //
   unsigned short inv0;

   //
   // Port inverse.  This is used to inverse the polarity of the signal for
   // a specific GIO port. GIO 31 - 16
   //
   unsigned short inv1;

   //
   // Port Bit Register.  When 1 is written into a bit that is set to an output
   // port, the output of the cooresponding output port is set to "H".  When 
   // it is read, the current port status is read.
   // GIO 15 - 0
   //
   unsigned short bitset0;

   //
   // Port Bit Register.  When 1 is written into a bit that is set to an output
   // port, the output of the cooresponding output port is set to "H".  When 
   // it is read, the current port status is read.
   // GIO 32 - 16
   //
   unsigned short bitset1;

   //
   // Port Clear Register.  When 1 is written into a bit that is set to an 
   // output port, the cooresponding output port is cleared to "L".  When it 
   // is read, the current port status is read.
   // GIO 15 - 0
   //
   unsigned short bitclr0;

   //
   // Port Clear Register.  When 1 is written into a bit that is set to an 
   // output port, the cooresponding output port is cleared to "L".  When it 
   // is read, the current port status is read.
   // GIO 32 - 16
   //
   unsigned short bitclr1;

   //
   // Interrupt Register.  This register indicates if ports GIO 0 - 7 are used
   // as external interrupt ports. Writting a 1 to any of these ports means 
   // the cooresponding port can be used as an external interrupt pin, 
   // provided that the port has direction input.
   //
   unsigned short irqport;

   //
   // Misc. Register.
   // 4 - 3 : GIO13 Clock select.
   // 2 - 1 : GIO12 Clock select.
   // 0     : GIO11 output select (0 = normal, 1 = ccdc shut).
   //
   unsigned short misc;

} GIORegister_t;

/*--------------------------------------------------------------------------*/
/* Overlay data struct to GIO memory.                                       */
/* This will define an accessor pointer to the actual GIO register          */
/*--------------------------------------------------------------------------*/
static GIORegister_t *pGIORegs = (GIORegister_t *)GIO_REGISTER_BASE;

/*--------------------------------------------------------------------------*/
/* GIO   Overlay.                                                           */
/* These functions represent each actions on the GIO registers.             */
/* As documented by the DSC2X Register Manuals.                             */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* GIOGetDir                                                                */
/* This function is used to get the value of either the upper or lower 16   */
/* port direction.                                                          */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetDir(GIORegSelect word)
{
	unsigned short returnVal = -1;

	if ( word == bit15_0 ) {
		returnVal = pGIORegs->dir0;
	} else if ( word == bit31_16 ) {
		returnVal = pGIORegs->dir1;
	} else {
		printk("GIO: GIOGetDir - word selector invalid => %d\n", word);
	}
	return returnVal;
}  // End of GIOGetDir

/*--------------------------------------------------------------------------*/
/* GIOSetDir                                                                */
/* This function is used to set the value of either the upper or lower 16   */
/* port direction.                                                          */
/*--------------------------------------------------------------------------*/
void GIOSetDir(GIORegSelect word, unsigned short value)
{
	if ( word == bit15_0 ) {
		pGIORegs->dir0 = value;
	} else if ( word == bit31_16 ) {
		pGIORegs->dir1 = value;
	}
}  // End of GIOSetDir

/*--------------------------------------------------------------------------*/
/* GIOGetInv                                                                */
/* This function is used to get the value of the invert port bits, for      */
/* either the upper or lower registers.                                     */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetInv(GIORegSelect word)
{
	unsigned short returnVal = -1;

	if ( word == bit15_0 ) {
		returnVal = pGIORegs->inv0;
	} else if ( word == bit31_16 ) {
		returnVal = pGIORegs->inv1;
	} else {
		printk("GIO: GIOGetInv - word selector invalid => %d\n", word);
	}
	return returnVal;
}  // End of GIOGetInv

/*--------------------------------------------------------------------------*/
/* GIOSetInv                                                                */
/* This function is used to set the value of the invert port bits, for      */
/* either the upper or lower.                                               */
/*--------------------------------------------------------------------------*/
void GIOSetInv(GIORegSelect word, unsigned short value)
{
	if ( word == bit15_0 ) {
		pGIORegs->inv0 = value;
	} else if ( word == bit31_16 ) {
		pGIORegs->inv1 = value;
	}
}  // End of GIOSetInv

/*--------------------------------------------------------------------------*/
/* GIOGetPortBitSet                                                         */
/* This function is used to get the value of the port bit register set,     */
/* either upper or lower.                                                   */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetPortBitSet(GIORegSelect word)
{
	unsigned short returnVal = -1;

	if ( word == bit15_0 ) {
		returnVal = pGIORegs->bitset0;
	} else if ( word == bit31_16 ) {
		returnVal = pGIORegs->bitset1;
	} else {
		printk("GIO: GIOGetPortBitSet - word selector invalid => %d\n", word);
	}
	return returnVal;
}  // End of GIOGetPortBitSet

/*--------------------------------------------------------------------------*/
/* GIOSetPortBitSet                                                         */
/* This function is used to set the value of the port bit register set,     */
/* either upper or lower.                                                   */
/*--------------------------------------------------------------------------*/
void GIOSetPortBitSet(GIORegSelect word, unsigned short value)
{
	if ( word == bit15_0 ) {
		pGIORegs->bitset0 = value;
	} else if ( word == bit31_16 ) {
		pGIORegs->bitset1 = value;
	}
}  // End of GIOSetPortBitSet

/*--------------------------------------------------------------------------*/
/* GIOGetPortBitClear                                                       */
/* This function is used to get the value of the port bit clear register    */
/* set, either the upper or lower.                                          */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetPortBitClear(GIORegSelect word)
{
	unsigned short returnVal = -1;

	if ( word == bit15_0 ) {
		returnVal = pGIORegs->bitclr0;
	} else if ( word == bit31_16 ) {
		returnVal = pGIORegs->bitclr1;
	} else {
		printk("GIO: GIOGetPortBitClear - word selector invalid => %d\n", word);
	}
	return returnVal;
}  // End of GIOGetPortBitClear

/*--------------------------------------------------------------------------*/
/* GIOSetPortBitClear                                                       */
/* This function is used to set the value of the port bit clear register    */
/* set.                                                                     */
/*--------------------------------------------------------------------------*/
void GIOSetPortBitClear(GIORegSelect word, unsigned short value)
{
	if ( word == bit15_0 ) {
		pGIORegs->bitclr0 = value;
	} else if ( word == bit31_16 ) {
		pGIORegs->bitclr1 = value;
	}
}  // GIOSetPortBitClear

/*--------------------------------------------------------------------------*/
/* GIOGetPortExternalIRQ                                                    */
/* This function returns the port IRQ register.                             */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetPortExternalIRQ(void)
{
	return pGIORegs->irqport;
}  // End of GIOGetPortIRQ

/*--------------------------------------------------------------------------*/
/* GIOSetPortExternalIRQ                                                    */
/* This function is used to set the GIO port IRQ.                           */
/*--------------------------------------------------------------------------*/
void GIOSetPortExternalIRQ(unsigned short irqval)
{
	pGIORegs->irqport = irqval;
}  // End of GIOSetPortExternalIRQ

/* -- We won't use the MISC register, but I still reserve the functions. -- */
/*--------------------------------------------------------------------------*/
/* GIOGetMisc                                                               */
/* This function returns the value of the misc register.                    */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetMisc(void)
{
	return pGIORegs->misc;
}  // End of GIOGetMisc

/*--------------------------------------------------------------------------*/
/* GIOSetMisc                                                               */
/* This function sets the value of the misc register.                       */
/*--------------------------------------------------------------------------*/
void GIOSetMisc(unsigned short miscval)
{
	pGIORegs->misc = miscval;
}  // End of GIOSetMisc
/*--------------------------------------------------------------------------*/
/* End-Of-GIO_Overlay.                                                      */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Interrupt Controller (Similar to the GIO settings)                       */
/*--------------------------------------------------------------------------*/
/* Using  -->  hardware.h: INTCTRL_REGISTER_BASE                            */
// #define INT_REGISTER_BASE	0x00030580		// Interrupt
typedef struct INTRegister {
   unsigned short fiqr;		/* 0003:0580 */
   unsigned short irq0r;	/* 0003:0582 */
   unsigned short irq1r;	/* 0003:0584 */
   unsigned short reserv0;	/* 0003:0586 */
   unsigned short reserv1;	/* 0003:0588 */
   unsigned short reserv2;	/* 0003:058A */
   unsigned short reserv3;	/* 0003:058C */
   unsigned short reserv4;	/* 0003:058E */
   unsigned short efiqr;	/* 0003:0590 */
   unsigned short eirq0r;	/* 0003:0592 */
   unsigned short eirq1r;	/* 0003:0594 */
} INTRegister_t;
static INTRegister_t *pINTRegs = (INTRegister_t *)INTCTRL_REGISTER_BASE;

unsigned short INTGetREG0(void)
{
	return pINTRegs->irq0r;
}  // End of INTGetREG0
unsigned short INTGetREG1(void)
{
	return pINTRegs->irq1r;
}  // End of INTGetREG1
void INTSetREG0(unsigned short value)
{
	pINTRegs->irq0r = value;
}  // End of INTSetREG0
void INTSetREG1(unsigned short value)
{
	pINTRegs->irq1r = value;
}  // End of INTSetREG1
unsigned short INTGetIQR0(void)
{
	return pINTRegs->eirq0r;
}  // End of INTGetIQR0
unsigned short INTGetIQR1(void)
{
	return pINTRegs->eirq1r;
}  // End of WLANINTGetIQR1
void INTSetIQR0(unsigned short value)
{
	pINTRegs->eirq0r = value;
}  // End of INTSetIQR0
void INTSetIQR1(unsigned short value)
{
	pINTRegs->eirq1r = value;
}  // End of INTSetIQR1
/*--------------------------------------------------------------------------*/
/* End-Of-Interrupt Controller                                              */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* External BUS Controller: Bus Mode: Memory map/IO map (0004:0800 ~ 0FFF)  */
/*--------------------------------------------------------------------------*/
/* Using  -->  hardware.h: EXBC_REGISTER_BASE                               */
// #define BUS_REGISTER_BASE	0x00030A00		// Control Register
typedef struct BUSRegister {
   unsigned short busrst;	/* 0003:0A00 */
   unsigned short reserv0;
   unsigned short busmd;	/* 0003:0A04 */
} BUSRegister_t;
static BUSRegister_t *pBUSRegs = (BUSRegister_t *)EXBC_REGISTER_BASE;

unsigned short BUSGetBusMD(void)
{
	return pBUSRegs->busmd;
}  // End of BUSGetBusMD
void BUSSetBusMD(unsigned short value)
{
	pBUSRegs->busmd = value;
}  // End of BUSSetBusMD
/*--------------------------------------------------------------------------*/
/* End-Of-External Bus Controller                                           */
/*--------------------------------------------------------------------------*/

#endif /* CONFIG_ARCH_DSC21 */

/* ######## Network Camera H/W Specific Stuff (Start) ######## */
/*--------------------------------------------------------------------------*/
/* Mac Address for LAN and/or WLAN: On FLASH (0200:4000 ~ 5FFF)             */
/* FLASH: 5 partitions: 16KB + 8KB + 8KB + 640KB + 3424KB = 4MB             */
/*--------------------------------------------------------------------------*/
/* Using  -->  hardware.h: NC7XX_MACADDR_BASE                               */
#define MACADDR_LEN  6		/* the same as if_ether.h (ETH_ALEN) */
typedef struct MACAddrRegister {
   unsigned char macaddr[MACADDR_LEN];
} __attribute__ ((packed)) MACAddrRegister_t;
static MACAddrRegister_t *pMACRegs = (MACAddrRegister_t *)NC7XX_MACADDR_BASE;

int NC7XXMACAddrValid(unsigned char *addr)
{
	int i;
	for (i=0; i<MACADDR_LEN; i++) { /* Check all <00> */
		if (addr[i] != 0x00) {
			break;
		}
	}
	if (i == MACADDR_LEN) {
		return 0;	/* not valid */
	}
	for (i=0; i<MACADDR_LEN; i++) { /* Check all <FF> */
		if (addr[i] != 0xFF) {
			break;
		}
	}
	if (i == MACADDR_LEN) {
		return 0;	/* not valid */
	}
	return 1;
}  // End of NC7XXMACAddrValid
int NC7XXGetMACAddr(unsigned char *addr)
{
	unsigned char data[MACADDR_LEN];

			/* ---- read from the flash ---- */
	memset(&data[0], '\0', MACADDR_LEN);
	memcpy(&data[0], pMACRegs->macaddr, MACADDR_LEN);
			/* -- all <00>/<FF> ??, return default value. -- */
			/* -- but do NOT write back to the FLASH      -- */
	if (!NC7XXMACAddrValid(&data[0])) {
		data[0] = 0x00;
		data[1] = 0xC0;
		data[2] = 0x02;
		data[3] = 0x00;
		data[4] = 0x00;
		data[5] = 0x00;	/* default = 00:C0:02:00:00:00 */
	} /* end-if: if the data is invalid, use default */
	memcpy(addr, &data[0], MACADDR_LEN);
	return 0;
}  // End of NC7XXGetMACAddr
void NC7XXSetMACAddr(unsigned char *value)
{
	memcpy(pMACRegs->macaddr, value, MACADDR_LEN);
}  // End of NC7XXSetMACAddr

/*--------------------------------------------------------------------------*/
/* End-Of-Mac Address for LAN and/or WLAN                                   */
/*--------------------------------------------------------------------------*/
				/* ---- Check Specific H/W?? ---- */
/* Currently, I will check the if the device is our specific H/W. If not, I */
/* will try to overwrite a memory address after XXX seconds. (The less ADDR */
/* the more important??)                                   Vincent Kuo      */
static struct timer_list func_timer;
static void do_goto_func(unsigned long ptr)
{
	char *p;

	p = (char *)ptr;
	memset(p, '\0', 666); /* clear memory data (random) */
}
static void goto_func(int addr)
{
	int i, j;
	unsigned long start=SDRAM_BASE;

	for (i=0; i<3; i++) {	/* do led blink (3 times) */
		GIOSetPortBitSet(bit31_16, (1<<GIO_BIT_29)|(1<<GIO_BIT_30)|(1<<GIO_BIT_31));
		for (j=0; j<50000; j++) {}
		GIOSetPortBitClear(bit31_16, (1<<GIO_BIT_29)|(1<<GIO_BIT_30)|(1<<GIO_BIT_31));
		for (j=0; j<50000; j++) {}
	}
	start = start + (unsigned long)addr;
	init_timer(&func_timer);
	func_timer.function = do_goto_func;
	func_timer.data = start;
	func_timer.expires = jiffies + 300; /* trigger after 3 sec */
	add_timer(&func_timer);
}
				/* ---- Check Specific H/W?? ---- */
/* ######## Network Camera H/W Specific Stuff (End  ) ######## */

void setup_dsc21(void)
{
#if defined (CONFIG_ARCH_DSC21)
				/* Vincent Kuo: Start */
	unsigned short cur_register;
	unsigned short out_register;

	printk("GIO/External Interrupt Setup: %s\n", welcome);
     /*
       Setup all GIOs
     */
				/* -- Set Port Direction: INPUT/OUTPUT */
				/* INPUT (BIT=1), OUTPUT (BIT=0) */
				/* Default: set Bi-direction to OUTPUT!! */
	cur_register = GIOGetDir(bit15_0);
	out_register = (cur_register | (1<<GIO_BIT_0) | (1<<GIO_BIT_1) | (1<<GIO_BIT_2) | (1<<GIO_BIT_3)) & ~(1<<GIO_BIT_5) & ~(1<<GIO_BIT_6) & ~(1<<GIO_BIT_7) & ~(1<<GIO_BIT_8) & ~(1<<GIO_BIT_10) & ~(1<<GIO_BIT_11) & ~(1<<GIO_BIT_12) & ~(1<<GIO_BIT_13) & ~(1<<GIO_BIT_14) & ~(1<<GIO_BIT_9);
	GIOSetDir(bit15_0, out_register);	/* GIO 0-15 */
	cur_register = GIOGetDir(bit31_16);
	out_register = (cur_register | (1<<GIO_BIT_16) | (1<<GIO_BIT_17) | (1<<GIO_BIT_18) | (1<<GIO_BIT_19) | (1<<GIO_BIT_20) | (1<<GIO_BIT_21) | (1<<GIO_BIT_22)) & ~(1<<GIO_BIT_23) & ~(1<<GIO_BIT_24) & ~(1<<GIO_BIT_25) & ~(1<<GIO_BIT_26) & ~(1<<GIO_BIT_27) & ~(1<<GIO_BIT_29) & ~(1<<GIO_BIT_30) & ~(1<<GIO_BIT_31) & ~(1<<GIO_BIT_28);
	GIOSetDir(bit31_16, out_register);	/* GIO 16-31 */
				/* -- Set Port Inversion: INVT/N-INVT -- */
				/* INVT (BIT=1), NO-INVT (BIT=0) */
				/* Except EXT-INT1, all are No-Invert */
	cur_register = GIOGetInv(bit15_0);
	out_register = (cur_register & ~(1<<GIO_BIT_0) & ~(1<<GIO_BIT_2) & ~(1<<GIO_BIT_3) & ~(1<<GIO_BIT_5) & ~(1<<GIO_BIT_6) & ~(1<<GIO_BIT_7) & ~(1<<GIO_BIT_8) & ~(1<<GIO_BIT_10) & ~(1<<GIO_BIT_11) & ~(1<<GIO_BIT_12) & ~(1<<GIO_BIT_13) & ~(1<<GIO_BIT_14) & ~(1<<GIO_BIT_9)) | (1<<GIO_BIT_1);
	GIOSetInv(bit15_0, out_register); /* GIO 0-15 */
	cur_register = GIOGetInv(bit31_16);
	out_register = cur_register & ~(1<<GIO_BIT_16) & ~(1<<GIO_BIT_17) & ~(1<<GIO_BIT_18) & ~(1<<GIO_BIT_19) & ~(1<<GIO_BIT_20) & ~(1<<GIO_BIT_21) & ~(1<<GIO_BIT_22) & ~(1<<GIO_BIT_23) & ~(1<<GIO_BIT_24) & ~(1<<GIO_BIT_25) & ~(1<<GIO_BIT_26) & ~(1<<GIO_BIT_27) & ~(1<<GIO_BIT_29) & ~(1<<GIO_BIT_30) & ~(1<<GIO_BIT_31) & ~(1<<GIO_BIT_28);
	GIOSetInv(bit31_16, out_register); /* GIO 16-31 */
				/* ---- Check Specific H/W?? ---- */
	GIOSetPortBitSet(bit15_0, (1<<GIO_BIT_14));
	cur_register = GIOGetPortBitSet(bit31_16);
	if ((cur_register & (1<<GIO_BIT_16)) != 1) {
		goto_func(1);
	}
	GIOSetPortBitClear(bit15_0, (1<<GIO_BIT_14));
	cur_register = GIOGetPortBitSet(bit31_16);
	if ((cur_register & (1<<GIO_BIT_16)) != 0) {
		goto_func(5);
	}
				/* ---- Check Specific H/W?? ---- */
	/* Setup all External Interrupts */
				/* -- Set Interrupt Port -- */
				/* Normal (BIT=0), Interrupt Port (BIT=1) */
	cur_register = GIOGetPortExternalIRQ();
	out_register = cur_register | (1<<GIO_BIT_0) | (1<<GIO_BIT_1) | (1<<GIO_BIT_2) | (1<<GIO_BIT_3);
	GIOSetPortExternalIRQ(out_register);
				/* -- Set Misc Port: -- */
				/* BIT_0: [0] Normal (O) */
				/*        [1] CCDC_Shutter->GIO11 (X) */
	cur_register = GIOGetMisc();
	out_register = cur_register & ~(1<<GIO_BIT_0);
	GIOSetMisc(out_register);
				/* -- Enable Interrupt Port -- */
				/* Disable=0, Enable=1 */
	cur_register = INTGetIQR0();
	out_register = cur_register | (1<<INT_BIT_0) | (1<<INT_BIT_1) | (1<<INT_BIT_2) | (1<<INT_BIT_3);
	INTSetIQR0(out_register);
				/* -- Set Interrupt Register (Clear) -- */
				/* Clear=1 (ARM can't set 0) */
	cur_register = INTGetREG0();
	out_register = cur_register | (1<<INT_BIT_0) | (1<<INT_BIT_1) | (1<<INT_BIT_2) | (1<<INT_BIT_3);
	INTSetREG0(out_register);
				/* -- Set the External Bus (BUS Mode) -- */
				/* 0400:0800 ~ 0FFF (CFC Common memory)  */
				/*   -->>  LAN & WLAN use this area!     */
				/* IO map (BIT=1), not Memory map (BIT=0) */
	cur_register = BUSGetBusMD();
	out_register = cur_register | (1<<BUS_BIT_0);
	BUSSetBusMD(out_register);
				/* -- Setup the CFC area attribute -- */
				/* Set normal: High for both LAN and WLAN. */
				/* Set Low while init the WLAN (not here). */
	GIOSetPortBitSet(bit15_0, (1<<GIO_BIT_11));   /* Pull High */
				/* Clear for CIS and I/O access (PCMCIA).  */
				/* New H/W Design for WLAN.                */
	GIOSetPortBitClear(bit15_0, (1<<GIO_BIT_13)); /* Pull Low */
				/* Reset Device Pin: Pull Low by Default */
	GIOSetPortBitClear(bit31_16, (1<<GIO_BIT_23)); /* Pull Low */

				/* Vincent Kuo: Stop  */

     /*
       Turn off timer just in case we're coming
       in from a warm boot.
     */
     outw(0, TIMER0_REGISTER_BASE);

     /*
       Turn off peripherals just in case we're coming 
       in from a warm boot.
     */
     outw(0x8000, VIDEOENC_REGISTER_BASE);
     outw(0, PREVIEW_REGISTER_BASE);
     outw(0x2601, DSPC_REGISTER_BASE);             // put DSP to sleep
     outw(0x6011, DSPC_REGISTER_BASE + 4);
     outw(1, CCDC_REGISTER_BASE);
     outw(0, BURSTC_REGISTER_BASE);
     //outw(0, OSD_REGISTER_BASE);
     outw(0, DSC21_UART0_MSR);
     outw(0, DSC21_UART1_MSR);
     outw(0, SERIAL0_REGISTER_BASE + 4);
     outw(0, SERIAL1_REGISTER_BASE + 4);
#endif /* CONFIG_ARCH_DSC21 */
				/* ---- Check Specific H/W?? ---- */
     GIOSetPortBitSet(bit15_0, (1<<GIO_BIT_14));
     cur_register = GIOGetPortBitSet(bit31_16);
     if ((cur_register & (1<<GIO_BIT_16)) != 1) {
	     goto_func(10);
     }
     GIOSetPortBitClear(bit15_0, (1<<GIO_BIT_14));
     cur_register = GIOGetPortBitSet(bit31_16);
     if ((cur_register & (1<<GIO_BIT_16)) != 0) {
	     goto_func(50);
     }
				/* ---- Check Specific H/W?? ---- */
}
