/*
 * io-dsc21.c:
 *         DSC21 I/O functions, copied from ATMEL code
 *
 * copyright:
 *         (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 * author: Steve Johnson <stevej@ridgerun.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/arch/iram.h>

static __inline__ void do_outw(unsigned int reg, const void* buf, unsigned int count)
{
	register const unsigned short* wbuf = (const unsigned short*) buf;
	while(count--)
		outw(*wbuf++, reg);
}

static __inline__ void do_inw(unsigned int reg, void* buf, unsigned int count)
{
	register unsigned short* wbuf = (unsigned short*) buf;
	while(count--)
		*wbuf++  = inw(reg);
}


IRAM_TEXT void outsw(unsigned to_reg, const void *from, int len_in_words)
{
	do_outw(to_reg, from, len_in_words);
}

IRAM_TEXT void outswb(unsigned  to_reg, const void *from, int len_in_bytes)
{
	do_outw(to_reg, from, len_in_bytes/2);
}



IRAM_TEXT void insw(unsigned from_port, void *to, int len_in_words)
{
	do_inw(from_port, to, len_in_words);
}

IRAM_TEXT void  inswb(unsigned from_port, void *to, int len_in_bytes)
{
	do_inw(from_port, to, len_in_bytes/2);
}


void arch_hard_reset (void)
{
    /*
      This can be accomplished by setting the watch dog timer to a very
      small number and having it cause an ARM reset.
    */

	while (1);
}
