/* ld script to make ARM Linux kernel
 * taken from the i386 version by Russell King
 * Written by Martin Mares <mj@atrey.karlin.mff.cuni.cz>
 */
OUTPUT_ARCH(arm)
ENTRY(stext)
SECTIONS
{

  . = 0x0;
  .vectors : {
  }

	. = 0x08800000;
	.init : {			/* Init code and data		*/
		stext = .;
    _stext = .;
		__init_begin = .;
			*(.text.init)
		__proc_info_begin = .;
			*(.proc.info)
		__proc_info_end = .;
		__arch_info_begin = .;
			*(.arch.info)
		__arch_info_end = .;
			*(.data.init)
		. = ALIGN(16);
		__setup_start = .;
			*(.setup.init)
		__setup_end = .;
		__initcall_start = .;
			*(.initcall.init)
		__initcall_end = .;
		. = ALIGN(4096);
		__init_end = .;
	}

	/DISCARD/ : {			/* Exit code and data		*/
		*(.text.exit)
		*(.data.exit)
		*(.exitcall.exit)
	}

	.text : {			/* Real text segment		*/
		_text = .;		/* Text and read-only data	*/
			*(.text)
			*(.fixup)
			*(.gnu.warning)
			*(.text.lock)	/* out-of-line lock text */
			*(.rodata)
			*(.kstrtab)
		. = ALIGN(16);
		__start___ex_table = .;	/* Exception table		*/
			*(__ex_table)
		__stop___ex_table = .;

		__start___ksymtab = .;	/* Kernel symbol table		*/
			*(__ksymtab)
		__stop___ksymtab = .;

		*(.got)			/* Global offset table		*/

		_etext = .;		/* End of text section		*/
    etext = .;
	}

  __RONLY_end = ABSOLUTE(.); /* Please See Appendix A */

	. = ALIGN(8192);
  . = ALIGN(8192);
	.data : {
     __data_start = ABSOLUTE(.);	/* DATA */
		/*
		 * first, the init task union, aligned
		 * to an 8192 byte boundary.
		 */
		*(.init.task)

		/*
		 * then the cacheline aligned data
		 */
		. = ALIGN(32);
		*(.data.cacheline_aligned)

		/*
		 * and the usual data section
		 */
		*(.data)
		CONSTRUCTORS

		__data_end = ABSOLUTE(.);
		_edata = .;
	}

  __DATASEG_size = SIZEOF(.data); /* Please See Appendix A */

  .iram 0x00000004 : AT((LOADADDR(.data)+SIZEOF(.data)+256-1) &~(256-1))
  {
  __iram_start = ABSOLUTE(.);	 /* IRAM usage start point */
  *(.text.iram);
  *(.data.iram);
  *(.bss.iram);
  __iram_end = ABSOLUTE(.);		 /* IRAM usage start point */
/*   . = 0x8000;                   Test that the stack isn't > 4K */
  }
  init_kernel_stack = 0x00007800;   /* IRAM, top 4K (0x7000-0x8000) */

	. = (LOADADDR(.iram)+SIZEOF(.iram)+256-1) &~(256-1);  

	.bss : {
		. = ALIGN(32);    /* for SDRAM burst mode use */
		__bss_start = .;	/* BSS				*/
		*(.bss)
		*(COMMON)
  	. = ALIGN(0x1000);
		_end = . ;
	}

  __BSSSEG_end = ABSOLUTE(.); /* Please See Appendix A */


					/* Stabs debugging sections.	*/
	.stab 0 : { *(.stab) }
	.stabstr 0 : { *(.stabstr) }
	.stab.excl 0 : { *(.stab.excl) }
	.stab.exclstr 0 : { *(.stab.exclstr) }
	.stab.index 0 : { *(.stab.index) }
	.stab.indexstr 0 : { *(.stab.indexstr) }
	.comment 0 : { *(.comment) }
}

/*
  ----------------
  -- Appendix A --
  ----------------

  The __RONLY_end number represents where the last
  *ReadOnly* (.text,.rodata,etc) section byte fell. The 
  significance of this marker is that this is where
  the top level Makefile will adjust the .data section to for 
  the purpose of flashing the system. We want a flashable
  unit that consists of all the ReadOnly sections +
  .data. At boot time the head-arm-${machine}.S file will move
  the .data from the flash space (represented by
  __RONLY_end) to the SDRAM location (represented
  by __data_start) where it really needs to reside for
  running.

  Summary -- at program invoke, the .data section needs 
             specific default values and hence must reside 
             in FLASH, yet the .data section is read-writable 
             by the program and hence needs to reside in 
             SDRAM when running. So, in *early* boot, move
             .data from flash to SDRAM and then run as normal.
*/
