/* krom.c: Kernel space ROM disk driver
 * Was z2ram - Amiga pseudo-driver to access 16bit-RAM in ZorroII space
 * Copyright (C) 1994 by Ingo Wilken (Ingo.Wilken@informatik.uni-oldenburg.de)
 * Copyright (C) 1997 by D. Jeff Dionne <jeff@pfnet.com>
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  This software is provided "as is" without express or
 * implied warranty.
*/


/* Major 31 is ROM devices */
#define MAJOR_NR    31

#include <linux/config.h>
#include <linux/major.h>
#include <linux/malloc.h>
/* all this stuff needs to go in blk.h */
#define DEVICE_NAME "Kernel ROM disk"
#define DEVICE_REQUEST do_krom_request
#define DEVICE_NR(device) (MINOR(device))
#define DEVICE_ON(device)
#define DEVICE_OFF(device)

#include <linux/blk.h>

#if 0
#include <linux/init.h>
#endif

#if defined(MODULE)
#include <linux/module.h>
#endif

#include <asm/bitops.h>

#define TRUE                  (1)
#define FALSE                 (0)



/* internal, cat on the end of kernel, or seperate fixed adderess romfs. */

#ifdef CONFIG_PILOT
#undef INTERNAL_ROMARRAY
#define CAT_ROMARRAY
#undef FIXED_ROMARRAY
#endif

#ifdef CONFIG_SHGLCORE
#undef INTERNAL_ROMARRAY
#undef CAT_ROMARRAY
#define FIXED_ROMARRAY 0xa0000
#endif

#ifdef INTERNAL_ROMARRAY
#include "testromfs.c"
#endif

#ifdef CAT_ROMARRAY
unsigned char *romarray;
extern char __data_rom_start;
extern char _edata;
extern char __data_start;
#endif

#ifdef FIXED_ROMARRAY
unsigned char *romarray = (char *)(FIXED_ROMARRAY);
#endif

struct arena_t {
	unsigned long address; /* Address of memory arena in main memory */
	unsigned long length;  /* Length of memory arena. If zero, try to get size from romfs header */
	
} arena[] = {
	{romarray, 0},
};



static int krom_blocksizes[3] = { 1024, 1024, 1024 };
static int krom_sizes[3]      = {    0,    0,    0 };

static int current_device   = -1;

static void
do_krom_request( void )
{
  unsigned long start;
  unsigned long len;
#if 0
  printk( KERN_ERR DEVICE_NAME ": request\n");
#endif
  while ( TRUE ) {
    INIT_REQUEST;
    
    /* sectors are 512 bytes */
    start = CURRENT->sector << 9;
    len  = CURRENT->current_nr_sectors << 9;

    if (start + len > (krom_sizes[0] << 10)) {
      printk( KERN_ERR DEVICE_NAME ": bad access: block=%ld, count=%ld\n",
	      CURRENT->sector,
	      CURRENT->current_nr_sectors);
      end_request( FALSE );
      continue;
    }

    if ( ( CURRENT->cmd != READ ) 
#if 0
	 && ( CURRENT->cmd != WRITE ) 
#endif
	 ) {
      printk( KERN_ERR DEVICE_NAME ": bad command: %d\n", CURRENT->cmd );
      end_request( FALSE );
      continue;
    }

    if ( CURRENT->cmd == READ ) 
      memcpy( CURRENT->buffer, &romarray[start], len );
#if 0
    else
      memcpy( &romarray[start], CURRENT->buffer, len );
#endif
    end_request( TRUE );
  }
}

#ifdef MAGIC_ROM_PTR
static int
krom_romptr( struct inode *inode, struct file *filp, struct vm_area_struct * vma)
{
  vma->vm_start = (unsigned long)&romarray[vma->vm_offset];
  return 0;
}
#endif

static int
krom_open( struct inode *inode, struct file *filp )
{
  int device;

  device = DEVICE_NR( inode->i_rdev );
#if 0
  printk( KERN_ERR DEVICE_NAME ": open: %d\n", device );
#endif
  /* This is support for more minor devices.  Later */
  if ( current_device != -1 && current_device != device ) {
    return -EBUSY;
  }

  if ( current_device == -1 ) {
    current_device = device;
  }

#if defined(MODULE)
  MOD_INC_USE_COUNT;
#endif
  return 0;
}

static void 
krom_release( struct inode *inode, struct file *filp )
{
#if 0
  printk( KERN_ERR DEVICE_NAME ": release: %d\n", current_device );
#endif
  if ( current_device == -1 )
    return;     

  sync_dev( inode->i_rdev );

#if defined(MODULE)
  MOD_DEC_USE_COUNT;
#endif

  return;
}

static struct file_operations krom_fops =
{
  NULL,                   /* lseek - default */
  block_read,             /* read - general block-dev read */
  block_write,            /* write - general block-dev write */
  NULL,                   /* readdir - bad */
  NULL,                   /* poll */
  NULL,                   /* ioctl */
  NULL,
  krom_open,                /* open */
  krom_release,             /* release */
  block_fsync,             /* fsync */
  NULL,			/* fasync */
  NULL,			/* check media change */
  NULL,			/* revalidate */
  krom_romptr,		  /* romptr */
};

int
krom_init( void )
{
  if ( register_blkdev( MAJOR_NR, DEVICE_NAME, &krom_fops )) {
    printk( KERN_ERR DEVICE_NAME ": Unable to get major %d\n",
            MAJOR_NR );
    return -EBUSY;
  }
 
#ifdef INTERNAL_ROMARRAY
  krom_sizes[0] = sizeof(romarray) >> 10;
#endif

#ifdef CAT_ROMARRAY
  krom_sizes[0] = (((1<<10) - 1) + *(volatile unsigned long *)
                   (&__data_rom_start + (&_edata - &__data_start) + 8)) >> 10;

  romarray = &__data_rom_start + (&_edata - &__data_start);
#endif

#ifdef FIXED_ROMARRAY
  krom_sizes[0] = (((1<<10) - 1) + *(volatile unsigned long *)(romarray + 8)) >> 10;
#endif

  printk("KROM block (c) 1998 D Jeff Dionne, SHG Ltd [%dk] at %.8x\n",
         krom_sizes[0],(unsigned int)romarray);
  blk_dev[ MAJOR_NR ].request_fn = DEVICE_REQUEST;
  blksize_size[ MAJOR_NR ] = krom_blocksizes;
  blk_size[ MAJOR_NR ] = krom_sizes;
  
  ROOT_DEV = MKDEV(MAJOR_NR,0);

  return 0;
}

#if defined(MODULE)

int
init_module( void )
{
  int error;
    
  error = krom_init();
  if ( error == 0 ) {
    printk( KERN_INFO DEVICE_NAME ": loaded as module\n" );
  }
    
  return error;
}

void
cleanup_module( void )
{
  if ( unregister_blkdev( MAJOR_NR, DEVICE_NAME ) != 0 )
    printk( KERN_ERR DEVICE_NAME ": unregister of device failed\n");

  if ( current_device != -1 ) {
    /* free whatever we resources remain here */
  }

  return;
}
#endif
