/****************************************************************************/

/*
 *	hifn7901.h   -- Definitions for Hi/fn 7901 encryption engine.
 *
 *	(C) Copyright 2000, Greg Ungerer (gerg@lineo.com)
 *
 *	This program is a part of the Linux kernel, and may be freely
 *	copied under the terms of the GNU General Public License (GPL),
 *	version 2, or at your option any later version.
 */

/****************************************************************************/
#ifndef _HIFN7901_H_
#define	_HIFN7901_H_
/****************************************************************************/

/*
 *	Address offsets in 7901 memory mapping.
 */
#define	HIFN7901_REGBASE	0		/* Register Base Address */

#define	HIFN7901_MEMBASE	0x2000		/* Memory Base Address */
#define	HIFN7901_CMDD		0x2000		/* Command Descriptors */
#define	HIFN7901_SRCD		0x2040		/* Source Decsriptors */
#define	HIFN7901_RESD		0x2080		/* Result Descriptors */
#define	HIFN7901_DSTD		0x20c0		/* Destination Descriptors */
#define	HIFN7901_PKTDATA	0x2100		/* Packet data */
#define	HIFN7901_MEMMAX		0x4000		/* Maximum address offset */

#define	HIFN7901_MEMSIZE	0x2000		/* Size of internal memory */


/*
 *	Register offsets within register table.
 */
#define	HIFN7901_GC		0x0004		/* General Configuration */
#define	HIFN7901_ID		0x0008		/* Chip ID */

#define	HIFN7901_HCI		0x0080		/* Host Command Index */
#define	HIFN7901_HSI		0x0084		/* Host Source Index */
#define	HIFN7901_HRI		0x0088		/* Host Result Index */
#define	HIFN7901_HDI		0x008c		/* Host Destination Index */
#define	HIFN7901_PKTS		0x0090		/* Packet Status */
#define	HIFN7901_PKTIE		0x0094		/* Packet Interrupt Enable */
#define	HIFN7901_PKTC		0x0098		/* Packet Configuration */

#define	HIFN7901_PBA		0x0100		/* Public Base Address */
#define	HIFN7901_POL		0x0104		/* Public Operand Length */
#define	HIFN7901_PO		0x0108		/* Public Operand */
#define	HIFN7901_PS		0x010c		/* Public Status */
#define	HIFN7901_PIE		0x0110		/* Public Interrupt Enable */
#define	HIFN7901_RNGC		0x0114		/* RNG Configuration */
#define	HIFN7901_RNGD		0x0118		/* RNG Data */


/*
 *	Defines to support the General Configuration register.
 */
#define	HIFN7901_GC_RESET	0x00000001	/* Reset the 7901 */


/*
 *	Defines to support Chip ID register.
 */
#define	HIFN7901_ID_PRODREV	0x00000800	/* Product/Revision ID 7901 */


/*
 *	Define Random Number Generator Configuration flags.
 */
#define	HIFN7901_RNGC_ENABLE	0x00000001	/* Enable RNG */


/*
 *	Descriptor defines.
 */
struct hifn7901desc {
	unsigned long		cmd;
	unsigned long		ptr;
};

#define	HIFN7901_DESC_LAST	0x20000000	/* Last fragment */
#define	HIFN7901_DESC_MASKDONE	0x02000000	/* Mask done status */

#define	HIFN7901_DESC_LENMASK	0x0000ffff	/* Mask for buffer length */


/*
 *	Base Command Structure.
 */
struct hifn7901cmd {
	unsigned long		cmd;
	unsigned short		scount;
	unsigned short		dcount;
};

#define	HIFN7901_CMD_SES(x)	((((x) & 0xff) << 8) | (((x) >> 8) & 0xf))

#define	HIFN7901_CMD_COMP	0x00010000	/* Enable compression */
#define	HIFN7901_CMD_PAD	0x00020000	/* Enable padding */
#define	HIFN7901_CMD_MAC	0x00040000	/* Enable mac processing */
#define	HIFN7901_CMD_ENCRYPT	0x00080000	/* Enable encryption */
#define	HIFN7901_CMD_IGNDSTCNT	0x01000000	/* Ignore destination count */

#define	HIFN7901_CMD_ENCODE	0x00000000	/* Encode basic command */
#define	HIFN7901_CMD_DECODE	0x00200000	/* Decode basic command */
#define	HIFN7901_CMD_READRAM	0x00400000	/* Read RAM basic command */
#define	HIFN7901_CMD_WRITERAM	0x00600000	/* Write RAM basic command */

#define	HIFN7901_SWAPB(x)	((((x) & 0xff) << 8) | (((x) >> 8) & 0xff))


/*
 *	Compress Command Structure.
 */
struct hifn7901comp {
	unsigned short		cmd;
	unsigned short		hdrcnt;
	unsigned short		srccnt;
	unsigned short		reserved;
};

#define	HIFN7901_COMP_LZS	0x0001		/* LZS compression */
#define	HIFN7901_COMP_MPPC	0x0101		/* MPPC compression */
#define	HIFN7901_COMP_STRIP0	0x0400		/* LZS strip 0 mode */
#define	HIFN7901_COMP_RESTART	0x0400		/* MPPC restart mode */
#define	HIFN7901_COMP_UPDTHIST	0x0800		/* Update decompress history */
#define	HIFN7901_COMP_CLEARHIST	0x1000		/* Clear compress history */


/*
 *	Pad Command Structure.
 */
struct hifn7901pad {
	unsigned short		cmd;
	unsigned short		srccnt;
};

#define	HIFN7901_PAD_MODE0	0x00000000	/* Mode 0 algorithm */
#define	HIFN7901_PAD_MODE1	0x01000000	/* Mode 1 algorithm */
#define	HIFN7901_PAD_MODE2	0x02000000	/* Mode 2 algorithm */
#define	HIFN7901_PAD_MODE3	0x03000000	/* Mode 3 algorithm */

#define	HIFN7901_PAD_CNTMODEM	0x00200000	/* Decrement count mode */


/*
 *	MAC Command Structure.
 */
struct hifn7901mac {
	unsigned short		cmd;
	unsigned short		hdrcnt;
	unsigned short		srccnt;
	unsigned short		reserved;
};

#define	HIFN7901_MAC_COMP2PAD	0x0000		/* Compress->MAC->Padding */
#define	HIFN7901_MAC_PAD2ENC	0x0001		/* Padding->MAC->Encryption */
#define	HIFN7901_MAC_AEBD	0x0002		/* After encode,before decode */
#define	HIFN7901_MAC_NEWKEY	0x0008		/* New key */
#define	HIFN7901_MAC_CNTMODE	0x0020		/* Continue counting */
#define	HIFN7901_MAC_SHA	0x0000		/* SHA mode */
#define	HIFN7901_MAC_MD5	0x0100		/* MD5 mode */
#define	HIFN7901_MAC_HMAC	0x0000		/* HMAC mode */
#define	HIFN7901_MAC_SSL	0x0400		/* SSL MAC mode */
#define	HIFN7901_MAC_HASH	0x0800		/* HASH only */
#define	HIFN7901_MAC_TRUNC	0x1000		/* Truncate to 12 bytes */
#define	HIFN7901_MAC_RESULT	0x2000		/* Generate result */
#define	HIFN7901_MAC_INSERT	0x4000		/* Insert MAC in data */


/*
 *	Encryption Command Structure.
 */
struct hifn7901enc {
	unsigned short		cmd;
	unsigned short		hdrcnt;
	unsigned short		srccnt;
	unsigned short		reserved;
};

#define	HIFN7901_ENC_NEWKEY	0x0008		/* New key */
#define	HIFN7901_ENC_NEWIV	0x0010		/* New initialization vector */
#define	HIFN7901_ENC_CNTMODE	0x0020		/* Continue counting */
#define	HIFN7901_ENC_DES	0x0000		/* DES encryption */
#define	HIFN7901_ENC_3DES	0x0100		/* Triple DES encryption */
#define	HIFN7901_ENC_RC4	0x0200		/* RC4 encryption */
#define	HIFN7901_ENC_ECB	0x0000		/* ECB DES mode */
#define	HIFN7901_ENC_CBC	0x0800		/* CBC DES mode */
#define	HIFN7901_ENC_CFB64	0x1000		/* CFB-64 DES mode */
#define	HIFN7901_ENC_OFB	0x1800		/* CFB DES mode */
#define	HIFN7901_ENC_CLEAR	0x4000		/* Clear context */


/*
 *	Base Result Structure.
 */
struct hifn7901res {
	unsigned long		cmd;
	unsigned short		srccnt;
	unsigned short		dstcnt;
};

#define	HIFN7901_RES_SES	HIFN7901_CMD_SES
#define	HIFN7901_RES_DSTOVERRUN	0x0002000	/* Destination overrun */


/*
 *	Compression Result Structure.
 */
struct hifn7901compres {
	unsigned short		cmd;
	unsigned short		crc;
};

#define	HIFN7901_COMPRES_NZERO	0x0100		/* Source count non-zero */
#define	HIFN7901_COMPRES_LZSEND	0x0200		/* LZS reached end marker */
#define	HIFN7901_COMPRES_RSTART	0x0400		/* MPPC restart */
#define	HIFN7901_COMPRES_LCBMSK	0x00ff		/* LCB check byte mask */


/*
 *	MAC Result Structure.
 */
struct hifn7901macres {
	unsigned long		cmd;
};

#define	HIFN7901_MACRES_NZERO	0x01000000	/* Source count non-zero */
#define	HIFN7901_MACRES_MISCMP	0x02000000	/* Mis-compare result */


/*
 *	Encryption Result Structure.
 */
struct hifn7901encres {
	unsigned long		cmd;
};

#define	HIFN7901_ENCRES_NZERO	0x01000000	/* Source count non-zero */


/*
 *	Packet Status Register support flags.
 */
#define	HIFN7901_PKTS_DLAST(x)	(((x) >> 24) & 0x1)
#define	HIFN7901_PKTS_DINDEX(x)	(((x) >> 20) & 0xf)
#define	HIFN7901_PKTS_RINDEX(x)	(((x) >> 14) & 0xf)
#define	HIFN7901_PKTS_SINDEX(x)	(((x) >> 8) & 0xf)
#define	HIFN7901_PKTS_CINDEX(x)	(((x) >> 2) & 0xf)

#define	HIFN7901_PKTS_DSTDONE	0x00080000	/* Destination done */
#define	HIFN7901_PKTS_DSTLAST	0x00040000	/* Destination last */
#define	HIFN7901_PKTS_RESDONE	0x00002000	/* Result done */
#define	HIFN7901_PKTS_RESLAST	0x00001000	/* Result last */
#define	HIFN7901_PKTS_SRCDONE	0x00000080	/* Source done */
#define	HIFN7901_PKTS_SRCLAST	0x00000040	/* Source last */
#define	HIFN7901_PKTS_CMDDONE	0x00000002	/* Command done */
#define	HIFN7901_PKTS_CMDLAST	0x00000001	/* Command last */

#define	HIFN7901_PKTS_ALLDONE	0x00082082	/* All done bits */


/*
 *	Packet Interrupt Enable Register defines.
 */
#define	HIFN7901_PKTIE_CMDLAST	0x00000001	/* Command last interrupt */
#define	HIFN7901_PKTIE_CMDDONE	0x00000002	/* Command done interrupt */
#define	HIFN7901_PKTIE_SRCLAST	0x00000040	/* Source done interrupt */
#define	HIFN7901_PKTIE_SRCDONE	0x00000080	/* Source done interrupt */
#define	HIFN7901_PKTIE_RESLAST	0x00001000	/* Result done interrupt */
#define	HIFN7901_PKTIE_RESDONE	0x00002000	/* Result done interrupt */
#define	HIFN7901_PKTIE_DSTLAST	0x00040000	/* Destination done interrupt */
#define	HIFN7901_PKTIE_DSTDONE	0x00080000	/* Destination done interrupt */


/*
 *	Packet Configuration Register defines.
 */
#define	HIFN7901_PKTC_COMP	0x00000004	/* Multiple session compres */
#define	HIFN7901_PKTC_ENCRYPT	0x00000002	/* 128 byte encrypt context */


/****************************************************************************/
#endif /* _HIFN7901_H_ */
