/*****************************************************************************/

/*
 *	lcdtxt.c -- driver for a text based LCD display
 *
 *	(C) Copyright 2000, Greg Ungerer (gerg@lineo.com)
 *	(C) Copyright 2000, Lineo Inc. (www.lineo.com) 
 */

/*****************************************************************************/

#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/delay.h>
#include <asm/param.h>
#include <asm/coldfire.h>

#undef CONFIG_OLDMASK
#include <asm/mcfsim.h>

/*****************************************************************************/

/*
 *	Define driver major number.
 */
#define	LCDTXT_MAJOR	120

/*
 *	Define LCD hardware address.
 */
#define	LCDTXT_ADDR	0x30800000

/*
 *	Local copy of display setting of LCD.
 */
unsigned char	lcdtxt_display;

/*****************************************************************************/

int lcdtxt_open(struct inode *inode, struct file *filp)
{
#if 0
	printk("lcdtxt_open()\n");
#endif

	return(0);
}

/*****************************************************************************/

void lcdtxt_release(struct inode *inode, struct file *filp)
{
#if 0
	printk("lcdtxt_close()\n");
#endif
}

/*****************************************************************************/

void lcdtxt_ceol(void)
{
	volatile unsigned char	*lcdp;
	unsigned char		cur, max;
	int			i;

#if 0
	printk("lcdtxt_ceol()\n");
#endif

	lcdp = (volatile char *) LCDTXT_ADDR;

	cur = lcdp[0] & 0x40;

	for (i = 0; (i < 16); i++) {
		lcdp[1] = ' ';
		udelay(2000);
	}

	lcdp[0] = 0x80 | cur;
	udelay(5000);
}

/*****************************************************************************/

int lcdtxt_write(struct inode *inode, struct file *filp, const char *buf, int count)
{
	volatile char	*lcdp;
	char		*dp, c;
	int		num;

#if 0
	printk("lcdtxt_write(buf=%x,count=%d)\n", (int) buf, count);
#endif

	dp = (char *) buf;
	lcdp = (volatile char *) LCDTXT_ADDR;

	for (num = count; (num > 0); num--) {
		c = get_user(dp++);

		switch (c) {
		case '\n':
			lcdp[0] = 0x80 | ((~lcdp[0]) & 0x40);
			udelay(5000);
			break;
		case '\r':
			lcdp[0] = 0x02;
			udelay(5000);
			break;
		case '\f':
			lcdp[0] = 0x01;
			udelay(5000);
			break;
		case 0x1:	/* CTRL-A  -- toggle cursor "on/off" */
			lcdtxt_display ^= 0x02;
			lcdp[0] = lcdtxt_display;
			udelay(5000);
			break;
		case 0x2:	/* CTRL-B  -- toggle blinking curser */
			lcdtxt_display ^= 0x01;
			lcdp[0] = lcdtxt_display;
			udelay(5000);
			break;
		case 0x5:	/* CTRL-E  -- clear to end of line */
			lcdtxt_ceol();
			break;
		case 0x6:	/* CTRL-F  -- advance curser 1 position */
			c = (lcdp[0] & 0x7f) + 1;
			lcdp[0] = 0x80 | c;
			udelay(2000);
			break;
		default:
			lcdp[1] = c;
			udelay(2000);
			break;
		}
	}

	return(count);
}

/*****************************************************************************/

int lcdtxt_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
	int	rc = 0;

#if 0
	switch (cmd) {
	default:
		rc = -EINVAL;
		break;
	}
#endif

	return(rc);
}

/*****************************************************************************/

/*
 *	Exported file operations structure for driver...
 */

struct file_operations	lcdtxt_fops = {
	NULL,		/* lseek */
	NULL,		/* read */
	lcdtxt_write,	/* write */
	NULL,		/* readdir */
	NULL,		/* poll */
	lcdtxt_ioctl,	/* ioctl */
	NULL,		/* mmap */
	lcdtxt_open,	/* open */
	lcdtxt_release,	/* release */
	NULL,		/* fsync */
	NULL,		/* fasync */
	NULL,		/* check_media_change */
	NULL		/* revalidate */
};

/*****************************************************************************/

void lcdtxt_init(void)
{
	volatile unsigned char	*lcdp;
	int			rc;

	/* Register lcdtxt as character device */
	rc = register_chrdev(LCDTXT_MAJOR, "lcdtxt", &lcdtxt_fops);
	if (rc < 0) {
		printk(KERN_WARNING "LCDTXT: can't get major %d\n",
			LCDTXT_MAJOR);
		return;
	}

	printk("LCDTXT: Copyright (C) 2000, Greg Ungerer "
		"(gerg@lineo.com)\n");

	/* Setup CS4 for our external hardware */
	*((volatile unsigned short *) (MCF_MBAR + MCFSIM_CSAR4)) = 0x3080;
	*((volatile unsigned long *) (MCF_MBAR + MCFSIM_CSMR4)) = 0x000f0001;
	*((volatile unsigned short *) (MCF_MBAR + MCFSIM_CSCR4)) = 0x3d40;
	/*udelay(500000);*/
	udelay(5000);

	/* LCD initialization sequence */
	lcdp = (volatile unsigned char *) LCDTXT_ADDR;

	/* Function set */
	*lcdp = 0x38;
	udelay(5000);

	/* Display on */
	lcdtxt_display = 0x0f;
	*lcdp = lcdtxt_display;
	udelay(5000);

	/* Display clear */
	*lcdp = 0x01;
	udelay(5000);

	/* Entry mode set */
	*lcdp = 0x06;
	udelay(5000);
}

/*****************************************************************************/
