/*
 *****************************************************
 *
 * Copyright (C) 2001 RidgeRun, Inc.
 * Author: Steve Kranz <skranz@ridgerun.com> <Oct 2001>
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS  PROVIDED  ``AS  IS''  AND   ANY  EXPRESS  OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Please report all bugs/problems to the author or <support@dsplinux.net>
 *
 *  File: drivers/char/serial_orion.c
 *
 *  Description:
 *    This implementation was leveraged from the standard
 *    serial.c Linux file. This version was generated for
 *    the RidgeRun DSPLinux distribution on Sept, 2001
 *    <skranz@ridgerun.com>
 *
 *  History:
 *    Sept 2001 -- Created and ported to the Orion
 *                 EVM board (C5472). <skranz@ridgerun.com>
 *
 ***************************************************** 
 */

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/interrupt.h>
#include <linux/tty.h>
#include <linux/tty_flip.h>
#include <linux/serial.h>
#include <linux/config.h>
#include <linux/major.h>
#include <linux/string.h>
#include <linux/fcntl.h>
#include <linux/ptrace.h>
#include <linux/ioport.h>
#include <linux/mm.h>
#include <asm/system.h>
#include <asm/io.h>
#include <asm/segment.h>
#include <asm/bitops.h>

static unsigned long IER_cached=0, LCR_cached=0, FCR_cached=0;
static char *serial_name = "Serial driver";
static char *serial_version = "4.13p1";
DECLARE_TASK_QUEUE(tq_serial);
struct tty_driver serial_driver, callout_driver;
static int serial_refcount;
#define SERIAL_TYPE_NORMAL  1
#define SERIAL_TYPE_CALLOUT 2
#define WAKEUP_CHARS        256
#define SERIAL_PARANOIA_CHECK
#define SERIAL_DO_RESTART
#if defined(MODULE) && defined(SERIAL_DEBUG_MCOUNT)
  #define DBG_CNT(s) printk("(%s): [%x] refc=%d, serc=%d, ttyc=%d -> %s\n", \
  kdevname(tty->device), (info->flags), serial_refcount,info->count,tty->count,s)
#else
  #define DBG_CNT(s)
#endif
#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif

static void autoconfig(struct async_struct * info);
static void change_speed(struct async_struct *info);
/*
 * tmp_buf is used as a temporary buffer by serial_write.  We need to
 * lock it in case the memcpy_fromfs blocks while swapping in a page,
 * and some other program tries to do a serial write at the same time.
 * Since the lock will only come under contention when the system is
 * swapping and available memory is low, it makes sense to share one
 * buffer across all the serial ports, since it significantly saves
 * memory if large numbers of serial ports are open.
 */
static unsigned char *tmp_buf = 0;
static struct semaphore tmp_buf_sem = MUTEX;

/*
 * This is used to figure out the divisor speeds and the timeouts
 */
static int baud_table[] = {
  0, 50, 75, 110, 134, 150, 200,
  300, 600, 1200, 1800, 2400, 4800,
  9600, 19200, 38400, 57600, 115200, 0
};

#ifdef CONFIG_ARCH_ORION
// ------------------------------------------
//       -- Begin; Platform Specifics --
// ------------------------------------------
//   Note: These definitions and routines will
//         surely need to change if porting this
//         driver to another platform.
//
#define IRQ_UART0 6
#define BAUD 115200
#define UART0_FIRST_IO 0xffff1000
#define UARTn_IO_RANGE 0x48
#define STD_COM_FLAGS (ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST)
struct async_struct rs_table[] = {
  { /* ttyS0 */                \
    magic:     0,              \
    baud_base: BAUD,           \
    port:      UART0_FIRST_IO, \
    irq:       IRQ_UART0,      \
    flags:     STD_COM_FLAGS,  \
  }
  // { /* ttyS1 */ }
  // { /* ttyS2 */ }
  // { /* etc   */ }
};
#define NR_PORTS (sizeof(rs_table)/sizeof(struct async_struct))
static struct tty_struct *serial_table[NR_PORTS];
static struct termios *serial_termios[NR_PORTS];
static struct termios *serial_termios_locked[NR_PORTS];
#define UART_RHR             ((unsigned long)0xffff1000)
#define UART_THR             ((unsigned long)0xffff1004)
#define UART_RFCR            ((unsigned long)0xffff1008)
#define UART_TFCR            ((unsigned long)0xffff1008)
#define UART_LCR             ((unsigned long)0xffff1010)
#define UART_LSR             ((unsigned long)0xffff1014)
#define UART_SSR             ((unsigned long)0xffff1018)
#define UART_IER             ((unsigned long)0xffff1024)
#define UART_EFR             ((unsigned long)0xffff102c)
#define UART_ISR             ((unsigned long)0xffff1028)
#define UART_DIV_BIT_RATE    ((unsigned long)0xffff1048)
#define UART_MDR             ((unsigned long)0xffff1054)
#define UART_RX_FIFO_NOEMPTY 0x00000001
#define UART_LSR_TREF        0x00000020
#define UART_SSR_TXFULL      0x00000001
#define UART_LCR_BOC         0x00000040
#define UART_LCR_7bits       0x00000002
#define UART_LCR_8bits       0x00000003
#define UART_LCR_2stop       0x00000004
#define UART_LCR_ParEn       0x00000008
#define UART_LCR_ParOdd      0x00000000
#define UART_FCR_FTL         0x00000000
#define UART_FCR_FIFO_EN     0x00000001
#define UART_FCR_TX_CLR      0x00000002
#define UART_FCR_RX_CLR      0x00000004
#define UART_IER_recvInt     0x00000001
#define UART_IER_xmitInt     0x00000002
#define UART_IER_IntPair     0x00000003
#define BAUD_115200          0x00000001
#define BAUD_57600           0x00000002
#define BAUD_38400           0x00000003
#define BAUD_19200           0x00000006
#define BAUD_9600            0x0000000C
#define MODE8_NOPARITY_STOP1 0x00000003
#define MDR_RESET_MODE       0x00000007
#define MDR_UART_MODE        0x00000000

/******************************
 Routine:
 Description:
 ******************************/
static inline unsigned long serial_in(struct async_struct *info, unsigned long offset)
{
  return inl(offset);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_out(struct async_struct *info,
                              unsigned long offset,
                              unsigned long value)
{
  outl(value,offset);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_reset_chip(struct async_struct *info)
{
  volatile long i;
  serial_out(info, UART_MDR, MDR_RESET_MODE);
  for (i=0; i<1000; i++) {}
  serial_out(info, UART_MDR, MDR_UART_MODE);
  for (i=0; i<1000; i++) {}
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_disable_tx_int(struct async_struct *info)
{
  
  IER_cached &= ~UART_IER_xmitInt;
  serial_out(info, UART_IER, IER_cached);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_disable_rx_int(struct async_struct *info)
{
  IER_cached &= ~UART_IER_recvInt;
  serial_out(info, UART_IER, IER_cached);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_enable_tx_int(struct async_struct *info)
{
  IER_cached |= UART_IER_xmitInt;
  serial_out(info, UART_IER, IER_cached);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_enable_rx_int(struct async_struct *info)
{
  IER_cached |= UART_IER_recvInt;
  serial_out(info, UART_IER, IER_cached);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_disable_ints(struct async_struct *info, u16 *msr)
{
  *msr = IER_cached & UART_IER_IntPair;
  IER_cached &= ~UART_IER_IntPair;
  serial_out(info, UART_IER, IER_cached);
}

/******************************
 Routine:
 Description:
******************************/
static inline void serial_restore_ints(struct async_struct *info, u16 msr)
{
  IER_cached |= msr & UART_IER_IntPair;
  serial_out(info, UART_IER, IER_cached);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_clear_fifos(struct async_struct *info)
{
  FCR_cached = 0;
  serial_out(info, UART_EFR,  0x0010); // enable fifo control reg.
  serial_out(info, UART_TFCR, FCR_cached);      // reset to 0
  FCR_cached = UART_FCR_RX_CLR | UART_FCR_TX_CLR;
  serial_out(info, UART_RFCR, FCR_cached); // clear fifos.
  FCR_cached = UART_FCR_FIFO_EN;       // enable RX and TX fifos.
  serial_out(info, UART_TFCR, FCR_cached);
}

/******************************
 Routine:
 Description:
 ******************************/
// Warning! for the Orion platform this paticular routine
//          failed to work correctly when made "inline". ??????
#define DONT_INLINE
static DONT_INLINE void serial_disable_breaks(struct async_struct *info)
{
  LCR_cached &= ~UART_LCR_BOC;
  serial_out(info, UART_LCR, LCR_cached);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_enable_breaks(struct async_struct *info)
{
  LCR_cached |= UART_LCR_BOC;
  serial_out(info, UART_LCR, LCR_cached);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_set_rate(struct async_struct *info, unsigned int rate)
{
  if (115200 == rate) serial_out(info, UART_DIV_BIT_RATE, BAUD_115200);
  if (57600  == rate) serial_out(info, UART_DIV_BIT_RATE, BAUD_57600);
  if (38400  == rate) serial_out(info, UART_DIV_BIT_RATE, BAUD_38400);
  if (19200  == rate) serial_out(info, UART_DIV_BIT_RATE, BAUD_19200);
  if (9600   == rate) serial_out(info, UART_DIV_BIT_RATE, BAUD_9600);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_set_mode(struct async_struct *info, u16 mode)
{
  LCR_cached &= 0xffffffe0;          // clear original field, and...
  LCR_cached |= (unsigned long)mode; // now set new bits in that field.
  serial_out(info, UART_LCR, LCR_cached);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_set_rx_trigger(struct async_struct *info, u16 val)
{
  /* does val need to be shifted to the right spot? */
  FCR_cached = (FCR_cached & 0xffffff3f) | (val & 0xc0);
  serial_out(info, UART_RFCR, FCR_cached);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_set_tx_trigger(struct async_struct *info, u16 val)
{
  /* does val need to be shifted to the right spot? */
  FCR_cached = (FCR_cached & 0xffffffcf) | (val & 0x30);
  serial_out(info, UART_RFCR, FCR_cached);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_char_out(struct async_struct *info, u8 val)
{
  serial_out(info, UART_THR, val);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline unsigned char serial_char_in(struct async_struct *info,
                                           unsigned short *status)
{
  volatile unsigned long val;
  // Next, construct a 16bit status word that
  // uses the high byte to hold the status bits
  // associated with framing,parity,break and
  // a low byte that holds error bits of LSR for
  // conditions such as overflow, etc.
  val = serial_in(info,UART_RHR);
  *status = (unsigned short)(val & 0x0000ff00);
  *status = (unsigned short)(*status | (serial_in(info,UART_LSR) & 0x000000ff));
  return (val & 0x000000ff);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline int serial_some_error_condition_present(unsigned short status)
{
  return (status & 0x0782);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline int serial_parity_error_condition_present(unsigned short status)
{
  return (status & 0x0100);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline int serial_framing_error_condition_present(unsigned short status)
{
  return (status & 0x0200);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline int serial_overrun_error_condition_present(unsigned short status)
{
  return (status & 0x0002);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline unsigned long Get_INT_Cause_Bits(struct async_struct *info)
{
  // Note:
  // This register is an autoclearing one. This means that
  // if we only want to read the value once and then with
  // the cached value do the various tests to see which of
  // the interrupts are asserted. Don't want to re-read the
  // register for each test since only the first read would
  // be valid and rest would see the auto-cleared state.
  return (serial_in(info,UART_ISR) & 0x0000003f);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline int serial_xmit_INT_asserted(struct async_struct *info, unsigned long Cause_Bits)
{
    return ((Cause_Bits & 0x00000002) == 0x00000002);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline int serial_recv_INT_asserted(struct async_struct *info, unsigned long Cause_Bits)
{
    return ((Cause_Bits & 0x0000000c) == 0x00000004);
  /*
  if ((0x00000004 == Cause_Bits) | (0x00000006 == Cause_Bits)) {
    return 1;
  }
  else {
    return 0;
  }
  */
}

/******************************
 Routine:
 Description:
 ******************************/
static inline int serial_recvTimeout_INT_asserted(struct async_struct *info, unsigned long Cause_Bits)
{
    return ((Cause_Bits & 0x0000000c) == 0x0000000c);

  /*
  if ((0x0000000c == Cause_Bits) | (0x0000000e == Cause_Bits)) {
    return 1;
  }
  else {
    return 0;
  }
  */
}

/******************************
 Routine:
 Description:
 ******************************/
static inline u8 serial_tx_fifo_is_empty(struct async_struct *info)
{
  return ((serial_in(info, UART_LSR) & UART_LSR_TREF) == 1);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline u8 serial_room_in_tx_fifo(struct async_struct *info)
{
  // return ((serial_in(info, UART_LSR) & UART_LSR_TREF));
  return ((serial_in(info, UART_SSR) & UART_SSR_TXFULL) == 0);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline u8 serial_rx_fifo_has_content(struct async_struct *info)
{
  return (serial_in(info, UART_LSR) & UART_RX_FIFO_NOEMPTY);
}

/******************************
 Routine:
 Description:
 ******************************/
static inline void serial_save_registers(struct async_struct *info)
{
  IER_cached = serial_in(info, UART_IER);
  LCR_cached = serial_in(info, UART_LCR);
}

// ------------------------------------------
//       -- End; Platform Specifics --
// ------------------------------------------
#endif



#ifdef CONFIG_SERIAL_DSPLINUX_CONSOLE

// NOTE:
// This block is enabled when the user has used `make xconfig`
// to enable kernel printk() to come out the serial port.
// The register_console(serial_console_print) call below
// is what hooks in our serial output routine here with the
// kernel's printk output.

static struct async_struct *dsplinux_sercons = &rs_table[0];
extern void register_console(void (*proc)(const char *));

/******************************
 Routine:
 Description:
******************************/
static inline void wait_for_xmitr(struct async_struct *info)
{
	int tmp;
        for (tmp=1000 ; tmp > 0 ; tmp--)
            if (serial_room_in_tx_fifo(info))
                break;
}

/******************************
 Routine:
 Description:
******************************/
void serial_console_print(const char *b)
{
  struct async_struct *info = dsplinux_sercons;
  u16 ier;

  serial_disable_ints(info, &ier);

  while(*b != 0) {
          wait_for_xmitr(info);
          serial_char_out(info, *b);
          if (*b == 10) { // LF? add CR
                  wait_for_xmitr(info);
                  serial_char_out(info, 13);
          }
          b++;
  }

  wait_for_xmitr(info);
  serial_restore_ints(info, ier);
}

/******************************
 Routine:
 Description:
******************************/
void serial_dsplinux_console_init(void)
{
  u16 ier;
  struct async_struct *info = dsplinux_sercons;
  serial_reset_chip(info);  
  serial_save_registers(info);
  serial_disable_ints(info,&ier);
  serial_clear_fifos(info);
  serial_disable_breaks(info);
  serial_set_rate(info,115200);
  serial_set_mode(info,MODE8_NOPARITY_STOP1);
  register_console(serial_console_print);
}

#endif /* CONFIG_SERIAL_DSPLINUX_CONSOLE */




/******************************
 Routine:
 Description:
 ******************************/
static inline int serial_paranoia_check(struct async_struct *info,
					kdev_t device, const char *routine)
{
#ifdef SERIAL_PARANOIA_CHECK
	static const char * const badmagic =
		"Warning: bad magic number for serial struct (%s) in %s\n";
	static const char * const badinfo =
		"Warning: null async_struct for (%s) in %s\n";

	if (!info) {
		printk(badinfo, kdevname(device), routine);
		return 1;
	}
	if (info->magic != SERIAL_MAGIC) {
		printk(badmagic, kdevname(device), routine);
		return 1;
	}
#endif
	return 0;
}

/*
 * ------------------------------------------------------------
 * rs_stop() and rs_start()
 *
 * This routines are called before setting or resetting tty->stopped.
 * They enable or disable transmitter interrupts, as necessary.
 * ------------------------------------------------------------
 */
static void rs_stop(struct tty_struct *tty)
{
	struct async_struct *info = (struct async_struct *)tty->driver_data;
	unsigned long flags;

        // checkpoint(13); // for debug; temp.

	if (serial_paranoia_check(info, tty->device, "rs_stop"))
		return;
	
	save_flags(flags); cli();
        serial_disable_tx_int(info);
	restore_flags(flags);
}

static void rs_start(struct tty_struct *tty)
{
        struct async_struct *info = (struct async_struct *)tty->driver_data;
        unsigned long flags;

        // checkpoint(14); // for debug; temp.
        
        if (serial_paranoia_check(info, tty->device, "rs_start"))
                return;
        
        save_flags(flags); cli();
        if (info->xmit_cnt && info->xmit_buf) {
          serial_enable_tx_int(info);
        }
        restore_flags(flags);
}

/*
 * This routine is used by the interrupt handler to schedule
 * processing in the software interrupt portion of the driver.
 */
static inline void rs_sched_event(struct async_struct *info,
				  int event)
{
	info->event |= 1 << event;
	queue_task_irq_off(&info->tqueue, &tq_serial);
	mark_bh(SERIAL_BH);
}

static inline void receive_chars(struct async_struct *info)
{
        struct tty_struct *tty = info->tty;
        unsigned short status;
        unsigned char ch;
        do {
                ch = serial_char_in(info,&status);
                if (tty->flip.count >= TTY_FLIPBUF_SIZE) break;
                tty->flip.count++;
                if (serial_some_error_condition_present(status)) {
                        if (serial_parity_error_condition_present(status))
                                *tty->flip.flag_buf_ptr++ = TTY_PARITY;
                        else if (serial_framing_error_condition_present(status))
                                *tty->flip.flag_buf_ptr++ = TTY_FRAME;
                        else if (serial_overrun_error_condition_present(status))
                                *tty->flip.flag_buf_ptr++ = TTY_OVERRUN;
                }
                else
                        *tty->flip.flag_buf_ptr++ = 0;

                *tty->flip.char_buf_ptr++ = ch;
        } while (serial_rx_fifo_has_content(info));

#ifdef CONFIG_SERIAL_DSPLINUX_CONSOLE
        if(info == dsplinux_sercons)
                /*
                 * We are the console. Tell tty_io.c that we received a
                 * character.
                 */
                wake_up(&keypress_wait);
#endif

        queue_task_irq_off(&tty->flip.tqueue, &tq_timer);
}

static inline void transmit_chars(struct async_struct *info)
{
        if (info->x_char) {
                serial_char_out(info, info->x_char);
                info->x_char = 0;
                return;
        }
        if ((info->xmit_cnt <= 0) || info->tty->stopped || info->tty->hw_stopped) {
                serial_disable_tx_int(info);
                return;
        }
        
        // send while we still have data & room in the fifo
        do {
                serial_char_out(info,info->xmit_buf[info->xmit_tail]);
                info->xmit_tail = (info->xmit_tail + 1) & (SERIAL_XMIT_SIZE-1);
                if (--info->xmit_cnt <= 0) goto done;
        } while(serial_room_in_tx_fifo(info));

 done:
        
        if (info->xmit_cnt < WAKEUP_CHARS)
                rs_sched_event(info, RS_EVENT_WRITE_WAKEUP);

        if (info->xmit_cnt <= 0)
                serial_disable_tx_int(info);
}

/*
 * This routine is used to handle the "top half" processing for the
 * serial driver.
 */
static void rs_interrupt(int irq, void *dev_id, struct pt_regs * regs)
{
  u16 sr = 0, msr = 0 ;
  int count = 0;
  unsigned short status;
  volatile unsigned char toss;
  volatile unsigned long Cause_Bits;
  struct async_struct *info;
  info = (struct async_struct *)dev_id;
  if (!info || !info->tty) {
    printk(__FUNCTION__ ": bad port or tty state\n");
    return;
  }
  if (count++ > 100) panic("Stuck in " __FUNCTION__ "\n");
  Cause_Bits = Get_INT_Cause_Bits(info);
  if (serial_recvTimeout_INT_asserted(info,Cause_Bits)) {
    receive_chars(info);
    /*
    toss = serial_char_in(info,&status); // read UART_RHR to clear int condition
    */
  }
  else {
    if (serial_recv_INT_asserted(info,Cause_Bits)) {
      receive_chars(info);
    }
  }
  if (serial_xmit_INT_asserted(info,Cause_Bits))
    transmit_chars(info);
}

/*
 * This routine is used to handle the "bottom half" processing for the
 * serial driver, known also the "software interrupt" processing.
 * This processing is done at the kernel interrupt level, after the
 * rs_interrupt() has returned, BUT WITH INTERRUPTS TURNED ON.  This
 * is where time-consuming activities which can not be done in the
 * interrupt driver proper are done; the interrupt driver schedules
 * them using rs_sched_event(), and they get done here.
 */
static void do_serial_bh(void)
{
	run_task_queue(&tq_serial);
}

static void do_softint(void *private_)
{
	struct async_struct	*info = (struct async_struct *) private_;
	struct tty_struct	*tty;
	
	tty = info->tty;
	if (!tty)
		return;

	if (clear_bit(RS_EVENT_WRITE_WAKEUP, &info->event)) {
		if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) &&
		    tty->ldisc.write_wakeup)
			(tty->ldisc.write_wakeup)(tty);
		wake_up_interruptible(&tty->write_wait);
	}
}

/*
 * This routine is called from the scheduler tqueue when the interrupt
 * routine has signalled that a hangup has occurred.  The path of
 * hangup processing is:
 *
 * 	serial interrupt routine -> (scheduler tqueue) ->
 * 	do_serial_hangup() -> tty->hangup() -> rs_hangup()
 * 
 */
static void do_serial_hangup(void *private_)
{
	struct async_struct	*info = (struct async_struct *) private_;
	struct tty_struct	*tty;
	
	tty = info->tty;
	if (!tty)
		return;

	tty_hangup(tty);
}

static int startup(struct async_struct * info)
{

	unsigned long flags;
	int	retval;

	unsigned long page;
	u16 junk;

	page = get_free_page(GFP_KERNEL);
	if (!page)
		return -ENOMEM;

	
	save_flags(flags); cli();

	if (info->flags & ASYNC_INITIALIZED) {
		free_page(page);
		restore_flags(flags);
		return 0;
	}

	if (info->xmit_buf)
		free_page(page);
	else
		info->xmit_buf = (unsigned char *) page;

	serial_disable_ints(info, &junk);
	serial_clear_fifos(info);
	serial_disable_breaks(info);	

	retval = request_irq(info->irq, rs_interrupt, 0, "serial", info);
	if (retval) {
		restore_flags(flags);
		if (suser()) {
			if (info->tty)
				set_bit(TTY_IO_ERROR,
					&info->tty->flags);
			return 0;
		} else
			return retval;
	}
	
	/*
	 * Finally, enable interrupts
	 */
	serial_enable_rx_int(info);

	if (info->tty)
		clear_bit(TTY_IO_ERROR, &info->tty->flags);
	info->xmit_cnt = info->xmit_head = info->xmit_tail = 0;


	/*
	 * and set the speed of the serial port
	 */
	change_speed(info);

	info->flags |= ASYNC_INITIALIZED;
	restore_flags(flags);
	return 0;
}

/*
 * This routine will shutdown a serial port; interrupts are disabled, and
 * DTR is dropped if the hangup on close termio flag is on.
 */
static void shutdown(struct async_struct * info)
{
	unsigned long	flags;

	u16             msr;

	if (!(info->flags & ASYNC_INITIALIZED))
		return;

	
	save_flags(flags); cli(); /* Disable interrupts */

	/*
	 * clear delta_msr_wait queue to avoid mem leaks: we may free the irq
	 * here so the queue might never be waken up
	 */
	wake_up_interruptible(&info->delta_msr_wait);
	
	free_irq(info->irq, info);

	if (info->xmit_buf) {
		free_page((unsigned long) info->xmit_buf);
		info->xmit_buf = 0;
	}

	serial_disable_ints(info, &msr);

	if (info->tty)
		set_bit(TTY_IO_ERROR, &info->tty->flags);
	
	info->flags &= ~ASYNC_INITIALIZED;
	restore_flags(flags);
}

/*
 * This routine is called to set the UART divisor registers to match
 * the specified baud rate for a serial port.
 */
static int tty_get_baud_rate(struct tty_struct *tty)
{
	struct async_struct * info = 
		(struct async_struct *)tty->driver_data;
	unsigned int cflag, i;

	cflag = tty->termios->c_cflag;

	i = cflag & CBAUD;
	if (i & CBAUDEX) {
		i &= ~CBAUDEX;
		if (i < 1 || i > 2) 
			tty->termios->c_cflag &= ~CBAUDEX;
		else
			i += 15;
	}
	if (i == 15) {
		if ((info->flags & ASYNC_SPD_MASK) == ASYNC_SPD_HI)
			i += 1;
		if ((info->flags & ASYNC_SPD_MASK) == ASYNC_SPD_VHI)
			i += 2;
	}
	return baud_table[i];
}

static void change_speed(struct async_struct *info)
{
        int     quot = 0, baud_base, baud;
        unsigned cflag, cval = 0;
        int     bits;
        unsigned long   flags;

        if (!info->tty || !info->tty->termios)
                return;
        cflag = info->tty->termios->c_cflag;


        /* byte size and parity */
        switch (cflag & CSIZE) {
        case CS7: cval = UART_LCR_7bits; bits = 9; break;
        case CS8: cval = UART_LCR_8bits; bits = 10; break;
        default:  cval = UART_LCR_8bits; bits = 7; break;
        }

        if (cflag & CSTOPB) {
                cval |= UART_LCR_2stop;
                bits++;
        }
        if (cflag & PARENB) {
                cval |= UART_LCR_ParEn;
                bits++;
        }
        if (cflag & PARODD)
                cval |= UART_LCR_ParOdd;

        /* Determine divisor based on baud rate */
        baud = tty_get_baud_rate(info->tty);
        if (!baud)
                baud = 9600;    /* B0 transition handled in rs_set_termios */
        baud_base = info->baud_base;
        if (baud == 38400 &&
            ((info->flags & ASYNC_SPD_MASK) == ASYNC_SPD_CUST))
                quot = info->custom_divisor;
        else {
                if (baud == 134)
                        /* Special case since 134 is really 134.5 */
                        quot = (2*baud_base / 269);
                else if (baud)
                        quot = (baud_base / baud) - 1;
        }
        /* As a last resort, if the quotient is zero, default to 9600 bps */
        if (!quot)
                quot = (baud_base / 9600) - 1;
        
        info->timeout = ((info->xmit_fifo_size*HZ*bits*quot) / baud_base);
        info->timeout += HZ/50;         /* Add .02 seconds of slop */

        /*
         * Set up parity check flag
         */
#define RELEVANT_IFLAG(iflag) (iflag & (IGNBRK|BRKINT|IGNPAR|PARMRK|INPCK))


        save_flags(flags); cli();

        serial_set_rate(info, baud_base);
        serial_set_mode(info, cval);

        /*
         * We should always set the receive FIFO trigger to the lowest value
         * because we don't poll.
         * --gmcnutt
         */
        serial_set_rx_trigger(info, UART_FCR_FTL);

        restore_flags(flags);

}

static void rs_put_char(struct tty_struct *tty, unsigned char ch)
{
	struct async_struct *info = (struct async_struct *)tty->driver_data;
	unsigned long flags;

        // checkpoint(4); // for debug; temp.

	if (serial_paranoia_check(info, tty->device, "rs_put_char"))
		return;

	if (!tty || !info->xmit_buf)
		return;

	save_flags(flags); cli();
	if (info->xmit_cnt >= SERIAL_XMIT_SIZE - 1) {
		restore_flags(flags);
		return;
	}

	info->xmit_buf[info->xmit_head++] = ch;
	info->xmit_head &= SERIAL_XMIT_SIZE-1;
	info->xmit_cnt++;
	restore_flags(flags);
}

static void rs_flush_chars(struct tty_struct *tty)
{
	struct async_struct *info = (struct async_struct *)tty->driver_data;
	unsigned long flags;

        // checkpoint(5); // for debug; temp.
				
	if (serial_paranoia_check(info, tty->device, "rs_flush_chars"))
		return;

	if (info->xmit_cnt <= 0 || tty->stopped || tty->hw_stopped ||
	    !info->xmit_buf)
		return;

	save_flags(flags); cli();
	serial_enable_tx_int(info);
	restore_flags(flags);
}

static int rs_write(struct tty_struct * tty, int from_user,
		    const unsigned char *buf, int count)
{
	int	c, total = 0;
	struct async_struct *info = (struct async_struct *)tty->driver_data;
	unsigned long flags;

        // checkpoint(3); // for debug; temp.
				
	if (serial_paranoia_check(info, tty->device, "rs_write"))
		return 0;

	if (!tty || !info->xmit_buf || !tmp_buf)
		return 0;
	    
	if (from_user)
		down(&tmp_buf_sem);
	save_flags(flags);
	while (1) {
		cli();		
		c = MIN(count, MIN(SERIAL_XMIT_SIZE - info->xmit_cnt - 1,
				   SERIAL_XMIT_SIZE - info->xmit_head));
		if (c <= 0)
			break;

		if (from_user) {
			memcpy_fromfs(tmp_buf, buf, c);
			c = MIN(c, MIN(SERIAL_XMIT_SIZE - info->xmit_cnt - 1,
				       SERIAL_XMIT_SIZE - info->xmit_head));
			memcpy(info->xmit_buf + info->xmit_head, tmp_buf, c);
		} else
			memcpy(info->xmit_buf + info->xmit_head, buf, c);
		info->xmit_head = (info->xmit_head + c) & (SERIAL_XMIT_SIZE-1);
		info->xmit_cnt += c;
		restore_flags(flags);
		buf += c;
		count -= c;
		total += c;
	}
	if (from_user)
		up(&tmp_buf_sem);
	if (info->xmit_cnt && 
	    !tty->stopped &&
	    !tty->hw_stopped) {
		if (serial_room_in_tx_fifo(info)) {
			transmit_chars(info);
		}
		serial_enable_tx_int(info);
	}

	restore_flags(flags);
	return total;
}

static int rs_write_room(struct tty_struct *tty)
{
	struct async_struct *info = (struct async_struct *)tty->driver_data;
	int	ret;

        // checkpoint(6); // for debug; temp.
				
	if (serial_paranoia_check(info, tty->device, "rs_write_room"))
		return 0;
	ret = SERIAL_XMIT_SIZE - info->xmit_cnt - 1;
	if (ret < 0)
		ret = 0;
	return ret;
}

static int rs_chars_in_buffer(struct tty_struct *tty)
{
	struct async_struct *info = (struct async_struct *)tty->driver_data;

        // checkpoint(7); // for debug; temp.
				
	if (serial_paranoia_check(info, tty->device, "rs_chars_in_buffer"))
		return 0;
	return info->xmit_cnt;
}

static void rs_flush_buffer(struct tty_struct *tty)
{
	struct async_struct *info = (struct async_struct *)tty->driver_data;

        // checkpoint(8); // for debug; temp.
				
	if (serial_paranoia_check(info, tty->device, "rs_flush_buffer"))
		return;
	{
		unsigned long flags;
		save_flags_cli(flags);
		info->xmit_cnt = info->xmit_head = info->xmit_tail = 0;
		restore_flags(flags);
	}
	wake_up_interruptible(&tty->write_wait);
	if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) &&
	    tty->ldisc.write_wakeup)
		(tty->ldisc.write_wakeup)(tty);
}

/*
 * ------------------------------------------------------------
 * rs_throttle()
 * 
 * This routine is called by the upper-layer tty layer to signal that
 * incoming characters should be throttled.
 * ------------------------------------------------------------
 */
static void rs_throttle(struct tty_struct * tty)
{
	struct async_struct *info = (struct async_struct *)tty->driver_data;
#ifdef SERIAL_DEBUG_THROTTLE
	char	buf[64];
	
	printk("throttle %s: %d....\n", _tty_name(tty, buf),
	       tty->ldisc.chars_in_buffer(tty));
#endif

        // checkpoint(10); // for debug; temp.

	if (serial_paranoia_check(info, tty->device, "rs_throttle"))
		return;
	
	if (I_IXOFF(tty))
		info->x_char = STOP_CHAR(tty);

}

static void rs_unthrottle(struct tty_struct * tty)
{
	struct async_struct *info = (struct async_struct *)tty->driver_data;
#ifdef SERIAL_DEBUG_THROTTLE
	char	buf[64];
	
	printk("unthrottle %s: %d....\n", _tty_name(tty, buf),
	       tty->ldisc.chars_in_buffer(tty));
#endif
        
        // checkpoint(11); // for debug; temp.

	if (serial_paranoia_check(info, tty->device, "rs_unthrottle"))
		return;
	
	if (I_IXOFF(tty)) {
		if (info->x_char)
			info->x_char = 0;
		else
			info->x_char = START_CHAR(tty);
	}
}

/*
 * ------------------------------------------------------------
 * rs_ioctl() and friends
 * ------------------------------------------------------------
 */

static int get_serial_info(struct async_struct * info,
			   struct serial_struct * retinfo)
{
	struct serial_struct tmp;

        // checkpoint(9); // for debug; temp.
  
	if (!retinfo)
		return -EFAULT;
	memset(&tmp, 0, sizeof(tmp));
	tmp.line = info->line;
	tmp.port = info->port;
	tmp.irq = info->irq;
	tmp.flags = info->flags;
	tmp.baud_base = info->baud_base;
	tmp.close_delay = info->close_delay;
	tmp.closing_wait = info->closing_wait;
	tmp.custom_divisor = info->custom_divisor;
	tmp.hub6 = info->hub6;
	memcpy_tofs(retinfo,&tmp,sizeof(*retinfo));
	return 0;
}

static int set_serial_info(struct async_struct * info,
			   struct serial_struct * new_info)
{
	struct serial_struct new_serial;
	struct async_struct old_info;
	unsigned int		i,change_irq,change_port;
	int 			retval = 0;

	if (!new_info)
		return -EFAULT;
	memcpy_fromfs(&new_serial,new_info,sizeof(new_serial));
	old_info = *info;

	change_irq = new_serial.irq != info->irq;
	change_port = (new_serial.port != info->port) || (new_serial.hub6 != info->hub6);

	if (!suser()) {
		if (change_irq || change_port ||
		    (new_serial.baud_base != info->baud_base) ||
		    (new_serial.close_delay != info->close_delay) ||
		    ((new_serial.flags & ~ASYNC_USR_MASK) !=
		     (info->flags & ~ASYNC_USR_MASK)))
			return -EPERM;
		info->flags = ((info->flags & ~ASYNC_USR_MASK) |
			       (new_serial.flags & ASYNC_USR_MASK));
		info->custom_divisor = new_serial.custom_divisor;
		goto check_and_exit;
	}

	if (new_serial.irq == 2)
		new_serial.irq = 9;

	if ((new_serial.irq > 15) || (new_serial.port > 0xffff))
		return -EINVAL;

	/* Make sure address is not already in use */
	for (i = 0 ; i < NR_PORTS; i++)
		if ((info != &rs_table[i]) &&
		    (rs_table[i].port == new_serial.port))
			return -EADDRINUSE;

	if ((change_port || change_irq) && (info->count > 1))
		return -EBUSY;

	/*
	 * OK, past this point, all the error checking has been done.
	 * At this point, we start making changes.....
	 */

	info->baud_base = new_serial.baud_base;
	info->flags = ((info->flags & ~ASYNC_FLAGS) |
			(new_serial.flags & ASYNC_FLAGS));
	info->custom_divisor = new_serial.custom_divisor;
	info->close_delay = new_serial.close_delay * HZ/100;
	info->closing_wait = new_serial.closing_wait * HZ/100;

	if (change_port || change_irq) {
		/*
		 * We need to shutdown the serial port at the old
		 * port/irq combination.
		 */
		shutdown(info);
		info->irq = new_serial.irq;
		info->port = new_serial.port;
		info->hub6 = new_serial.hub6;
	}

	
check_and_exit:
	if (info->flags & ASYNC_INITIALIZED) {
		if (((old_info.flags & ASYNC_SPD_MASK) !=
		     (info->flags & ASYNC_SPD_MASK)) ||
		    (old_info.custom_divisor != info->custom_divisor))
			change_speed(info);
	} else
		retval = startup(info);
	return retval;
}


static int do_autoconfig(struct async_struct * info)
{
	int			retval;
	
	if (!suser())
		return -EPERM;
	
	if (info->count > 1)
		return -EBUSY;
	
	shutdown(info);

	{
		unsigned long flags;
		save_flags_cli(flags);
		autoconfig(info);
		restore_flags(flags);
	}

	retval = startup(info);
	if (retval)
		return retval;
	return 0;
}

static void send_break(	struct async_struct * info, int duration)
{
	current->state = TASK_INTERRUPTIBLE;
	current->timeout = jiffies + duration;
	{
		unsigned long flags;
		save_flags_cli(flags);
		serial_enable_breaks(info);
		schedule();
		serial_disable_breaks(info);
		restore_flags(flags);
	}
}

static void rs_break(struct async_struct * info, int break_state)
{
        unsigned long flags;
        
        save_flags_cli(flags);
        if (break_state == -1)
		serial_enable_breaks(info);
        else
		serial_disable_breaks(info);
        restore_flags(flags);
}

static int rs_ioctl(struct tty_struct *tty, struct file * file,
		    unsigned int cmd, unsigned long arg)
{
	int error;
	struct async_struct * info = (struct async_struct *)tty->driver_data;
	int retval;
	struct async_icount cnow;	/* kernel counter temps */
	struct serial_icounter_struct *p_cuser;	/* user space */

	if (serial_paranoia_check(info, tty->device, "rs_ioctl"))
		return -ENODEV;

	if ((cmd != TIOCGSERIAL) && (cmd != TIOCSSERIAL) &&
	    (cmd != TIOCSERCONFIG) && (cmd != TIOCSERGWILD)  &&
	    (cmd != TIOCSERSWILD) && (cmd != TIOCSERGSTRUCT) &&
	    (cmd != TIOCMIWAIT) && (cmd != TIOCGICOUNT)) {
		if (tty->flags & (1 << TTY_IO_ERROR))
		    return -EIO;
	}
	
	switch (cmd) {
                case TIOCSBRK:  /* Turn break on, unconditionally */
			retval = tty_check_change(tty);
			if (retval)
				return retval;
			tty_wait_until_sent(tty, 0);
			rs_break(info,-1);
			return 0;
                case TIOCCBRK:  /* Turn break off, unconditionally */
			retval = tty_check_change(tty);
			if (retval)
				return retval;
			tty_wait_until_sent(tty, 0);
			rs_break(info,0);
			return 0;
		case TCSBRK:	/* SVID version: non-zero arg --> no break */
			retval = tty_check_change(tty);
			if (retval)
				return retval;
			tty_wait_until_sent(tty, 0);
			if (!arg)
				send_break(info, HZ/4);	/* 1/4 second */
			return 0;
		case TCSBRKP:	/* support for POSIX tcsendbreak() */
			retval = tty_check_change(tty);
			if (retval)
				return retval;
			tty_wait_until_sent(tty, 0);
			send_break(info, arg ? arg*(HZ/10) : HZ/4);
			return 0;
		case TIOCGSOFTCAR:
			error = verify_area(VERIFY_WRITE, (void *) arg,sizeof(long));
			if (error)
				return error;
			put_fs_long(C_CLOCAL(tty) ? 1 : 0,
				    (unsigned long *) arg);
			return 0;
		case TIOCSSOFTCAR:
			error = verify_area(VERIFY_READ, (void *) arg,sizeof(long));
			if (error)
				return error;
			arg = get_fs_long((unsigned long *) arg);
			tty->termios->c_cflag =
				((tty->termios->c_cflag & ~CLOCAL) |
				 (arg ? CLOCAL : 0));
			return 0;
		case TIOCGSERIAL:
			error = verify_area(VERIFY_WRITE, (void *) arg,
						sizeof(struct serial_struct));
			if (error)
				return error;
			return get_serial_info(info,
					       (struct serial_struct *) arg);
		case TIOCSSERIAL:
			error = verify_area(VERIFY_READ, (void *) arg,
						sizeof(struct serial_struct));
			if (error)
				return error;
			return set_serial_info(info,
					       (struct serial_struct *) arg);
		case TIOCSERCONFIG:
			return do_autoconfig(info);

		case TIOCSERGSTRUCT:
			error = verify_area(VERIFY_WRITE, (void *) arg,
						sizeof(struct async_struct));
			if (error)
				return error;
			memcpy_tofs((struct async_struct *) arg,
				    info, sizeof(struct async_struct));
			return 0;
			
		/* 
		 * Get counter of input serial line interrupts (DCD,RI,DSR,CTS)
		 * Return: write counters to the user passed counter struct
		 * NB: both 1->0 and 0->1 transitions are counted except for
		 *     RI where only 0->1 is counted.
		 */
		case TIOCGICOUNT:
			error = verify_area(VERIFY_WRITE, (void *) arg,
				sizeof(struct serial_icounter_struct));
			if (error)
				return error;
			{
				unsigned long flags;
				save_flags_cli(flags);
				cnow = info->icount;
				restore_flags(flags);
			}
			p_cuser = (struct serial_icounter_struct *) arg;
			put_user(cnow.cts, &p_cuser->cts);
			put_user(cnow.dsr, &p_cuser->dsr);
			put_user(cnow.rng, &p_cuser->rng);
			put_user(cnow.dcd, &p_cuser->dcd);
			return 0;

		default:
			return -ENOIOCTLCMD;
		}
	return 0;
}

static void rs_set_termios(struct tty_struct *tty, struct termios *old_termios)
{
	struct async_struct *info = (struct async_struct *)tty->driver_data;

        // checkpoint(12); // for debug; temp.

	if (   (tty->termios->c_cflag == old_termios->c_cflag)
	    && (   RELEVANT_IFLAG(tty->termios->c_iflag) 
		== RELEVANT_IFLAG(old_termios->c_iflag)))
	  return;

	change_speed(info);

	if ((old_termios->c_cflag & CRTSCTS) &&
	    !(tty->termios->c_cflag & CRTSCTS)) {
		tty->hw_stopped = 0;
		rs_start(tty);
	}

}

/*
 * ------------------------------------------------------------
 * rs_close()
 * 
 * This routine is called when the serial port gets closed.  First, we
 * wait for the last remaining data to be sent.  Then, we unlink its
 * async structure from the interrupt chain if necessary, and we free
 * that IRQ if nothing is left in the chain.
 * ------------------------------------------------------------
 */
static void rs_close(struct tty_struct *tty, struct file * filp)
{
	struct async_struct * info = (struct async_struct *)tty->driver_data;
	unsigned long flags;
	unsigned long timeout;

        // checkpoint(2); // for debug; temp.

	if (!info || serial_paranoia_check(info, tty->device, "rs_close"))
		return;
	
	save_flags_cli(flags);
	
	if (tty_hung_up_p(filp)) {
		DBG_CNT("before DEC-hung");
		MOD_DEC_USE_COUNT;
		restore_flags(flags);
		return;
	}
	
	if ((tty->count == 1) && (info->count != 1)) {
		/*
		 * Uh, oh.  tty->count is 1, which means that the tty
		 * structure will be freed.  Info->count should always
		 * be one in these conditions.  If it's greater than
		 * one, we've got real problems, since it means the
		 * serial port won't be shutdown.
		 */
		printk("rs_close: bad serial port count; tty->count is 1, "
		       "info->count is %d\n", info->count);
		info->count = 1;
	}
	if (--info->count < 0) {
		printk("rs_close: bad serial port count for ttys%d: %d\n",
		       info->line, info->count);
		info->count = 0;
	}
	if (info->count) {
		DBG_CNT("before DEC-2");
		MOD_DEC_USE_COUNT;
		restore_flags(flags);
		return;
	}
	info->flags |= ASYNC_CLOSING;
	/*
	 * Save the termios structure, since this port may have
	 * separate termios for callout and dialin.
	 */
	if (info->flags & ASYNC_NORMAL_ACTIVE)
		info->normal_termios = *tty->termios;
	if (info->flags & ASYNC_CALLOUT_ACTIVE)
		info->callout_termios = *tty->termios;
	/*
	 * Now we wait for the transmit buffer to clear; and we notify 
	 * the line discipline to only process XON/XOFF characters.
	 */
	tty->closing = 1;
	if (info->closing_wait != ASYNC_CLOSING_WAIT_NONE)
		tty_wait_until_sent(tty, info->closing_wait);
	/*
	 * At this point we stop accepting input.  To do this, we
	 * disable the receive line status interrupts, and tell the
	 * interrupt driver to stop checking the data ready bit in the
	 * line status register.
	 */
	serial_disable_rx_int(info);	

	if (info->flags & ASYNC_INITIALIZED) {
		serial_enable_tx_int(info);
		/*
		 * Before we drop DTR, make sure the UART transmitter
		 * has completely drained; this is especially
		 * important if there is a transmit FIFO!
		 */
		timeout = jiffies+HZ;
		while (!serial_tx_fifo_is_empty(info)) {
			current->state = TASK_INTERRUPTIBLE;
			current->timeout = jiffies + info->timeout;
			schedule();
			if (jiffies > timeout)
				break;
		}
	}
	shutdown(info);
	if (tty->driver.flush_buffer)
		tty->driver.flush_buffer(tty);
	if (tty->ldisc.flush_buffer)
		tty->ldisc.flush_buffer(tty);
	tty->closing = 0;
	info->event = 0;
	info->tty = 0;
	if (info->blocked_open) {
		if (info->close_delay) {
			current->state = TASK_INTERRUPTIBLE;
			current->timeout = jiffies + info->close_delay;
			schedule();
		}
		wake_up_interruptible(&info->open_wait);
	}
	info->flags &= ~(ASYNC_NORMAL_ACTIVE|ASYNC_CALLOUT_ACTIVE|
			 ASYNC_CLOSING);
	wake_up_interruptible(&info->close_wait);
	MOD_DEC_USE_COUNT;
	restore_flags(flags);
}

/*
 * rs_hangup() --- called by tty_hangup() when a hangup is signaled.
 */
void rs_hangup(struct tty_struct *tty)
{
	struct async_struct * info = (struct async_struct *)tty->driver_data;

        // checkpoint(15); // for debug; temp.
	
	if (serial_paranoia_check(info, tty->device, "rs_hangup"))
		return;
	
	rs_flush_buffer(tty);
	shutdown(info);
	info->event = 0;
	info->count = 0;
	info->flags &= ~(ASYNC_NORMAL_ACTIVE|ASYNC_CALLOUT_ACTIVE);
	info->tty = 0;
	wake_up_interruptible(&info->open_wait);
}


static int block_til_ready(struct tty_struct *tty, struct file * filp,
			   struct async_struct *info)
{
	struct wait_queue wait = { current, NULL };
	int		retval;
	int		do_clocal = 0;

	/*
	 * If the device is in the middle of being closed, then block
	 * until it's done, and then try again.
	 */
	if (tty_hung_up_p(filp) ||
	    (info->flags & ASYNC_CLOSING)) {
		if (info->flags & ASYNC_CLOSING)
			interruptible_sleep_on(&info->close_wait);
#ifdef SERIAL_DO_RESTART
		if (info->flags & ASYNC_HUP_NOTIFY)
			return -EAGAIN;
		else
			return -ERESTARTSYS;
#else
		return -EAGAIN;
#endif
	}

	/*
	 * If this is a callout device, then just make sure the normal
	 * device isn't being used.
	 */
	if (tty->driver.subtype == SERIAL_TYPE_CALLOUT) {
		if (info->flags & ASYNC_NORMAL_ACTIVE)
			return -EBUSY;
		if ((info->flags & ASYNC_CALLOUT_ACTIVE) &&
		    (info->flags & ASYNC_SESSION_LOCKOUT) &&
		    (info->session != current->session))
		    return -EBUSY;
		if ((info->flags & ASYNC_CALLOUT_ACTIVE) &&
		    (info->flags & ASYNC_PGRP_LOCKOUT) &&
		    (info->pgrp != current->pgrp))
		    return -EBUSY;
		info->flags |= ASYNC_CALLOUT_ACTIVE;
		return 0;
	}
	
	/*
	 * If non-blocking mode is set, or the port is not enabled,
	 * then make the check up front and then exit.
	 */
	if ((filp->f_flags & O_NONBLOCK) ||
	    (tty->flags & (1 << TTY_IO_ERROR))) {
		if (info->flags & ASYNC_CALLOUT_ACTIVE)
			return -EBUSY;
		info->flags |= ASYNC_NORMAL_ACTIVE;
		return 0;
	}

	if (info->flags & ASYNC_CALLOUT_ACTIVE) {
		if (info->normal_termios.c_cflag & CLOCAL)
			do_clocal = 1;
	} else {
		if (tty->termios->c_cflag & CLOCAL)
			do_clocal = 1;
	}
	
	/*
	 * Block waiting for the carrier detect and the line to become
	 * free (i.e., not in use by the callout).  While we are in
	 * this loop, info->count is dropped by one, so that
	 * rs_close() knows when to free things.  We restore it upon
	 * exit, either normal or abnormal.
	 */
	retval = 0;
	add_wait_queue(&info->open_wait, &wait);
	{
		unsigned long flags;
		save_flags_cli(flags);
		if (!tty_hung_up_p(filp)) 
			info->count--;
		restore_flags(flags);
	}
	info->blocked_open++;
	while (1) {
		current->state = TASK_INTERRUPTIBLE;
		if (tty_hung_up_p(filp) ||
		    !(info->flags & ASYNC_INITIALIZED)) {
#ifdef SERIAL_DO_RESTART
			if (info->flags & ASYNC_HUP_NOTIFY)
				retval = -EAGAIN;
			else
				retval = -ERESTARTSYS;	
#else
			retval = -EAGAIN;
#endif
			break;
		}
		if (!(info->flags & ASYNC_CALLOUT_ACTIVE) &&
		    !(info->flags & ASYNC_CLOSING))
			break;
		if (current->signal & ~current->blocked) {
			retval = -ERESTARTSYS;
			break;
		}
		schedule();
	}
	current->state = TASK_RUNNING;
	remove_wait_queue(&info->open_wait, &wait);
	if (!tty_hung_up_p(filp))
		info->count++;
	info->blocked_open--;
	if (retval)
		return retval;
	info->flags |= ASYNC_NORMAL_ACTIVE;
	return 0;
}	

/*
 * This routine is called whenever a serial port is opened.  It
 * enables interrupts for a serial port, linking in its async structure into
 * the IRQ chain.   It also performs the serial-specific
 * initialization for the tty structure.
 */
int rs_open(struct tty_struct *tty, struct file * filp)
{
	struct async_struct	*info;
	int 			retval, line;
	unsigned long		page;

        // checkpoint(1); // for debug; temp.

	line = MINOR(tty->device) - tty->driver.minor_start;
	if ((line < 0) || (line >= NR_PORTS))
		return -ENODEV;
	info = rs_table + line;
	if (serial_paranoia_check(info, tty->device, "rs_open"))
		return -ENODEV;

	info->count++;
	tty->driver_data = info;
	info->tty = tty;

	if (!tmp_buf) {
		page = get_free_page(GFP_KERNEL);
		if (!page)
			return -ENOMEM;
		if (tmp_buf)
			free_page(page);
		else
			tmp_buf = (unsigned char *) page;
	}
	
	/*
	 * Start up serial port
	 */
	retval = startup(info);
	if (retval)
		return retval;

	MOD_INC_USE_COUNT;
	retval = block_til_ready(tty, filp, info);
	if (retval) {
		return retval;
	}

	if ((info->count == 1) && (info->flags & ASYNC_SPLIT_TERMIOS)) {
		if (tty->driver.subtype == SERIAL_TYPE_NORMAL)
			*tty->termios = info->normal_termios;
		else 
			*tty->termios = info->callout_termios;
		change_speed(info);
	}

	info->session = current->session;
	info->pgrp = current->pgrp;

	return 0;
}

/*
 * This routine prints out the appropriate serial driver version
 * number, and identifies which options were configured into this
 * driver.
 */
static void show_serial_version(void)
{
 	printk(KERN_INFO "%s version %s with", serial_name, serial_version);
	printk(" no serial options enabled\n");
}

/*
 * This routine is called to initialize a specific serial
 * port.  It determines what type of UART chip this serial port is
 * using: 8250, 16450, 16550, 16550A.  The important question is
 * whether or not this UART is a 16550A or not, since this will
 * determine whether or not we can use its FIFO features or not.
 */
static void autoconfig(struct async_struct * info)
{

	unsigned long flags;
	u16 mrs;

	save_flags_cli(flags);
	
	serial_disable_ints(info, &mrs);
	serial_clear_fifos(info);
	serial_disable_breaks(info);
	serial_set_tx_trigger(info, UART_FCR_FTL);
	serial_set_rx_trigger(info, UART_FCR_FTL);

	restore_flags(flags);
}

void init_port(struct async_struct *info, int num)
{
  info->magic = SERIAL_MAGIC;
  info->line = num;
  info->tty = 0;

  info->custom_divisor = 0;
  info->close_delay = 5*HZ/10;
  info->closing_wait = 30*HZ;
  info->x_char = 0;
  info->event = 0;
  info->count = 0;
  info->blocked_open = 0;
  info->tqueue.routine = do_softint;
  info->tqueue.data = info;
  info->tqueue_hangup.routine = do_serial_hangup;
  info->tqueue_hangup.data = info;
  info->callout_termios =callout_driver.init_termios;
  info->normal_termios = serial_driver.init_termios;
  info->open_wait = 0;
  info->close_wait = 0;
  info->delta_msr_wait = 0;
  info->icount.cts = info->icount.dsr = 
	info->icount.rng = info->icount.dcd = 0;
  info->next_port = 0;
  info->prev_port = 0;
  
  /*
   * Initialize all the shadow registers to reflect current (bootup) h/w state.
   * This way we won't clobber the settings established for console printk's
   * during boot.
   */
  serial_save_registers(info);

  autoconfig(info);

  // Next, Be a good citizen and do what it takes to insure
  // the user gets accurate info if they type "cat /proc/ioports"
  request_region(info->port,UARTn_IO_RANGE,"serial");

  printk(KERN_INFO "ttyS%d%s%s at 0x%04x (irq = %d) is a DSPLinux UART\n", 
	 info->line, 
	 (info->flags & ASYNC_FOURPORT) ? " FourPort" : "",
	 (info->flags & ASYNC_PCI)      ? " PCI" : "",
	 info->port, info->irq);
}

/*
 * The serial driver boot-time initialization code!
 */

int serial_dsplinux_init(void)
{
	int i;
	struct async_struct * info;
	
	init_bh(SERIAL_BH, do_serial_bh);
	timer_table[RS_TIMER].expires = 0;

	show_serial_version();

	/* Initialize the tty_driver structure */
	
	memset(&serial_driver, 0, sizeof(struct tty_driver));
	serial_driver.magic = TTY_DRIVER_MAGIC;
	serial_driver.name = "ttyS";
	serial_driver.major = TTY_MAJOR;
	serial_driver.minor_start = 64;
	serial_driver.num = NR_PORTS;
	serial_driver.type = TTY_DRIVER_TYPE_SERIAL;
	serial_driver.subtype = SERIAL_TYPE_NORMAL;
	serial_driver.init_termios = tty_std_termios;
	serial_driver.init_termios.c_cflag =
		B115200 | CS8 | CREAD | HUPCL | CLOCAL;
	serial_driver.flags = TTY_DRIVER_REAL_RAW;
	serial_driver.refcount = &serial_refcount;
	serial_driver.table = serial_table;
	serial_driver.termios = serial_termios;
	serial_driver.termios_locked = serial_termios_locked;

	serial_driver.open = rs_open;
	serial_driver.close = rs_close;
	serial_driver.write = rs_write;
	serial_driver.put_char = rs_put_char;
	serial_driver.flush_chars = rs_flush_chars;
	serial_driver.write_room = rs_write_room;
	serial_driver.chars_in_buffer = rs_chars_in_buffer;
	serial_driver.flush_buffer = rs_flush_buffer;
	serial_driver.ioctl = rs_ioctl;
	serial_driver.throttle = rs_throttle;
	serial_driver.unthrottle = rs_unthrottle;
	serial_driver.set_termios = rs_set_termios;
	serial_driver.stop = rs_stop;
	serial_driver.start = rs_start;
	serial_driver.hangup = rs_hangup;

	/*
	 * The callout device is just like normal device except for
	 * major number and the subtype code.
	 */
	callout_driver = serial_driver;
	callout_driver.name = "cua";
	callout_driver.major = TTYAUX_MAJOR;
	callout_driver.subtype = SERIAL_TYPE_CALLOUT;

	if (tty_register_driver(&serial_driver))
		panic("Couldn't register serial driver\n");
	if (tty_register_driver(&callout_driver))
		panic("Couldn't register callout driver\n");
	
	for (i = 0, info = rs_table; i < NR_PORTS; i++,info++) {
		init_port(info, i);
	};

	return 0;
}

#ifdef MODULE
int init_module(void)
{
  serial_dsplinux_console_init();
  return serial_dsplinux_init();
}

void cleanup_module(void) 
{
	unsigned long flags;
	int e1, e2;
	int i;

	/* printk("Unloading %s: version %s\n", serial_name, serial_version); */
	save_flags_cli(flags);
	timer_active &= ~(1 << RS_TIMER);
	timer_table[RS_TIMER].fn = NULL;
	timer_table[RS_TIMER].expires = 0;
	if ((e1 = tty_unregister_driver(&serial_driver)))
		printk("SERIAL: failed to unregister serial driver (%d)\n",
		       e1);
	if ((e2 = tty_unregister_driver(&callout_driver)))
		printk("SERIAL: failed to unregister callout driver (%d)\n", 
		       e2);
	restore_flags(flags);

	for (i = 0; i < NR_PORTS; i++) {
		release_region(rs_table[i].port, UARTn_IO_RANGE);
	}


	if (tmp_buf) {
		free_page((unsigned long) tmp_buf);
		tmp_buf = NULL;
	}
}
#endif /* MODULE */
