/***********************************************************************

af_ridgerun.h

This module provides generic support for the app level BSD socket
interface to communicate with other modules in kernel mode via the
"layercomm" api. See layercomm.c/h. Layercomm provides handle-based
communications between to opaque endpoints.

This module should remain data/link agnostic for reusablility.

5/7/01 - This is to be used with the USB driver.

Installation
------------
*** This requires kernel 2.0.38 with net/netsyms.c modified to export memcpy_toiovec.

An address family number is passed to the socket() api. This module
registers as family AF_RIDGERUN (defined below). If this code is loaded
as a module, you can pass an int parameter "familynum" to change this.
"familynum" is defined at the top of af_ridgerun.c.

Applications can include this file an use AF_RIDGERUN hardcoded. Or optionally,
an application can query the protocol number by name as follows:

    int p = getprotobyname ( "RIDGERUN" );

In order to support this, add the following line to /etc/protocols:

ridgerun	13	RIDGERUN	# RidgeRun kernel comms

Usage
-----
This module does not perform internal buffering. This means that while
non-blocking read is supported, you should refrain from using it because the
underlying layercomm api receives data as a callback. Non-blocking write
is usable.

This provides support for creating a socket, binding, listening, accepting,
reading and writing.

Select and other stuff not supported yet.

TODO - add a buffer posting mode to layercomm or double buffered to af_ridgerun.c

***********************************************************************/

/****************************************************************************
                                 Defines
****************************************************************************/

#define AF_RIDGERUN 13

struct sockaddr_rr {
    int family;
    char servicename [ 16 ];         /* Other driver registers through layercomm */
};
