/*
 *   CCD Control header.
 *    
 *   Derived from TI's CCDC.h
 *       
 *   This header is to be used by application programs needing to control the
 *   CCDC register on the DSC21.
 *            
 *   Author:  Marcus Smith
 *   Date: 05/14/2001
 *             
 */

#ifndef __CCDC_H__
#define __CCDC_H__
 
#include <asm/ioctl.h>
 
 
/* List of ioctl commands. */
 
#define CCDC_MAGIC_NUMBER 'D'

struct CCDCEnable
{
   unsigned char bVd;            // [IN] Vd input/output enable 
   unsigned char bRawData;       // [IN] Raw data output enable

};
#define CCDC_ENABLE _IOW(CCDC_MAGIC_NUMBER, 0x0, sizeof(struct CCDCEnable))

#define CCDC_SET_LINE_PER_FRAME _IOW(CCDC_MAGIC_NUMBER, 0x1, sizeof(unsigned short))

struct CCDCSetRawDataMode
{
   unsigned short nX;              // [IN] right position of the window
   unsigned short nY;              // [IN] top position of the window
   unsigned short nWidth;          // [IN] window width
   unsigned short nHeight;         // [IN] window height
   unsigned char nHevenPttn;       // [IN] horizontal down-sampling pattern for even line
   unsigned char nHoddPttn;        // [IN] horizontal down-sampling pattern for odd line
   unsigned char nVPttn;           // [IN] vertical down-sampling pattern
   unsigned char bInterlace;       // [IN] TRUE:interlace mode, FALSE: non-interlace mode
   unsigned char b2LineAlternate;  // [IN] TRUE:2-line alternated, FALSE:1-line (for bInterlace == TRUE)

};
#define CCDC_SET_RAW_DATA_MODE _IOW(CCDC_MAGIC_NUMBER, 0x2, sizeof(struct CCDCSetRawDataMode))
#define CCDC_GET_FRAME         _IOR(CCDC_MAGIC_NUMBER, 0x3, sizeof(unsigned char))
#define CCDC_SET_SDRAM_ADDR    _IOW(CCDC_MAGIC_NUMBER, 0x4, sizeof(unsigned long))

typedef enum
{
   eDraft,
   eFine
} modeType;
#define CCDC_SET_MODE _IOW(CCDC_MAGIC_NUMBER, 0x5, sizeof(modeType))

#define CCDC_IOC_MAXNR 0x5

/* Major numbers, (from Documentation/devices.txt local/experimental list)  */
 
#define CCD_CONTROL_MAJOR     122

#endif /* __CCDC_H__ */
