/*
 
  clockc_ioctl.h
 
  This file represents the DSC21 timer peripheral.  

  Copyright (C) 2001, RidgeRun, Inc.
 
*/ 
#ifndef __CLOCKC_IOCTL_H__
#define __CLOCKC_IOCTL_H__
 
#include <asm/ioctl.h>
 

/*
 *   Generic disable/enable defines.
 */
#define CLOCKC_DISABLE 0
#define CLOCKC_ENABLE 1

/*
 * Enums for various Clocks
 */

typedef enum {
#if defined (CONFIG_ARCH_DSC21)
	ePLLARM,
#endif
	ePLLDSP,
	ePLLSDR
} DPLL_t;


#if defined (CONFIG_ARCH_DSC21)

typedef enum {
	eARM_DIVDWN = 7,
	eARM_N,
	eARM_M = 14
} PLL_ARM_t;

#define ARM1_BASE 0
#define ARM2_BASE 16
#define ARM3_BASE 32
typedef enum {
	eA_DSPC = ARM1_BASE + 8,
	eA_HPIB,
	eA_INTC = ARM1_BASE + 12,
	eA_IM,
	eA_RM_BUSC,
	eA_WDT = ARM2_BASE,
	eA_TMR0,
	eA_TMR1,
	eA_TMR2,
	eA_TMR3,
	eA_VENCIF,
	eA_SDRC,
	eA_PRV,
	eA_OSD,
	eA_LOS,
	eA_CCDC,
	eA_SERI = ARM3_BASE,
	eA_GIO,
	eA_EXDIF,
	eA_EXBC,
	eA_IRD = ARM3_BASE + 10,
	eA_USB,
	eA_UART0,
	eA_UART1,

	eARM_ALL_CLK
} ARM_t;

typedef enum {
	eDSP_DIVDWN = 7,
	eDSP_N,
	eDSP_M = 10
} PLL_DSP_t;

typedef enum {
	eSDRAM_DIVDWN = 7,
	eSDRAM_N,
	eSDRAM_M = 10
} PLL_SDRAM_t;

#define CLOCK_DSP_BASE 0
#define CLOCK_SDRAM_BASE 16
#define CLOCK_ENCODER_BASE 32
#define CLOCK_USB_BASE 48
#define CLOCK_CCD_BASE 64
typedef enum {
	// Clock DSP
	eD_IMG = CLOCK_DSP_BASE,
	eD_IMX,
	eD_VLC,
	eD_HPIB = CLOCK_DSP_BASE + 9,
	eD_LEAD,

	// Clock SDRAM
	eS_BUSC = CLOCK_SDRAM_BASE + 1,
	eS_SDRC,
	eS_PRV,
	eS_OSD,
	eS_LOS,
	eS_CCDC,
	eS_IMG = CLOCK_SDRAM_BASE + 8,

	// Clock Encoder
	eE_WDT = CLOCK_ENCODER_BASE,
	eE_TMR0,
	eE_TMR1,
	eE_TMR2,
	eE_TMR3,
	eE_EXBC = CLOCK_ENCODER_BASE + 6,
	eE_DAC0 = CLOCK_ENCODER_BASE + 8,
	eE_DAC1,
	eE_VENC,
	eE_OSD = CLOCK_ENCODER_BASE + 14,

	// Clock USB
	eU_IRDA = CLOCK_USB_BASE,
	eU_USB,

	// Clock CCD
	eC_PREVIEW = CLOCK_CCD_BASE,
	eC_LOSSLESS,
	eC_CCDC,
	eC_LLCLKSEL = CLOCK_CCD_BASE + 7,
	eC_PCLK,

	eCMISC_ALL_CLK
} CMISC_t;


typedef struct {
	ARM_t			name;
	int				state;              // ENABLE/DISABLE
} arm_control_t;

typedef struct {
	CMISC_t			name;
	int				state;              // ENABLE/DISABLE
} cmisc_control_t;

#elif defined (CONFIG_ARCH_DSC24)

typedef enum {
	eDPN,
	eDPM = 4
} PLLDSP_t;

typedef enum {
	eSPN,
	eSPM = 4
} PLLSDR_t;

typedef enum {
	eCLK_DIV_BY_1,
	eCLK_DIV_BY_2,
	eCLK_DIV_BY_4,
	eCLK_DIV_BY_8
} CKCTL_DIV_t;

typedef enum {
	eCADIV = 4,
	eSCDIV = 8,
	eDCDIV = 10
} CLKC_DIV_t;

typedef struct {
	CLKC_DIV_t		name;
	CKCTL_DIV_t		value;
} divider_t;
 

typedef enum {
	COSDINV,
	eCCINV,
	eCMUX,
	eEXCINV,
	eCAMUX = 6,
	eCASEL,
	eFINSEL = 12,
	eBPSMD,
	eSLPMD,
	eOSC48,
	eCLKC_ALL_CLK
} CLKC_t;

#define MOD1_BASE 0
#define MOD2_BASE 16
#define MOD3_BASE 32
typedef enum {
// MOD 1
	eCAIM = MOD1_BASE,
	eCACORE,
	eCINTC,
	eCGIO,
	eCSDRC,
	eCHPIB,
	eCVLC,
	eCIMX,
	eCIMG,
	eC5409,
	eCERAM,
// MOD 2		
	eCWDT = MOD2_BASE,
	eCTMR0,
	eCTMR1,
	eCTMR2,
	eCTMR3,
	eUAT0,
	eUAT1,
	eCUSB,
// MOD 3
	eCEMIF = MOD3_BASE + 1,
	eCHSTIF,
	eCVIF,
	eCSIF0,
	eCSIF1,
	eCSIF2,
	eCBUSC,
	eCOSD,
	eCBRST,
	eBCINV,
	eBAINV,
	eMOD_ALL_CLK
} MOD_t;

typedef struct {
	MOD_t 			name;
	int				state;              // ENABLE/DISABLE
} mod_control_t;

typedef struct {
	CLKC_t			name;
	int				state;              // ENABLE/DISABLE
} clkc_control_t;

#else
#error - Forced Compile error - Not defined either DSC-21 or DSC-24
#endif

/*
*   Structures for ioctl messages.
*   */
typedef struct {
	__u8 			frequency;
	DPLL_t 			DPLL;
} clock_freq_t;
 
typedef struct {
	__u8 			multiplier;
	__u8 			divider;
#if defined (CONFIG_ARCH_DSC21)
	int 			divdown;            // ENABLE/DISABLE
#endif
	DPLL_t 			DPLL;
} clock_md_t;
 
#if defined (CONFIG_ARCH_DSC21)
#define MULTIPLIER_MAX 31
#elif defined (CONFIG_ARCH_DSC24)
#define MULTIPLIER_MAX 20
#endif
/*
  Defines for Clock enable/disable calls.
*/
#define DISABLE	0
#define ENABLE	1

/*
  Defines for DpllSetClockandLock.
*/
#define PLL_DISABLE 0
#define PLL_ENABLE 0x10

/*
  Defines for Drive*DspReset.  
*/
#define DSP_CLOCKC_RESET 0
#define DSP_ENABLE 1

/*
  Defines for SetIdle.  Sets ARM_IDLECT1 WKUP_MODE.
*/
#define IDLE_PIN_WAKEUP 0
#define IDLE_INT_WAKEUP 1

/*
  Defines for ARMPER_nRST pin low/high voltage levels.
*/
#define LOW_LEVEL 0
#define HIGH_LEVEL 1


/*
  Ioctl defines.
*/

#define CLOCKC_MAGIC_NUMBER 'c'

#define CLOCKC_SET_LOCK_DPLL_BY_FREQ        _IOW(CLOCKC_MAGIC_NUMBER, 0, sizeof(clock_freq_t)) 
#define CLOCKC_SET_LOCK_DPLL        _IOW(CLOCKC_MAGIC_NUMBER, 1, sizeof(clock_md_t)) 

#if defined (CONFIG_ARCH_DSC21)

#define CLOCKC_SET_ARM				_IOW(CLOCKC_MAGIC_NUMBER, 2, sizeof(arm_control_t))
#define CLOCKC_SET_CMISC             _IOW(CLOCKC_MAGIC_NUMBER, 3, sizeof(cmisc_control_t))

#define CLOCKC_IOC_MAXNR 3

#elif defined (CONFIG_ARCH_DSC24)

#define CLOCKC_SET_DIVIDER          _IOW(CLOCKC_MAGIC_NUMBER, 2, sizeof(divider_t))
#define CLOCKC_SET_MOD				_IOW(CLOCKC_MAGIC_NUMBER, 3, sizeof(mod_control_t))
#define CLOCKC_SET_CLKC				_IOW(CLOCKC_MAGIC_NUMBER, 4, sizeof(clkc_control_t))

#define CLOCKC_IOC_MAXNR 4
#endif /* DSC 24 */


/* Major numbers, (from Documentation/devices.txt local/experimental list)  */
 

#define CLOCKC_MAJOR 119
#define CLOCKC_MINOR 0
 
#endif 		//ifndef __CLOCKC_IOCTL_H__
