/*
  DSP Control header.

  Derived from TI's DSPC.h
  Copyright (C) 2001, RidgeRun, Inc.

*/

#ifndef __DSPC_IOCTL_H__
#define __DSPC_IOCTL_H__
    
#include <asm/ioctl.h>

/*
  See DSC21 Register Manual for definitions.
*/

/*  IOCTL numbers (doesn't conflict with Documentation/ioctl_number.txt list) */
#define DSP_MAGIC_NUMBER 'D'

#define DSP_WAKEUP    _IO(DSP_MAGIC_NUMBER, 0)
#define DSP_SLEEP     _IO(DSP_MAGIC_NUMBER, 1)
#define DSP_GO        _IO(DSP_MAGIC_NUMBER, 2)
#define DSP_INT_NMI   _IO(DSP_MAGIC_NUMBER, 3) // generate NMI on DSP
#define DSP_INT_INT0  _IO(DSP_MAGIC_NUMBER, 4) // generate INT0 on DSP
#define DSP_INT_HPI   _IO(DSP_MAGIC_NUMBER, 5) // generate HPI interrupt on DSP
#define DSP_INT_AINT  _IO(DSP_MAGIC_NUMBER, 6)
#define DSP_REQ_INT   _IOR(DSP_MAGIC_NUMBER, 7, char) // is DSP requesting an interrupt?
#define DSP_ARMCR     _IO(DSP_MAGIC_NUMBER, 8)
#define DSP_DSPCR     _IO(DSP_MAGIC_NUMBER, 9)
#define DSP_START     _IO(DSP_MAGIC_NUMBER, 10)
#define DSP_RESET     _IO(DSP_MAGIC_NUMBER, 11)
#define DSP_SET_COMM_BASE_ADDRESS      _IO(DSP_MAGIC_NUMBER, 12)
#define DSP_SET_DSPCR_BASE_ADDRESS     _IO(DSP_MAGIC_NUMBER, 13)
#define DSP_SET_DSPCCR_BASE_ADDRESS    _IO(DSP_MAGIC_NUMBER, 14)
#define DSP_SET_ARMCR_BASE_ADDRESS     _IO(DSP_MAGIC_NUMBER, 15)
#define DSP_SET_ARMCCR_BASE_ADDRESS    _IO(DSP_MAGIC_NUMBER, 16)
#define DSP_SET_BUF_SIZE               _IO(DSP_MAGIC_NUMBER, 17)

#define DSP_IOC_MAXNR 17

/* 
   Major/minor numbers, 
   (from Documentation/devices.txt local/experimental list).

   dsp_control           120, 2   -- control DSP state
   dsp_command_to_dsp    120, 0   -- pass data to and from DSP
   dsp_command_to_arm    120, 1   -- pass reply of DSP command to and from DSP using second data channel
   dsp_download          120, 3   -- download code to DSP using HPI16

   NOTE: Always keep the message passing minor numbers
   at the beginning of this list.  That way, you can
   have local data for as many minor numbers as you
   want without changing any code, just DSP_DATA_MINOR_MAX.  
*/

#define DSP_CONTROL_MAJOR     120
#define DSP_DATA_MINOR          0
#define DSP_REPLY_MINOR         1
#define DSP_CONTROL_MINOR       2
#define DSP_DOWNLOAD_MINOR      3

#define DSP_DATA_MINOR_MAX      2

#endif				// __DSPC_IOCTL_H__
