/*
 * File: dspi.h
 *
 * Defines the kernel interface for dsp communications.
 *
 * Copyright (C) 2001 RidgeRun, Inc.
 * Author: Clint Medley <cmedley@ridgerun.com> 10/30/01
 *         Based on the original hpi.h by Gordon McNutt <gmcnutt@ridgerun.com>
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS  PROVIDED  ``AS  IS''  AND   ANY  EXPRESS  OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Please report all bugs/problems to the author or <support@dsplinux.net>
 *
 * key: RRGPLCR (do not remove)
 *
 */

#ifndef dspi_h
#define dspi_h

#include <linux/types.h>
#include <asm/arch/hardware.h>

#if defined(CONFIG_ARCH_DSC21) || defined(CONFIG_ARCH_DSC24)
#define DSPI_DSP_START   0 
#define DSPI_DSP_END     0x7fff
#elif defined(CONFIG_ARCH_ORION)
#define DSPI_DSP_START   0x2000
#define DSPI_DSP_END     0x3fff
#endif

#define DSPI_ARM_START  DSPRAM_BASE
#define DSPI_ARM_END    DSPRAM_END

/*The dspi interface as it stands here assumes that one day we will be dealing
  with multiple dsps, and that each dsp will have a unique mapped memory area
  in the arm address space. Hence the functions and macros that take a dsp
  address have an additional paramater (cpu) to identify the dsp that the
  address applies to. Calls with a arm address do not need this extra paramater
  since the code will be able to determine the mapping based on the address
  value. 
*/

/* Revisit -- change this to test masks?*/
#define __dspi_in_range(addr, start, end) ((((__u32)(addr)) >= (start)) && \
                                           (((__u32)(addr)) <= (end)))
#define dspi_dsp_aligned(addr, cpu) ((((__u32)(addr)) % 2) == 0)
/*if we eventually use the cpu variable these will return cpu number if true,
  but for now they are a boolean*/
#define dspi_in_dsp(addr, cpu) __dspi_in_range(addr,DSPI_DSP_START,DSPI_DSP_END)
#define dspi_in_arm(addr) __dspi_in_range(addr, DSPI_ARM_START, DSPI_ARM_END)

#if defined(CONFIG_ARCH_DSC21) || defined(CONFIG_ARCH_DSC24)
#define dspi_dsp_to_arm(addr, cpu) (((__u32)(addr) << 1) + DSPI_ARM_START)
#define dspi_arm_to_dsp(addr) (((__u32)(addr) - DSPI_ARM_START) >> 1)
#elif defined(CONFIG_ARCH_ORION)
#define dspi_dsp_to_arm(addr, cpu) ((((__u32)(addr) - DSPI_DSP_START) << 1) + \
                                    DSPI_ARM_START)
#define dspi_arm_to_dsp(addr) ((((__u32)(addr) - DSPI_ARM_START) >> 1) + \
                               DSPI_DSP_START)
#endif

/* These calls use dsp addressing for loader support. The cpu paramater is a
 place holder for the possible day when we may have multiple cpus*/
extern int dspi_load_code(void *to, const void *from, int count, int cpu);
extern int dspi_dsp_memset(void *to, __u16 pattern, int count, int cpu);

/* Used for byte user space accesses to support DSP processes. */
extern int dspi_memcpy_from_dsp(void *to, const void *from, int count);
extern int dspi_memcpy_to_dsp(void *to, const void *from, int count);
extern void dspi_put_user(unsigned long x, void *y, int size);
extern unsigned long dspi_get_user(const void *y, int size);
extern unsigned int dspi_csum_partial_copy_from_dsp(const char *src, char *dst,
                                                    int len, int sum);

#endif //dspi_h
