/*
 
  gio_ioctl.h
 
  This file represents the General Purpose I/O peripheral.  

  Copyright (C) 2001, RidgeRun, Inc.
 
*/ 
#ifndef __GIO_IOCTL_H__
#define __GIO_IOCTL_H__
 
#include <asm/ioctl.h>
 
/* List of ioctl commands. */
 
#define GIO_MAGIC_NUMBER 'g'

/*
  The GIO direction can be input or output.  Default is input.
*/
#define GIO_OUTPUT 0
#define GIO_INPUT 1

#define GIO_DIRECTION _IO(GIO_MAGIC_NUMBER, 0)
 
/*
  To set or clear an output bit, use these ioctls.  If the direction
  is input, then the return value from the call will be the value of
  the pin (0 or 1, anything negative is an error code).
*/
#define GIO_BITSET _IO(GIO_MAGIC_NUMBER, 1)
#define GIO_BITCLR _IO(GIO_MAGIC_NUMBER, 2)

#define GIO_NOINTERRUPT 0
#define GIO_INTERRUPT 1

#define GIO_IRP _IO(GIO_MAGIC_NUMBER, 3)

#define GIO_READ _IO(GIO_MAGIC_NUMBER, 4)
 
#define GIO_COUNT_PULSE _IO(GIO_MAGIC_NUMBER, 5)

#if defined(CONFIG_ARCH_DSC2X)
/* Only DSC platforms support these following functions */
#if defined(CONFIG_ARCH_DSC21)
/*
  Pin GIO11 can be normal or the CCDC shutter output function.

  Pins GIO12 and GIO13 can take different values to set clock output on them.
*/
#	define GIO_ORDINARY          0
#	define GIO_FUNCTION          1

#	define GIO_NORMAL            0
#	define GIO_13_5_CLOCK        1
#	define GIO_24_CLOCK          2
#	define GIO_SER0_FREERUN      2
#	define GIO_SER1_FREERUN      3

#elif defined(CONFIG_ARCH_DSC24)
/*
  GIO pins 8-20 can be set as functions.  Default is ordinary
  GIO.  

  Pins GIO16 and GIO17 can take different values to set clock output on them.
*/
#	define GIO_ORDINARY 0
#	define GIO_FUNCTION 1

#	define GIO_DIVIDED_CLOCK     1
#	define GIO_MXI_CLOCK         2
#	define GIO_MXI_CLOCK_DIVIDED 3

#else
#error Not defined as either DSC21 or DSC24.
#endif

#define GIO_FSEL _IO(GIO_MAGIC_NUMBER, 6)

/*
  Transfer rate setting bit.  Sets the division ratio of clock output 
  of GIO16 or GIO17 when pin is set to GIO divided clock output.
*/
#define GIO_BITRATE _IO(GIO_MAGIC_NUMBER, 7)

#define DIR0_OFFSET    0x00
#define DIR1_OFFSET    0x01
#define INV0_OFFSET    0x02
#define INV1_OFFSET    0x03
#define BITSET0_OFFSET 0x04
#define BITSET1_OFFSET 0x05
#define BITCLR0_OFFSET 0x06
#define BITCLR1_OFFSET 0x07
#define IRQPORT_OFFSET 0x08
#define MISC_OFFSET    0x09 // DSC21 only
#define FSEL_OFFSET    0x09 // DSC24 only
#define BITRATE_OFFSET 0x0a // DSC24 only

/*
  The polarity of the GIO bits can be inverted.  Default is
  not inverted.
*/
#define GIO_UNINVERTED 0
#define GIO_INVERTED 1

#define GIO_INVERSION _IO(GIO_MAGIC_NUMBER, 8)

#define GIO_IOC_MAXNR 8
#else
#define GIO_IOC_MAXNR 5
#endif
 
/* Major numbers, (from Documentation/devices.txt local/experimental list)  */
 
#define GIO_MAJOR     125
 
#endif            // __GIO_IOCTL_H__
