/*
 * hardware.h  defines for the DSC21.
 *         
 * copyright:
 *         (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 * author: Gordon McNutt <gmcnutt@ridgerun.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __ASM_ARCH_HARDWARE_H
#define __ASM_ARCH_HARDWARE_H

#include <linux/autoconf.h>    // DSC21 vs DSC24 defines

#define HARD_RESET_NOW()  { arch_hard_reset(); }

#define KERNTOPHYS(a) ((unsigned long)(&a))

/* ######## Network Camera H/W Specific Stuff (Start) ######## */
				/* LAN/WLAN */
//#define NC7XX_MACADDR_BASE	0x02004000	/* 2nd Partition on FLASH */
#define NC7XX_MACADDR_BASE	0x02005F00	/* 2nd Partition on FLASH */
				/* VIDEO */
#define IMG_BUF_START           0x08000000      /* Start address of image   */
						/* buffer (CMOS, DSP...)    */
#define IMG_DATA_SIZE           0x96000		/* The Max. frame size      */
						/* 640*480*2 (Yuv, not RGB) */
#define IMG_FRAME_NUM           3		/* The Max. frame number in */
						/* image buffer (CMOS...)   */
/* ######## Network Camera H/W Specific Stuff (End  ) ######## */

#ifdef CONFIG_ARCH_DSC2X

/* ARM clock rate (MHz) */
#if defined(CONFIG_ARCH_DSC21)
#define DSC21_CLOCK_RATE     37125000
#endif

/* external bus */
#define DSC21_EXTBUS_RESET    0x00030a00
#define DSC21_EXTBUS_MODE     0x00030a04
#define DSC21_EXTBUS_INTEN    0x00030a08

/* uarts (note that the register fields aren't needed by the standard linux
   serial driver -- we only use them for our initial debugging console) */
#define DSC21_UART0       0x00030380
#define DSC21_UART0_DTRR  0x00030380
#define DSC21_UART0_BRSR  0x00030382
#define DSC21_UART0_MSR   0x00030384
#define DSC21_UART0_RFCR  0x00030386
#define DSC21_UART0_TFCR  0x00030388
#define DSC21_UART0_LCR	  0x0003038a
#define DSC21_UART0_SR 	  0x0003038c

#define DSC21_UART1       0x00030400
#define DSC21_UART1_DTRR  0x00030400
#define DSC21_UART1_BRSR  0x00030402
#define DSC21_UART1_MSR   0x00030404
#define DSC21_UART1_RFCR  0x00030406
#define DSC21_UART1_TFCR  0x00030408
#define DSC21_UART1_LCR	  0x0003040a
#define DSC21_UART1_SR 	  0x0003040c

/* timers */
#define DSC21_TIMER0_MODE    0x00030000
#define DSC21_TIMER0_SEL     0x00030002
#define DSC21_TIMER0_SCAL    0x00030004
#define DSC21_TIMER0_DIV     0x00030006
#define DSC21_TIMER0_TRG     0x00030008
#define DSC21_TIMER0_CNT     0x0003000a
#define DSPLINUX_TIMER0_CNT  DSC21_TIMER0_CNT

#define DSC21_TIMER1_MODE    0x00030080
#define DSC21_TIMER1_SEL     0x00030082
#define DSC21_TIMER1_SCAL    0x00030084
#define DSC21_TIMER1_DIV     0x00030086
#define DSC21_TIMER1_TRG     0x00030088
#define DSC21_TIMER1_CNT     0x0003008a

#define DSC21_TIMER2_MODE    0x00030100
#define DSC21_TIMER2_SEL     0x00030102
#define DSC21_TIMER2_SCAL    0x00030104
#define DSC21_TIMER2_DIV     0x00030106
#define DSC21_TIMER2_TRG     0x00030108
#define DSC21_TIMER2_CNT     0x0003010a

#define DSC21_TIMER3_MODE    0x00030180
#define DSC21_TIMER3_SEL     0x00030182
#define DSC21_TIMER3_SCAL    0x00030184
#define DSC21_TIMER3_DIV     0x00030186
#define DSC21_TIMER3_TRG     0x00030188
#define DSC21_TIMER3_CNT     0x0003018a

/* interrupt */
#define DSC21_FIQ_STATUS     0x00030580
#if defined(CONFIG_ARCH_DSC21)
#define DSC21_IRQ0_STATUS    0x00030582
#define DSC2X_IRQ0_STATUS    0x00030582
#define DSC21_IRQ1_STATUS    0x00030584
#define DSC2X_IRQ1_STATUS    0x00030584
#endif
#define DSC21_FIQ_ENABLE     0x000305a0
#if defined(CONFIG_ARCH_DSC21)
#define DSC21_IRQ0_ENABLE    0x000305a2
#define DSC2X_IRQ0_ENABLE    0x000305a2
#define DSC21_IRQ1_ENABLE    0x000305a4
#define DSC2X_IRQ1_ENABLE    0x000305a4
#endif
#define DSC21_INTIDR         0x000305c0
#define DSC2X_INTIDR         0x000305c0

/*  GIO  */
#define DSC21_GIO_DIR0       0x00030600
#define DSC21_GIO_DIR1       0x00030602
#define DSC21_GIO_INV0       0x00030604
#define DSC21_GIO_INV1       0x00030606
#define DSC21_GIO_BITSET0    0x00030608
#define DSC21_GIO_BITSET1    0x0003060a
#define DSC21_GIO_BITCLR0    0x0003060c
#define DSC21_GIO_BITCLR1    0x0003060e
#define DSC21_GIO_IRQPORT    0x00030610
#define DSC21_GIO_MISC       0x00030612

#define GIO_0_BIT      (1 << 0)
#define GIO_1_BIT      (1 << 1)
#define GIO_2_BIT      (1 << 2)
#define GIO_3_BIT      (1 << 3)
#define GIO_4_BIT      (1 << 4)
#define GIO_5_BIT      (1 << 5)
#define GIO_6_BIT      (1 << 6)
#define GIO_7_BIT      (1 << 7)
#define GIO_ALL_BITS   0xff

#if defined(CONFIG_ARCH_DSC21)
#define MAX_GIO 31
#endif

#define MAX_I2C 31

/* reference */
/////////////////////////////////////////////////
//
//	register and SDRAM map
//
//  (c) VSA/TITRDC   06/1999
//
/////////////////////////////////////////////////

// memory mapped registers
#define TIMER0_REGISTER_BASE    0x00030000		// timer0
#define TIMER1_REGISTER_BASE	0x00030080		// timer1
#define TIMER2_REGISTER_BASE	0x00030100		// timer2
#define TIMER3_REGISTER_BASE	0x00030180		// timer3
#define SERIAL0_REGISTER_BASE	0x00030200		// serial0
#define SERIAL1_REGISTER_BASE	0x00030280		// serial1
#if defined(CONFIG_ARCH_DSC21)
#define WDT_REGISTER_BASE	0x00030300		// watch dog timer
#endif
#define UART0_REGISTER_BASE	0x00030380		// UART0
#define UART1_REGISTER_BASE	0x00030400		// UART0
#define USB_REGISTER_BASE	0x00030480		// USB
#if defined(CONFIG_ARCH_DSC21)
#define IRDA_REGISTER_BASE	0x00030500		// IrDA
#endif
#define INTCTRL_REGISTER_BASE	0x00030580		// INTC
#define GIO_REGISTER_BASE	0x00030600		// GIO
#define OSD_REGISTER_BASE	0x00030680		// OSD
#define DSPC_REGISTER_BASE	0x00030700		// DSP controller
#if defined(CONFIG_ARCH_DSC21)
#define CCDC_REGISTER_BASE	0x00030780		// CCD controller
#define PREVIEW_REGISTER_BASE	0x00030800		// preview engine
#define BURSTC_REGISTER_BASE	0x00030880		// burst mode compression
#define VIDEOENC_REGISTER_BASE	0x00030900		// NTSC/PAL encoder
#endif
#define SDRAMC_REGISTER_BASE	0x00030980		// SDRAM controller
#define EXBC_REGISTER_BASE	0x00030A00		// external bus controller

#if defined(CONFIG_ARCH_DSC21)
#define CLOCKC_REGISTER_BASE	0x00030A80		// clock controller
#define BUSC_REGISTER_BASE	0x00030B00		// bus controller
#define DMAIF_REGISTER_BASE	0x00030B80		// external bus DMA I/F
#endif

// DSPRAM
#if defined(CONFIG_ARCH_DSC21)
#define DSPRAM_BASE         0x00100000  // DSPRAM base address
#define DSPRAM_END          0x0010ffff
#endif

// SDRAM address (real byte address)
#if defined(CONFIG_ARCH_DSC21)
#define SDRAM_BASE          0x08000000  // sdram base address
#define SDRAM_SIZE          0x01000000  // hard-code 16MB for now
#define SDRAM_PREVIEW       0x00000000  // output from preview engine
#define SDRAM_RAWDATA       0x00100000  // output from CCDC
#define SDRAM_WORK1         0x00100000  // WORK area #1 for burst-capture & decoded data
#define SDRAM_WORK2         0x00500000  // WORK area #2 for burst-capture & decoded data
#define SDRAM_ENCODED1      0x00900000  // work area #1 for encoded data
#define SDRAM_ENCODED2      0x00B00000  // work area #2 for encoded data
#define SDRAM_BURSTDATA     0x00B00000  // BURST-compressed data start address
#define SDRAM_BURSTDATA_END 0x00EFFFFF  // BURST-compressed data end address
#define SDRAM_OSD           0x00F00000  // OSD bitmap start address

// CARD map (tentative)
#define CARD_BASE               0x04000000              // CFC card base address
#define CARD_ENCODED_START      0x00000800              // encoded image start address
#define CARD_ENCODED_SLOT_SIZE  0x00044000              // encoded image slot size

#endif /* CONFIG_ARCH_DSC21 */

#endif /* CONFIG_ARCH_DSC2X */

#endif  // __ASM_ARCH_HARDWARE_H
