/*****************************************************************************
  hpi.h:
        Kernel interface to the ARM/DSP HPI bridge.
  copyright:
        (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
  author: 
        Gordon McNutt <gmcnutt@ridgerun.com>

  This program is free software; you can redistribute  it and/or modify it
  under  the terms of  the GNU General  Public License as published by the
  Free Software Foundation;  either version 2 of the  License, or (at your
  option) any later version.
 
  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
  You should have received a copy of the  GNU General Public License along
  with this program; if not, write  to the Free Software Foundation, Inc.,
  675 Mass Ave, Cambridge, MA 02139, USA.
  ****************************************************************************/
#ifndef hpi_h
#define hpi_h

#include <linux/types.h>
#include <asm/arch/hardware.h>

#define in_fake_dsp(addr) __in_range(addr, FAKE_START, FAKE_END)
/* Revisit -- change these to test masks? */
#define __in_range(addr, start, end) ((((__u32)(addr)) >= (start)) && \
                                      (((__u32)(addr)) < (end)))

#define dsp_aligned(addr) ((((__u32)(addr)) % 2) == 0)

#ifndef CONFIG_ARCH_ORION
#define HPI8_START      0
#define HPI8_END        0x8000
#define HPI16_START     0x100000
#define HPI16_END       0x11ffff

#define ADDRESS_DSP_RAM_AS_BYTES

#ifdef ADDRESS_DSP_RAM_AS_BYTES
#  define FAKE_START      0xfffd0000
#  define FAKE_END        0xfffe0000
/* These macros expand the DSP word addresses to byte addresses. */
#define hpi8_to_fake(addr) ((addr) * 2 + FAKE_START)
#define fake_to_hpi8(addr) (((addr) - FAKE_START) / 2)
#else //ADDRESS_DSP_RAM_AS_BYTES
#define FAKE_START      0xfffd0000
#define FAKE_END        0xfffd8000
/* These macros leave the DSP addresses as word addresses. */
#define hpi8_to_fake(addr) ((addr) + FAKE_START)
#define fake_to_hpi8(addr) ((addr) - FAKE_START)
#endif //ADDRESS_DSP_RAM_AS_BYTES

/* These macros test where an address comes from. */
#define in_hpi16(addr) __in_range(addr, HPI16_START, HPI16_END)
#define in_hpi8(addr) __in_range(addr, HPI8_START, HPI8_END)

/* These macros convert between HPI8 & HPI16 (used to support the loader). */
#define hpi8_to_hpi16(addr) ((addr) * 2 + DSPRAM_BASE)
#define hpi16_to_hpi8(addr) (((addr) - DSPRAM_BASE) / 2)

/* Used across HPI16 to support the loader. */
extern int hpi16_load_code(void *to /* hpi16 */, const void *from, int count);
extern int hpi16_memset(void *to /* hpi16 */, __u16 pattern, int count);

/* Used across HPI8 to support DSP processes. */
extern int hpi8_memcpy_from_dsp(void *to, const void *from /* fake */, 
                                int count);
extern int hpi8_memcpy_to_dsp(void *to /* fake */, const void *from, 
                              int count);
extern void hpi8_put_user(unsigned long x, void * y /* fake */, int size);
extern unsigned long hpi8_get_user(const void *y /* fake */, int size);
extern unsigned int hpi8_csum_partial_copy_from_dsp(
        const char *src /* fake */, char *dst, int len, int sum);

#else //CONFIG_ARCH_ORION

#define API_START  0x2000
#define API_END    0x4000

#define FAKE_START  0xffe00000
#define FAKE_END    0xffe04000
/* These macros expand the DSP word addresses to byte addresses. */
#define api_to_fake(addr) ((((long int)(addr) - API_START) * 2) + FAKE_START)
#define fake_to_api(addr) ((((long int)(addr) - FAKE_START) / 2) + API_START)

/* These macros test where an address comes from. */
#define in_api(addr) __in_range(addr, API_START, API_END)

/* Used to support the loader. */
extern int api16_load_code(void *to /* __u16 */, const void *from, int count);
extern int api16_memset(void *to /* __u16 */, __u16 pattern, int count);

/* Used to support DSP processes. */
extern int api8_memcpy_from_dsp(void *to,const void *from /* fake */,int count);
extern int api8_memcpy_to_dsp(void *to /* fake */, const void *from, int count);
extern void api8_put_user(unsigned long x, void * y /* fake */, int size);
extern unsigned long api8_get_user(const void *y /* fake */, int size);
extern unsigned int api8_csum_partial_copy_from_dsp(const char *src /* fake */,
                                                    char *dst,int len, int sum);
#endif //CONFIG_ARCH_ORION

#endif
