/*
 * File: irqs.h:
 *
 * Copyright:
 *         (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 *
 * Author: Gordon McNutt <gmcnutt@ridgerun.com>
 *
 * Modifications:
 *    -- Oct 2001, Extended for Orion EVM
 *       Steve Kranz <skranz@ridgerun.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#ifndef __ASM_ARCH_IRQS_H__
#define __ASM_ARCH_IRQS_H__

#include <linux/autoconf.h>

#ifdef CONFIG_ARCH_DSC2X
#define DSPLINUX_IRQ_TIMER  0 /* interrupt number */
#endif

#ifdef CONFIG_ARCH_ORION
#define DSPLINUX_IRQ_TIMER  ORION_IRQ_TIMER2 /* interrupt number */

#define ORION_IRQ_TIMER0        0
#define ORION_IRQ_WATCHDOG      0
#define ORION_IRQ_TIMER1        1
#define ORION_IRQ_TIMER2        2
#define ORION_IRQ_ETHER         3
#define ORION_IRQ_GPIO0         4
#define ORION_IRQ_KBGPIO_0_7    5
#define ORION_IRQ_UART          6
#define ORION_IRQ_UART_IRDA     7
#define ORION_IRQ_KBGPIO_8_15   8
#define ORION_IRQ_GPIO3         9
#define ORION_IRQ_GPIO2         10
#define ORION_IRQ_I2C           11
#define ORION_IRQ_GPIO1         12
#define ORION_IRQ_SPI           13
#define ORION_IRQ_GPIO_4_19     14
#define ORION_IRQ_API           15

#endif

#define IRQ_TIMER DSPLINUX_IRQ_TIMER

/* flags for request_irq().  IRQ_FLG_STD is apparently a uClinux-ism,
   so I'll keep it around even though it is just a mapping for the
   real SA_INTERRUPT. -- skj 
*/
#define IRQ_FLG_STD SA_INTERRUPT

/*  If you want to use the watchdog timer, send in this
    as its interrupt flag
*/
#define FIQ_FORCE 0x2


#if defined(CONFIG_ARCH_DSC2X)

#if defined (CONFIG_ARCH_DSC21)
/* irq #s as reported by the INTIDR register (used) */
#define DSC21_IRQ_TIMER1  1
#define DSC21_IRQ_TIMER2  2
#define DSC21_IRQ_TIMER3  3
#define DSC21_IRQ_CCDVD0  4
#define DSC21_IRQ_CCDVD1  5
#define DSC21_IRQ_CCDVD2  6
#define DSC21_IRQ_ENCVD   7
#define DSC21_IRQ_SERIAL0 8
#define DSC21_IRQ_SERIAL1 9
#define DSC21_IRQ_DSP     10
#define DSC21_IRQ_UART0   11
#define DSC21_IRQ_UART1   12
#define DSC21_IRQ_USB     13
#define DSC21_IRQ_BURST   14
#define DSC21_IRQ_CARD    15
#define DSC21_IRQ_EXB0    16
#define DSC21_IRQ_EXB1    17
#define DSC21_IRQ_EXB2    18
#define DSC21_IRQ_EXB3    19
#define DSC21_IRQ_EXB4    20
#define DSC21_IRQ_EXB5    21
#define DSC21_IRQ_EXB6    22
#define DSC21_IRQ_EXB7    23
#define DSC21_IRQ_DMA     24
#define DSC21_IRQ_FLASH   25
#define DSC21_IRQ_IRRX    26
#define DSC21_IRQ_IRTX    27
#define NR_IRQS           DSC21_IRQ_IRTX
#endif /* CONFIG_ARCH_DSC21 */
#ifdef CONFIG_ARCH_DSC24
/*
  NOTE:  For convenience, I'm not going to change the
  name from DSC21_IRQ to DSC24 or DSC2X.  skj
*/
/* irq #s as reported by the INTIDR register (used) */
#define DSC21_IRQ_TIMER1  1
#define DSC21_IRQ_TIMER2  2
#define DSC21_IRQ_TIMER3  3
#define DSC21_IRQ_CCDVD0  4
#define DSC21_IRQ_CCDVD1  5
#define DSC21_IRQ_CCDVD2  6
#define DSC21_IRQ_OSD     7
#define DSC21_IRQ_SERIAL0 8
#define DSC21_IRQ_SERIAL1 9
#define DSC21_IRQ_SERIAL2 10
#define DSC21_IRQ_DSP     11
#define DSC21_IRQ_UART0   12
#define DSC21_IRQ_UART1   13
#define DSC21_IRQ_USB     14
#define DSC21_IRQ_BURSTD  15
#define DSC21_IRQ_BURSTC  16
#define DSC21_IRQ_EXIF0   17
#define DSC21_IRQ_EXIF1   18
#define DSC21_IRQ_EXB0    19
#define DSC21_IRQ_EXB1    20
#define DSC21_IRQ_EXB2    21
#define DSC21_IRQ_EXB3    22
#define DSC21_IRQ_EXB4    23
#define DSC21_IRQ_EXB5    24
#define DSC21_IRQ_EXB6    25
#define DSC21_IRQ_EXB7    26
#define NR_IRQS           DSC21_IRQ_EXB7
#endif /* CONFIG_ARCH_DSC24 */

#endif /* CONFIG_ARCH_DSC2X */

#ifdef CONFIG_ARCH_ORION

#define INT_WD_TIMER	0
#define INT_TIMER1		1
#define INT_TIMER2		2
#define	INT_GPIO0		3
#define	INT_ETH			4
#define	INT_KBGPIO_0_7	5
#define	INT_UART		6
#define	INT_UART_IRDA	7
#define	INT_KGPIO_8_15	8
#define	INT_GPIO3		9
#define	INT_GPIO2		10
#define	INT_I2C			11
#define	INT_GPIO1		12
#define	INT_SPI			13
#define	INT_GPIO_4_19	14
#define	INT_API			15
#define NR_IRQS			INT_API+1

#endif /* CONFIG_ARCH_ORION */

#endif /* __ASM_ARCH_IRQS_H__ */
