/*
 
  PrevEng.h
 
  This file represents the Preview Engine DSC21 peripheral.  The CCD
  Controller uses the Preview Engine to adjust the image in reading and/or
  writting to SDRAM.
 
  Copyright (C) 2001, RidgeRun, Inc.
 
*/ 
#ifndef __PREVENG_H__
#define __PREVENG_H__
 
#include <asm/ioctl.h>
 

/* List of ioctl commands. */
 
#define PREVENG_MAGIC_NUMBER 'D'
 
struct PrevEngComponentGainControlStruct
{
   /* gain for Gr or Ye */
   unsigned char nGR_Ye;

   /* gain for R or Mg */
   unsigned char nR_Mg;

   /* gain for Gb or Cy */
   unsigned char nGB_Cy;

   /* gain for B or G */
   unsigned char nB_G;

};
#define PREVENG_SET_COMPONENT_GAIN_CONTROL  _IOW(PREVENG_MAGIC_NUMBER, 0x0, sizeof(struct PrevEngComponentGainControlStruct))
#define PREVENG_GET_COMPONENT_GAIN_CONTROL  _IOR(PREVENG_MAGIC_NUMBER, 0x1, sizeof(struct PrevEngComponentGainControlStruct))

#define PREVENG_PREV_ENABLE      _IOW(PREVENG_MAGIC_NUMBER, 0x2, sizeof(unsigned char))

struct PrevEngDSPattern
{
   /* Is this pattern horizontal or vertical */
   unsigned char fHorizontal;

   /* Number of pattern bits. */
   unsigned char nDenominator;

   /* coefficients 63:48 */
   unsigned short nPattern1;

   /* coefficients 47:32 */
   unsigned short nPattern2;

   /* coefficients 31:16 */
   unsigned short nPattern3;

   /* coefficients 15:0 */
   unsigned short nPattern4;
};

#define PREVENG_SET_DS_PATTERN   _IOW(PREVENG_MAGIC_NUMBER, 0x3, sizeof(struct PrevEngDSPattern))

typedef enum
{
   COLOR_B,
   COLOR_G,
   COLOR_R
}ColorTriplet;

struct PrevEngGamma
{
   /* Which color ? */
   ColorTriplet nColor;

   /* Gain region values */
   unsigned char nGain[4];

   /* Gamma offset values */
   unsigned char nOffset[3];

   /* Gamma area values */
   unsigned char nArea[3];
};
#define PREVENG_SET_GAMMA        _IOW(PREVENG_MAGIC_NUMBER, 0x4, sizeof(struct PrevEngGamma))

#define PREVENG_SET_GAIN         _IOW(PREVENG_MAGIC_NUMBER, 0x5, sizeof(unsigned char))
#define PREVENG_GET_GAIN         _IOR(PREVENG_MAGIC_NUMBER, 0x6, sizeof(unsigned char))

struct PrevEngRange
{
   /* Which direction - horiz or vert */
   unsigned char bHorizontal;

   /* Start position, relative to Vd */
   unsigned short nStart;

   /* End position, relative to Vd */
   unsigned short nEnd;
};
#define PREVENG_SET_RANGE        _IOW(PREVENG_MAGIC_NUMBER, 0x7, sizeof(struct PrevEngRange))

#define PREVENG_SET_ADDR         _IOW(PREVENG_MAGIC_NUMBER, 0x8, sizeof(unsigned long))

struct PrevEngConvTableCoef
{
   unsigned short coef1;
   unsigned short coef2;
   unsigned short coef3;
   unsigned short coef4;
   unsigned short coef5;
   unsigned short coef6;
   unsigned short coef7;
   unsigned short coef8;
   unsigned short coef9;

};

#define PREVENG_GET_CONV_TABLE_COEF _IOR(PREVENG_MAGIC_NUMBER, 0x9, sizeof(struct PrevEngConvTableCoef))
#define PREVENG_SET_CONV_TABLE_COEF _IOW(PREVENG_MAGIC_NUMBER, 0xa, sizeof(struct PrevEngConvTableCoef))

//
// OSD Video Window ioctl's
// These ioctls were in the TI code exposed through their OSD module.  We've choosen to expose non-frame
// buffer functionality through the Preview Engine, and document that they are not really part of the
// Preview Engine peripheral, but of OSD.  Since the Preview Engine produces all data for OSD video,
// this was thought to be an adequate thing to do (rather than creating a completely new video driver
// to handle the few OSD video cases, that is).
//
#define PREVENG_SET_VIDEO_FRAME_MODE _IOW(PREVENG_MAGIC_NUMBER, 0xb, sizeof(unsigned char))

struct PrevEngVideoAttributes
{
   unsigned long sdram_addr;       // Address to which OSD video will be consumed
   unsigned short pixels_per_line; // Number of pixels per line of YCC data

};
#define PREVENG_SET_VIDEO_ATTRIBUTES _IOW(PREVENG_MAGIC_NUMBER, 0xc, sizeof(struct PrevEngVideoAttributes))

struct PrevEngVideoPosition
{
   unsigned short left;
   unsigned short top;
   unsigned short right;
   unsigned short bottom;

};
#define PREVENG_SET_VIDEO_POSITION _IOW(PREVENG_MAGIC_NUMBER, 0xd, sizeof(struct PrevEngVideoPosition))

//
// This ioctl is used to separately control the OSD video 0 window.  Sometimes, like in movie decomp, the preview
// engine should be turned off while the decompressor is writting bits to a memory address for OSD video consumption.
// This unsigned char parameter will specify whether OSD should be enabled (tgon bit) and whether video 0 should blank,
// which is the negate of the parameter (i.e. OSD standby, OSD blank - OSD active, OSD no blank)
//
#define PREVENG_SET_VIDEO_ENABLE _IOW(PREVENG_MAGIC_NUMBER, 0xe, sizeof(unsigned char))

#define PREVENG_IOC_MAXNR 15
 
/* Major numbers, (from Documentation/devices.txt local/experimental list)  */
 
#define PREV_ENGINE_MAJOR     121
 
#endif            // __PREVENG_H__
