/*
 * linux/include/asm-arm/arch-sa1100/time.h
 *
 * Copyright (C) 1998 Deborah Wallach.
 * Twiddles  (C) 1999 	Hugo Fiennes <hugo@empeg.com>
 * 
 * 2000/03/29 (C) Nicolas Pitre <nico@cam.org>
 *	Rewritten: big cleanup, much simpler, better HZ acuracy.
 *
 * 2001/02/20 (C) RidgeRun, Inc (http://www.ridgerun.com)
 *      Leveraged for the dsc21
 */
#ifndef __ASM_ARCH_TIME_H__
#define __ASM_ARCH_TIME_H__

#include <asm/io.h>
#include <asm/hardware.h>

// ==========================================================
#ifdef CONFIG_ARCH_ORION

#define COUNTS_PER_INTERRUPT 29687
#define reset_timer() 1
#define update_rtc() {}

extern __inline__ unsigned long gettimeoffset (void)
{
     unsigned long elapsed;
     /* 
      * Compute the elapsed count since the last interrupt.
      * Our counter counts down starting at COUNTS_PER_INTERRUPT.
      */
     elapsed = COUNTS_PER_INTERRUPT - inl(DSPLINUX_TIMER2_CNT);
     /* 
      * Convert the elapsed count to usecs. 'tick' is assigned in sched.c.
      * Normally it's 10,000 -- the # of usecs/interrupt.
      */
     return (unsigned long)((elapsed * tick) / COUNTS_PER_INTERRUPT);
}

void __inline__ setup_timer (void)
{
     #define CNTL_TIMER2 ((volatile unsigned long *)0xffff2c00)
     #define COUNTS_PER_INTERRUPT_field_shift 5
     #define AR  0x00000010
     #define ST  0x00000008
     #define PTV 0x00000003
     // Start the general purpose timer running in auto-reload mode
     // so that an interrupt is generated at the rate of HZ (param.h)
     // which typically is set to 100HZ.
     // Note: When HZ is set to 100 then COUNTS_PER_INTERRUPT should be
     //       established to reflect timer counts that occur every 10ms
     //       (giving you 100 of them per second -- 100HZ). Orion's clock
     //       is 47.5MHz and we're using a timer PTV value of 3 (3 == divide
     //       incoming frequency by 16) which then yields a 16 bit
     //       COUNTS_PER_INTERRUPT value of 29687.
     *CNTL_TIMER2 = (((COUNTS_PER_INTERRUPT-1)<<COUNTS_PER_INTERRUPT_field_shift) | AR | ST | PTV);

     // Next, Be a good citizen and do what it takes to insure
     // the user gets accurate info if they type "cat /proc/ioports"
     request_region(CNTL_TIMER2,0x8,"system timer (TIMER2)");
}

#endif /* CONFIG_ARCH_ORION */


// ==========================================================
#ifdef CONFIG_ARCH_DSC2X

// Note: When HZ (param.h) is set to 100 then COUNTS_PER_INTERRUPT
//       should be computed to reflect timer counts that occur every
//       10ms (giving you 100 of them per second, i.e 100HZ).
#define COUNTS_PER_INTERRUPT 27000

extern __inline__ unsigned long gettimeoffset (void)
{
     u16 elapsed;

     /* 
      * Compute the elapsed count since the last interrupt. Our counter 
      * apparently counts UP from zero, so we just need to read it.
      */
     elapsed = inw(DSPLINUX_TIMER0_CNT);

     /* 
      * Convert the elapsed count to usecs. 'tick' is assigned in sched.c.
      * Normally it's 10,000 -- the # of usecs/interrupt.
      */
     return (unsigned long)((elapsed * tick) / COUNTS_PER_INTERRUPT);

}

/*
 * No need to reset the timer at every irq, it's set to FREE_RUN
 */
#define reset_timer() 1

/*
 * Updating of the RTC.  We don't have one.
 */
#define update_rtc() {}

void __inline__ setup_timer (void)
{
	/* disable */
	outw(0, DSC21_TIMER0_MODE);
	outw(0, DSC21_TIMER1_MODE);
	outw(0, DSC21_TIMER2_MODE);
	outw(0, DSC21_TIMER3_MODE);

	/*
	 * System clock formula:
	 *         freq = clock / (div * scale)
	 * Where:
	 *         clock = 27MHz
	 *         div   = 27,000
	 *         scale = 10
	 *
	 * Which gives us 100Hz, as desired.
	 */

	/* 27MHz clock (not ARM clock) */
	outw(1, DSC21_TIMER0_SEL);
	
	/* prescale 10 */
	outw(9, DSC21_TIMER0_SCAL);
	
	/* div 27000 */
	outw(COUNTS_PER_INTERRUPT - 1, DSC21_TIMER0_DIV);
	
	/* periodic */
	outw(2, DSC21_TIMER0_MODE);

}

#endif /* CONFIG_ARCH_DSC2X */

#endif /* __ASM_ARCH_TIME_H__ */
