/*
 
  timer_ioctl.h
 
  This file represents the DSC21 timer peripheral.  

  Copyright (C) 2001, RidgeRun, Inc.
 
*/ 
#ifndef __TIMER_IOCTL_H__
#define __TIMER_IOCTL_H__

#include <asm/ioctl.h>
 
/* List of ioctl commands. */
 
#define TIMER_MAGIC_NUMBER 'T'
 
/*
  blocking call to wait for timer interrupt to fire.
*/
#define TIMER_WAIT _IO(TIMER_MAGIC_NUMBER, 0)

/*
  start one shot timer.
*/
#define TIMER_TRIGGER_ONE_SHOT _IO(TIMER_MAGIC_NUMBER, 1)

/*
  return current value of timer.
*/
#define TIMER_GET_VALUE _IOR(TIMER_MAGIC_NUMBER, 2, short int)

/*

  If you want total control of your timer, use this IOCTL to set it.

  timer set takes this parameter (timer_setup_t):
    mode (timer_mode_t): Stop, OneShot (run once), or FreeRun (run continually)
    timer clock selection (timer_clock_select_t): ARM clock or 27MHz clock.
    prescaler division ratio (u16).
    timer division ratio (u16).
*/

typedef enum {
     eStop,
     eOneShot,
     eFreeRun
} timer_mode_t;

#ifdef CONFIG_ARCH_DSC2X
typedef enum {
     eARMClock,
     e27MHzClock
} timer_clock_select_t;
#endif

typedef struct {
     timer_mode_t mode;
#ifdef CONFIG_ARCH_DSC2X
     timer_clock_select_t select;
#endif
     __u16 divisor;
     __u16 prescaler;  // Max value of 0x3ff(10 bits) on DSC2X or 0xff on Orion
} timer_setup_t;

#define TIMER_SET_ALL _IOW(TIMER_MAGIC_NUMBER, 3, timer_setup_t)

/*

  If you want a simple interface, send in the number of microseconds
  you want to occur before the timer interrupt fires.

*/
#define TIMER_SET_USEC _IOW(TIMER_MAGIC_NUMBER, 4, __u32)

#define TIMER_IOC_MAXNR 4
 
/* Major numbers, (from Documentation/devices.txt local/experimental list)  */
 
#define TIMER_MAJOR     124
 
#endif            // __TIMER_IOCTL_H__
