/*
 *  
 *    venc.h
 *     
 *    This file represents the DSC24 video encoder. 
 *
 *    Copyright (C) 2001, RidgeRun, Inc.
 *          
*/

#ifndef __VENC_OKI_H__
#define __VENC_OKI_H__
 
#include <asm/ioctl.h>

/* List of ioctl commands. */
 
#define VENC_OKI_MAGIC_NUMBER 'v'

/*
  Mode Register 0
*/

#define VENC_OKI_SET_MR0 _IO(VENC_OKI_MAGIC_NUMBER, 0)
 
/*
  Mode Register 1
*/

#define VENC_OKI_SET_MR1 _IO(VENC_OKI_MAGIC_NUMBER, 1)

/*
  Command Register 0
*/

#define VENC_OKI_SET_CR0 _IO(VENC_OKI_MAGIC_NUMBER, 2)

/*
  Command Register 1
*/

#define VENC_OKI_SET_CR1 _IO(VENC_OKI_MAGIC_NUMBER, 3)
 
/*
  All Registers
*/

#define VENC_OKI_SET_ALL _IO(VENC_OKI_MAGIC_NUMBER, 4)

#define VENC_OKI_IOC_MAXNR 4
 
/* Major numbers, (from Documentation/devices.txt local/experimental list)  */
 
#define VENC_OKI_MAJOR     126

// OKI Register Definitions

#define SLAVE_ADDR 0x88

#define MR0_ADDR 0x00
#define MR1_ADDR 0x01
#define CR0_ADDR 0x02
#define CR1_ADDR 0x03

// MR0 (Mode Register 0)

// MR0[7]
#define INTERNAL_REG_SET			0x80
#define EXTERNAL_REG_SET 			0x00

// MR0[6]
#define CHROMA_2S_COMP				0x40
#define CHROMA_OFFSET_BINARY                    0x00

// MR0[5]
#define DAC_SLEEP				0x20
#define DAC_ACTIVE                              0x00

// MR0[4]
#define RGB_MODE				0x10
#define YCC                                     0x00

// MR0[3:0]
#define	NTSC_ITU_R_BT656			0x0
#define	NTSC_27MHZ_YCBCR			0x1
#define	NTSC_24_52MHZ_SQ_PXL		        0x2
#define	NTSC_28_64MHZ_4FSC			0x3
#define	NTSC_13_5MHZ_YCBCR			0x5
#define	NTSC_12_27MHZ				0x6
#define	NTSC_14_32MHZ				0x7
#define PAL_ITU_R_BT656				0x8	
#define	PAL_27MHZ_YCBCR				0x9
#define	PAL_29_5MHZ_SQ_PXL			0xA
#define	PAL_13_5_MHZ				0xD
#define	PAL_14_75_MHZ				0xE

// MR1 (Mode Register 1)

// MR1[7]
#define BLACK_LEVEL_7_5IRE			0x80
#define BLACK_LEVEL_0_IRE                       0x00

// MR1[6]
#define COUNTER_CTL_BOTH			0x40
#define COUNTER_STD_ONLY                        0x00

// MR1[5]
#define RGB_INP_LEVEL_16			0x20
#define RGB_INP_LEVEL_0                         0x00

// MR1[4]
#define RGB_OUTSEL				0x10
#define S_VIDEO_OUTSEL                          0x00

// MR1[3]
#define MASTER					0x08
#define SLAVE                                   0x00

// MR1[2]
#define	INTERLACE				0x04
#define NONINTERLACE                            0x00

// MR1[1]
#define GENLOCK_OFF				0x02
#define GENLOCK_ON                              0x00

// MR[0]
#define GENLOCK_SEL				0x01


// CR0 (Command Register 0)

// CR0[6]
#define CSYNC_ADD				0x40

// CR0[5]
#define FOUT_L_ODD_H_EVEN			0x20

// CR0[4]
#define TRAP_FILTER_ON				0x10

// CR0[3]
#define LUM_COLOR_BAR				0x08

// CR0[2:1]
#define OVLY_25					0x06
#define OVLY_50					0x04
#define OVLY_75					0x02
#define OVLY_100				0x00

// CR0[0]
#define SAMP_411				0x01

// CR1 (Command Register 1)

// CR1[3:0]
#define	LUMN_100				0x00
#define	LUMN_097				0x01
#define	LUMN_094				0x02
#define	LUMN_091				0x03
#define	LUMN_088				0x04
#define	LUMN_084				0x05
#define	LUMN_081				0x06
#define	LUMN_078				0x07
#define	LUMN_075				0x08
#define	LUMN_072				0x09
#define	LUMN_069				0x0a

#endif            // __VENC_OKI_H__
